#############################################################################
##
#A  anupqeg.tst               ANUPQ package                       Greg Gamble
##
##  Tests all but one of the ANUPQ examples.
##  Execute this file with `Test( "anupqeg.tst" );'.
##  This is a *big* test, taking some 40 minutes on a *fast* (1GHz) machine.
##  The number of GAPstones returned at the end do not mean much as  they  do
##  not measure the time spent by the `pq' binary.
##

gap> START_TEST( "Testing ANUPQ examples" );
gap> SetInfoLevel(InfoANUPQ, 1);
gap> examples := AllPqExamples();
[ "11gp-3-Engel-Id", "11gp-3-Engel-Id-i", "11gp-PG-i", "11gp-Rel-i", 
  "11gp-SP-a-Rel-1-i", "11gp-SP-a-Rel-i", "11gp-SP-a-i", "11gp-SP-b-Rel-i", 
  "11gp-SP-b-i", "11gp-SP-c-Rel-i", "11gp-a-Rel-i", "11gp-i", "2gp-PG-2-i", 
  "2gp-PG-3-i", "2gp-PG-4-i", "2gp-PG-e4-i", "2gp-PG-i", "2gp-Rel", 
  "2gp-Rel-i", "2gp-SP-1-Rel-i", "2gp-SP-2-Rel-i", "2gp-SP-3-Rel-i", 
  "2gp-SP-4-Rel-i", "2gp-SP-Rel-i", "2gp-SP-d-Rel-i", "2gp-a-Rel-i", 
  "3gp-PG-4-i", "3gp-PG-i", "3gp-PG-x-1-i", "3gp-PG-x-i", "3gp-Rel-i", 
  "3gp-SP-1-Rel-i", "3gp-SP-2-Rel-i", "3gp-SP-3-Rel-i", "3gp-SP-4-Rel-i", 
  "3gp-SP-Rel-i", "3gp-a-Rel", "3gp-a-Rel-i", "3gp-a-x-Rel-i", 
  "3gp-maxoccur-Rel-i", "5gp-PG-i", "5gp-Rel-i", "5gp-SP-Rel-i", 
  "5gp-SP-a-Rel-i", "5gp-SP-b-Rel-i", "5gp-SP-big-Rel-i", "5gp-SP-d-Rel-i", 
  "5gp-a-Rel-i", "5gp-b-Rel-i", "5gp-c-Rel-i", "5gp-maxoccur-Rel-i", 
  "5gp-metabelian-Rel", "5gp-metabelian-Rel-i", "7gp-PG-i", "7gp-Rel-i", 
  "7gp-SP-Rel-i", "7gp-SP-a-Rel-i", "7gp-SP-b-Rel-i", "B2-4", "B2-4-Id", 
  "B2-4-SP-i", "B2-5", "B2-5-i", "B2-8-i", "B4-4-a-i", "B4-4-i", "B5-4.g", 
  "B5-5-Engel3-Id", "EpimorphismStandardPresentation", 
  "EpimorphismStandardPresentation-i", "F2-5-i", "G2-SP-Rel-i", 
  "G3-SP-Rel-i", "G5-SP-Rel-i", "G5-SP-a-Rel-i", "IsIsomorphicPGroup-ni", 
  "Nott-APG-Rel-i", "Nott-PG-Rel-i", "Nott-SP-Rel-i", "Pq", "Pq-ni", 
  "PqDescendants-1", "PqDescendants-1-i", "PqDescendants-2", 
  "PqDescendants-3", "PqDescendants-treetraverse-i", 
  "PqDescendantsTreeCoclassOne-16-i", "PqDescendantsTreeCoclassOne-25-i", 
  "PqDescendantsTreeCoclassOne-9-i", "PqEpimorphism", "PqPCover", 
  "PqSupplementInnerAutomorphisms", "R2-5-i", "R2-5-x-i", 
  "StandardPresentation", "StandardPresentation-i", "gp-256-SP-Rel-i" ]
gap> RemoveSet(examples, "EpimorphismStandardPresentation-i");
gap> nexamples := Length( examples );
96
gap> ##Example: "11gp-3-Engel-Id" . . . 3-Engel group for prime 11
gap> ##Non-trivial example of using the `Identities' option
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := F/[ a^11, b^11 ];
<fp group on the generators [ a, b ]>
gap> # All word pairs u, v in the pc generators of the 11-quotient Q of G 
gap> # must satisfy the Engel identity: [u, v, v, v] = 1.
gap> f := function(u, v) return PqLeftNormComm( [u, v, v, v] ); end;
function( u, v ) ... end
gap> Q := Pq( G : Prime := 11, Identities := [ f ] );
#I  Class 1 with 2 generators.
#I  Class 2 with 3 generators.
#I  Class 3 with 5 generators.
#I  Class 3 with 5 generators.
<pc group of size 161051 with 5 generators>
gap> # We do a ``sample'' check that pairs of elements of Q do satisfy
gap> # the given identity:
gap> f( Random(Q), Random(Q) );
<identity> of ...
gap> f( Q.1, Q.2 );
<identity> of ...
gap> # Executing interactive variant of example: "11gp-3-Engel-Id"
gap> ##Example: "11gp-3-Engel-Id" . . . 3-Engel group for prime 11
gap> ##Non-trivial example of using the `Identities' option
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := F/[ a^11, b^11 ];
<fp group on the generators [ a, b ]>
gap> # All word pairs u, v in the pc generators of the 11-quotient Q of G 
gap> # must satisfy the Engel identity: [u, v, v, v] = 1.
gap> f := function(u, v) return PqLeftNormComm( [u, v, v, v] ); end;
function( u, v ) ... end
gap> procId := PqStart( G );
1
gap> Q := Pq( procId : Prime := 11, Identities := [ f ] );
#I  Class 1 with 2 generators.
#I  Class 2 with 3 generators.
#I  Class 3 with 5 generators.
#I  Class 3 with 5 generators.
<pc group of size 161051 with 5 generators>
gap> # We do a ``sample'' check that pairs of elements of Q do satisfy
gap> # the given identity:
gap> f( Random(Q), Random(Q) );
<identity> of ...
gap> f( Q.1, Q.2 );
<identity> of ...
gap> ##Example: "11gp-3-Engel-Id-i" . . . 3-Engel grp for prime 11
gap> ##Variation of "11gp-3-Engel-Id" broken down into its lower-level component
gap> ##command parts.
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := F/[ a^11, b^11 ];
<fp group on the generators [ a, b ]>
gap> # All word pairs u, v in the pc generators of the 11-quotient Q of G 
gap> # must satisfy the Engel identity: [u, v, v, v] = 1.
gap> f := function(u, v) return PqLeftNormComm( [u, v, v, v] ); end;
function( u, v ) ... end
gap> procId := PqStart( G : Prime := 11 );
2
gap> PqPcPresentation( procId : ClassBound := 1);
gap> PqEvaluateIdentities( procId : Identities := [f] );
#I  Class 1 with 2 generators.
gap> for c in [2 .. 4] do
>   PqNextClass( procId : Identities := [] ); #reset `Identities' option
>   PqEvaluateIdentities( procId : Identities := [f] );
> od;
#I  Class 2 with 3 generators.
#I  Class 3 with 5 generators.
#I  Class 3 with 5 generators.
gap> Q := PqCurrentGroup( procId );
<pc group of size 161051 with 5 generators>
gap> # We do a ``sample'' check that pairs of elements of Q do satisfy
gap> # the given identity:
gap> f( Random(Q), Random(Q) );
<identity> of ...
gap> f( Q.1, Q.2 );
<identity> of ...
gap> ##Example: "11gp-PG-i" . . . based on: examples/pga_11gp
gap> ##Descendants of C11 x C11 x C11
gap> F := FreeGroup("a", "b", "c");
<free group on the generators [ a, b, c ]>
gap> procId := PqStart(F : Prime := 11);
3
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-11 central series for [grp]
#I  Group: [grp] to lower exponent-11 central class 1 has order 11^3
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-11 central class 2 has order 11^9
gap> PqPGSupplyAutomorphisms(procId, [ [[ 2, 0, 0],
>                                    [ 0, 1, 0],
>                                    [ 0, 0, 1]],
> 
>                                   [[10, 0, 1],
>                                    [10, 0, 0],
>                                    [ 0,10, 0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 2,
>                                   CapableDescendants,
>                                   StepSize := 1,
>                                   RankInitialSegmentSubgroups := 3);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 11^3
#I  Nuclear rank: 6
#I  11-multiplicator rank: 6
#I  # of immediate descendants of order 11^4 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
2
gap> PqQuitAll();
gap> ##Example: "11gp-Rel-i" . . . based on: examples/11gp
gap> ##(equivalent to "11gp-i" example but uses `Relators' option)
gap> F := FreeGroup("a", "b", "c");
<free group on the generators [ a, b, c ]>
gap> rels := ["[b, a, a, b, c]^11", "[a, b, b, a, b, c]^11", "(a * b)^11"];
[ "[b, a, a, b, c]^11", "[a, b, b, a, b, c]^11", "(a * b)^11" ]
gap> procId := PqStart(F : Prime := 11, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 7, 
>                           OutputLevel := 1);
#I  Lower exponent-11 central series for [grp]
#I  Group: [grp] to lower exponent-11 central class 1 has order 11^3
#I  Group: [grp] to lower exponent-11 central class 2 has order 11^8
#I  Group: [grp] to lower exponent-11 central class 3 has order 11^19
#I  Group: [grp] to lower exponent-11 central class 4 has order 11^42
#I  Group: [grp] to lower exponent-11 central class 5 has order 11^98
#I  Group: [grp] to lower exponent-11 central class 6 has order 11^228
#I  Group: [grp] to lower exponent-11 central class 7 has order 11^563
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "11gp-SP-a-Rel-1-i" . . . based on: isom/11gp_a.com
gap> ##(like "11gp-SP-a-Rel-i" but the initial input presentation
gap> ## is only to class 1).
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^11", "b^11*[b, a, a]^-2", "[b, a, b, b, b, b]"];
[ "a^11", "b^11*[b, a, a]^-2", "[b, a, b, b, b, b]" ]
gap> procId := PqStart(F : Prime := 11, Relators := rels);
2
gap> PqSetOutputLevel(procId, 0);
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[1,0],
>                                     [0,1]],
> 
>                                    [[1,0],
>                                     [0,1]],
> 
>                                    [[1,0],
>                                     [0,1]],
> 
>                                    [[1,0],
>                                     [3,1]],
> 
>                                    [[1,0],
>                                     [9,3]],
> 
>                                    [[1,0],
>                                     [6,6]],
> 
>                                    [[10,0],
>                                     [2,1]] ]
> 
>                          : # options
>                            ClassBound := 19,
>                            PcgsAutomorphisms);
gap> ##Example: "11gp-SP-a-Rel-i" . . . based on: isom/11gp_a.com
gap> ##(equivalent to "11gp-SP-a-i" but uses the `Relators' option)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^11", "b^11*[b, a, a]^-2", "[b, a, b, b, b, b]"];
[ "a^11", "b^11*[b, a, a]^-2", "[b, a, b, b, b, b]" ]
gap> procId := PqStart(F : Prime := 11, Relators := rels);
3
gap> PqSetOutputLevel(procId, 0);
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0,1],
>                                     [0,1,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,1,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,1]],
> 
>                                    [[1,0,0,0,0],
>                                     [3,1,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [9,3,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [6,6,0,0,0]],
> 
>                                    [[10,0,0,0,0],
>                                     [2,1,0,0,0]] ]
> 
>                          : # options
>                            ClassBound := 19,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "11gp-SP-a-i" . . . based on: isom/11gp_a.com
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> R := [a^11, b^11/PqLeftNormComm([b, a, a])^2, 
>       PqLeftNormComm([b, a, b, b, b, b])];;
gap> procId := PqStart(F/R : Prime := 11);
1
gap> PqSetOutputLevel(procId, 0);
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0,1],
>                                     [0,1,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,1,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,1]],
> 
>                                    [[1,0,0,0,0],
>                                     [3,1,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [9,3,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [6,6,0,0,0]],
> 
>                                    [[10,0,0,0,0],
>                                     [2,1,0,0,0]] ]
> 
>                          : # options
>                            ClassBound := 19,
>                            PcgsAutomorphisms);
gap> ##Example: "11gp-SP-b-Rel-i" . . . based on: isom/11gp_b.com
gap> ##(equivalent to "11gp-SP-b-i" but uses the `Relators' option)
gap> F := FreeGroup("a", "b", "c");
<free group on the generators [ a, b, c ]>
gap> rels := ["a^11", "b^11", "c^11", "[b, a, a, a, b, a]", 
>          "[c, a]", "[c, b]", "[b, a, b]"];
[ "a^11", "b^11", "c^11", "[b, a, a, a, b, a]", "[c, a]", "[c, b]", 
  "[b, a, b]" ]
gap> procId := PqStart(F : Prime := 11, Relators := rels);
2
gap> PqSetOutputLevel(procId, 0);
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0,0],
>                                     [0,1,0,0,1],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,1]],
> 
>                                    [[1,0,9,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,7,8,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[10,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[2,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,8,0,0],
>                                     [0,1,3,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,9,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,3,0,0]],
> 
>                                    [[1,0,2,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,10,0,0]],
> 
>                                    [[1,9,10,0,0],
>                                     [0,3,7,0,0],
>                                     [0,0,6,0,0]],
> 
>                                    [[1,5,9,0,0],
>                                     [0,7,4,0,0],
>                                     [0,0,10,0,0]]]
> 
>                          : # options
>                            ClassBound := 8,
>                            PcgsAutomorphisms);
gap> ##Example: "11gp-SP-b-i" . . . based on: isom/11gp_b.com
gap> F := FreeGroup("a", "b", "c"); a := F.1; b := F.2; c := F.3;
<free group on the generators [ a, b, c ]>
a
b
c
gap> R := [a^11, b^11, c^11, PqLeftNormComm([b, a, a, a, b, a]),
>       Comm(c, a), Comm(c, b), PqLeftNormComm([b, a, b])];;
gap> procId := PqStart(F/R : Prime := 11);
3
gap> PqSetOutputLevel(procId, 0);
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0,0],
>                                     [0,1,0,0,1],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,1]],
> 
>                                    [[1,0,9,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,7,8,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[10,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[2,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,8,0,0],
>                                     [0,1,3,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,9,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,3,0,0]],
> 
>                                    [[1,0,2,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,10,0,0]],
> 
>                                    [[1,9,10,0,0],
>                                     [0,3,7,0,0],
>                                     [0,0,6,0,0]],
> 
>                                    [[1,5,9,0,0],
>                                     [0,7,4,0,0],
>                                     [0,0,10,0,0]]]
> 
>                          : # options
>                            ClassBound := 8,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "11gp-SP-c-Rel-i" . . . based on: isom/11gp_c.com
gap> F := FreeGroup("a", "b", "c");
<free group on the generators [ a, b, c ]>
gap> rels := ["a^11", "b^11", "c^11", "[b, a, a, a, b]", 
>          "[c, a]", "[c, b]", "[b, a, b]"];
[ "a^11", "b^11", "c^11", "[b, a, a, a, b]", "[c, a]", "[c, b]", "[b, a, b]" ]
gap> procId := PqStart(F : Prime := 11, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0,0],
>                                     [0,1,0,0,1],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,1]],
> 
>                                    [[1,0,9,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,7,8,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[10,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[2,0,0,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,8,0,0],
>                                     [0,1,3,0,0],
>                                     [0,0,1,0,0]],
> 
>                                    [[1,0,9,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,3,0,0]],
> 
>                                    [[1,0,2,0,0],
>                                     [0,1,0,0,0],
>                                     [0,0,10,0,0]],
> 
>                                    [[1,9,10,0,0],
>                                     [0,3,7,0,0],
>                                     [0,0,6,0,0]],
> 
>                                    [[1,5,9,0,0],
>                                     [0,7,4,0,0],
>                                     [0,0,10,0,0]]]
> 
>                          : # options
>                            ClassBound := 8,#for 9 perm.deg.>2^31, pq dies
>                            PcgsAutomorphisms);
gap> ##Example: "11gp-a-Rel-i" . . . based on: examples/11gpA
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["[b, a, a, b, b]^11", "[a, b, b, a, b, b]^11", "(a * b)^11"];
[ "[b, a, a, b, b]^11", "[a, b, b, a, b, b]^11", "(a * b)^11" ]
gap> procId := PqStart(F : Prime := 11, Relators := rels);
2
gap> PqPcPresentation(procId : ClassBound := 8, 
>                           OutputLevel := 1);
#I  Lower exponent-11 central series for [grp]
#I  Group: [grp] to lower exponent-11 central class 1 has order 11^2
#I  Group: [grp] to lower exponent-11 central class 2 has order 11^4
#I  Group: [grp] to lower exponent-11 central class 3 has order 11^7
#I  Group: [grp] to lower exponent-11 central class 4 has order 11^11
#I  Group: [grp] to lower exponent-11 central class 5 has order 11^18
#I  Group: [grp] to lower exponent-11 central class 6 has order 11^28
#I  Group: [grp] to lower exponent-11 central class 7 has order 11^47
#I  Group: [grp] to lower exponent-11 central class 8 has order 11^78
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "11gp-i" . . . based on: examples/11gp
gap> F := FreeGroup("a", "b", "c"); a := F.1; b := F.2; c := F.3;
<free group on the generators [ a, b, c ]>
a
b
c
gap> R := [PqLeftNormComm([b, a, a, b, c])^11, 
>       PqLeftNormComm([a, b, b, a, b, c])^11, (a * b)^11];;
gap> procId := PqStart(F/R : Prime := 11);
3
gap> PqPcPresentation(procId : ClassBound := 7, 
>                           OutputLevel := 1);
#I  Lower exponent-11 central series for [grp]
#I  Group: [grp] to lower exponent-11 central class 1 has order 11^3
#I  Group: [grp] to lower exponent-11 central class 2 has order 11^8
#I  Group: [grp] to lower exponent-11 central class 3 has order 11^19
#I  Group: [grp] to lower exponent-11 central class 4 has order 11^42
#I  Group: [grp] to lower exponent-11 central class 5 has order 11^98
#I  Group: [grp] to lower exponent-11 central class 6 has order 11^228
#I  Group: [grp] to lower exponent-11 central class 7 has order 11^563
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "2gp-PG-2-i" . . . based on: examples/pga_example
gap> ##All class 3 descendants of C2 x C2 with extensive output
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 2);
1
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
gap> PqPGSupplyAutomorphisms(procId, [ [[0,1],
>                                    [1,1]],
> 
>                                   [[0,1],
>                                    [1,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 3,
>                                   PcgsAutomorphisms,
>                                   CustomiseOutput := rec(group := [,,1],
>                                                          autgroup := [,1]));
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 2^2
#I  Nuclear rank: 3
#I  2-multiplicator rank: 3
#I  Group: [grp] #1;1 to lower exponent-2 central class 2 has order 2^3
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  Non-trivial powers:
#I   .1^2 = .3
#I  Non-trivial commutators:
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 
#I  Generator  2 --> 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 
#I  Generator  2 --> 0 1 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 1 0 
#I  Generator  2 --> 0 1 1 
#I  Group: [grp] #2;1 to lower exponent-2 central class 2 has order 2^3
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  Non-trivial powers:
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 0 1 0 
#I  Generator  2 --> 1 0 0 
#I  Group: [grp] #3;1 to lower exponent-2 central class 2 has order 2^3
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .2^2 = .3
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 0 1 0 
#I  Generator  2 --> 1 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 0 1 0 
#I  Generator  2 --> 1 0 0 
#I  # of immediate descendants of order 2^3 is 3
#I  # of capable immediate descendants is 2
#I  Group: [grp] #4;2 to lower exponent-2 central class 2 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  4 is defined on 2^2 = 2 2 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .2^2 = .4
#I  Non-trivial commutators:
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 1 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 0 1 0 0 
#I  Generator  2 --> 1 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 0 1 0 0 
#I  Generator  2 --> 1 0 0 0 
#I  Group: [grp] #5;2 to lower exponent-2 central class 2 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 1 1 0 
#I  Generator  2 --> 0 1 1 1 
#I  Group: [grp] #6;2 to lower exponent-2 central class 2 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 1 1 0 
#I  Generator  2 --> 0 1 1 1 
#I  # of immediate descendants of order 2^4 is 3
#I  # of capable immediate descendants is 3
#I  Group: [grp] #7;3 to lower exponent-2 central class 2 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 0 1 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 0 1 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 
#I  # of immediate descendants of order 2^5 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 2^3
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group: [grp] #1;1 #1;1 to lower exponent-2 central class 3 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  Class 3
#I  4 is defined on 3^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .3^2 = .4
#I  Non-trivial commutators:
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Group: [grp] #1;1 #2;1 to lower exponent-2 central class 3 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  Class 3
#I  4 is defined on 3^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .3^2 = .4
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 
#I  Generator  2 --> 0 1 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 0 0 
#I  Generator  2 --> 0 1 0 0 
#I  # of immediate descendants of order 2^4 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1
#I  Order: 2^3
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group: [grp] #2;1 #1;1 to lower exponent-2 central class 3 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  Class 3
#I  4 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .3^2 = .4
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .3, .2 ] = .4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 0 1 0 0 
#I  Generator  2 --> 1 0 0 0 
#I  Group: [grp] #2;1 #2;1 to lower exponent-2 central class 3 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  Class 3
#I  4 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .4
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .3, .2 ] = .4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 0
#I  Group: [grp] #2;1 #3;1 to lower exponent-2 central class 3 has order 2^4
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  Class 3
#I  4 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .4
#I   .3^2 = .4
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .3, .2 ] = .4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 
#I  Generator  2 --> 0 1 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 0 1 0 0 
#I  Generator  2 --> 1 0 0 0 
#I  # of immediate descendants of order 2^4 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #3;1
#I  Order: 2^3
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2
#I  Order: 2^4
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group: [grp] #4;2 #1;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  4 is defined on 2^2 = 2 2 
#I  Class 3
#I  5 is defined on 3^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .2^2 = .4
#I   .3^2 = .5
#I  Non-trivial commutators:
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 0 0 
#I  Group: [grp] #4;2 #2;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  4 is defined on 2^2 = 2 2 
#I  Class 3
#I  5 is defined on 3^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .2^2 = .4
#I   .3^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 0 0 
#I  # of immediate descendants of order 2^5 is 2
#I  # of capable immediate descendants is 2
#I  Group: [grp] #4;2 #3;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  4 is defined on 2^2 = 2 2 
#I  Class 3
#I  5 is defined on 3^2 = 1 1 1 
#I  6 is defined on 4^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .2^2 = .4
#I   .3^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 
#I  Group: [grp] #4;2 #4;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on 1^2 = 1 1 
#I  4 is defined on 2^2 = 2 2 
#I  Class 3
#I  5 is defined on 3^2 = 1 1 1 
#I  6 is defined on 4^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .3
#I   .2^2 = .4
#I   .3^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2
#I  Order: 2^4
#I  Nuclear rank: 3
#I  2-multiplicator rank: 4
#I  Group: [grp] #5;2 #1;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Group: [grp] #5;2 #2;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .4, .2 ] = .5
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Group: [grp] #5;2 #3;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .4, .2 ] = .5
#I  Number of stabiliser generators is 3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 1 1 0 
#I  Group: [grp] #5;2 #4;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .4, .2 ] = .5
#I  Number of stabiliser generators is 3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 1 1 0 
#I  Group: [grp] #5;2 #5;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Group: [grp] #5;2 #6;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Group: [grp] #5;2 #7;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .5
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  # of immediate descendants of order 2^5 is 7
#I  # of capable immediate descendants is 3
#I  Group: [grp] #5;2 #8;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .4, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 1 
#I  Group: [grp] #5;2 #9;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .4, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 1 
#I  Group: [grp] #5;2 #10;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Group: [grp] #5;2 #11;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Group: [grp] #5;2 #12;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Group: [grp] #5;2 #13;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Group: [grp] #5;2 #14;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .6
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Group: [grp] #5;2 #15;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #5;2 #16;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .6
#I   .4^2 = .5 .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #5;2 #17;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .5 .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Group: [grp] #5;2 #18;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .6
#I   .3^2 = .6
#I   .4^2 = .5 .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  # of immediate descendants of order 2^6 is 11
#I  # of capable immediate descendants is 10
#I  Group: [grp] #5;2 #19;3 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .3^2 = .6
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 1 
#I  Group: [grp] #5;2 #20;3 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 1
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Group: [grp] #5;2 #21;3 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on [3, 2] = 2 1 2 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .6
#I   .3^2 = .6
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  [ .3, .2 ] = .6
#I  [ .4, .2 ] = .5 .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 1 
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #6;2
#I  Order: 2^4
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group: [grp] #6;2 #1;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 1 0 1 
#I  Group: [grp] #6;2 #2;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3
#I   .3^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 1 1 0 
#I  Group: [grp] #6;2 #3;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3 .5
#I   .3^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 1 1 0 
#I  Group: [grp] #6;2 #4;1 to lower exponent-2 central class 3 has order 2^5
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3
#I   .3^2 = .5
#I   .4^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 
#I  Generator  2 --> 0 1 1 0 0 
#I  # of immediate descendants of order 2^5 is 4
#I  # of capable immediate descendants is 3
#I  Group: [grp] #6;2 #5;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3
#I   .3^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 0 1 1 
#I  Group: [grp] #6;2 #6;2 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  Class 3
#I  5 is defined on [3, 1] = 2 1 1 
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .3 .5
#I   .3^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 2
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 0 1 1 
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3
#I  Order: 2^5
#I  Nuclear rank: 5
#I  2-multiplicator rank: 5
#I  Group: [grp] #7;3 #1;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #7;3 #2;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .6
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #3;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .6
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .6
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #4;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Group: [grp] #7;3 #5;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #7;3 #6;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .6
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #7;3 #7;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #7;3 #8;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 
#I  Group: [grp] #7;3 #9;1 to lower exponent-2 central class 3 has order 2^6
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 
#I  # of immediate descendants of order 2^6 is 9
#I  # of capable immediate descendants is 3
#I  Group: [grp] #7;3 #10;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on 4^2 = 1 1 1 
#I  7 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #11;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [4, 2] = 1 1 2 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 1 0 
#I  Group: [grp] #7;3 #12;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [4, 2] = 1 1 2 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 1 0 
#I  Group: [grp] #7;3 #13;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #14;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #15;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #16;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #17;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #18;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #19;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .7
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #20;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #21;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #22;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #23;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #24;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .6
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #25;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #26;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #27;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .6
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #28;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .7
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #29;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #30;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .6 .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 
#I  Group: [grp] #7;3 #31;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #32;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #33;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #34;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #35;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6 .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #36;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .6
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #37;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #38;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #39;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #40;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #41;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .6
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #42;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .6
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #43;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #44;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #45;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  Group: [grp] #7;3 #46;2 to lower exponent-2 central class 3 has order 2^7
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 
#I  # of immediate descendants of order 2^7 is 37
#I  # of capable immediate descendants is 28
#I  Group: [grp] #7;3 #47;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [4, 2] = 1 1 2 
#I  7 is defined on 4^2 = 1 1 1 
#I  8 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #48;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  8 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .5, .1 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #49;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on 4^2 = 1 1 1 
#I  8 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .7
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #50;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #51;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #52;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #53;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 
#I  Group: [grp] #7;3 #54;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #55;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .6 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #56;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .7 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #57;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .6 .7 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #58;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 1 0 0 
#I  Group: [grp] #7;3 #59;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .8
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #60;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .8
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 1 0 0 
#I  Group: [grp] #7;3 #61;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #62;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #63;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #64;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #65;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #66;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #67;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6 .7
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #68;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #69;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #70;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6 .7
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #71;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .8
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #72;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6 .8
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #73;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6 .8
#I   .5^2 = .6 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #74;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .7 .8
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #75;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .6 .7 .8
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #76;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .8
#I   .5^2 = .6 .7 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #77;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #78;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .6
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #79;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .7
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #80;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #81;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .8
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #82;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .8
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  Group: [grp] #7;3 #83;3 to lower exponent-2 central class 3 has order 2^8
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .8
#I   .5^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 
#I  # of immediate descendants of order 2^8 is 37
#I  # of capable immediate descendants is 37
#I  Group: [grp] #7;3 #84;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [4, 2] = 1 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  9 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .4^2 = .8
#I   .5^2 = .9
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 0 0 
#I  Group: [grp] #7;3 #85;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  9 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .4^2 = .8
#I   .5^2 = .9
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .6
#I  [ .5, .1 ] = .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 0 
#I  Group: [grp] #7;3 #86;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 1 0 
#I  Group: [grp] #7;3 #87;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I   .5^2 = .6
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Group: [grp] #7;3 #88;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I   .5^2 = .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 1 0 
#I  Group: [grp] #7;3 #89;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I   .5^2 = .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 4
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Group: [grp] #7;3 #90;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I   .5^2 = .6 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 1 0 
#I  Group: [grp] #7;3 #91;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I   .5^2 = .6 .7 .8
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 1 1 0 0 0 1 0 
#I  Group: [grp] #7;3 #92;4 to lower exponent-2 central class 3 has order 2^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on 4^2 = 1 1 1 
#I  9 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6
#I   .4^2 = .8
#I   .5^2 = .9
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 0 
#I  Automorphism 6:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 5
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 1 0 0 0 0 0 0 
#I  # of immediate descendants of order 2^9 is 9
#I  # of capable immediate descendants is 9
#I  Group: [grp] #7;3 #93;5 to lower exponent-2 central class 3 has order 2^10
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  4 is defined on 1^2 = 1 1 
#I  5 is defined on 2^2 = 2 2 
#I  Class 3
#I  6 is defined on [3, 1] = 2 1 1 
#I  7 is defined on [3, 2] = 2 1 2 
#I  8 is defined on [4, 2] = 1 1 2 
#I  9 is defined on 4^2 = 1 1 1 
#I  10 is defined on 5^2 = 2 2 2 
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .8
#I   .4^2 = .9
#I   .5^2 = .10
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .3, .2 ] = .7
#I  [ .4, .2 ] = .8
#I  [ .5, .1 ] = .6 .7 .8
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 1 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 1 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 0 
#I  Automorphism 7:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 1 
#I  Number of stabiliser generators is 6
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 0 0 0 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 1 0 0 0 0 0 
#I  Automorphism 5:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 1 0 0 0 0 0 0 0 0 
#I  Automorphism 6:
#I  Generator  1 --> 0 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 0 0 0 0 0 
#I  # of immediate descendants of order 2^10 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
136
gap> ##Example: "2gp-PG-3-i" . . . based on: examples/pga_3-2.com
gap> ##All descendants of C2 x C2 x C2
gap> F := FreeGroup("a", "b", "c");
<free group on the generators [ a, b, c ]>
gap> procId := PqStart(F : Prime := 2);
2
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^3
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^9
gap> PqPGSupplyAutomorphisms(procId, [ [[1,1,0],
>                                    [0,1,0],
>                                    [0,0,1]],
> 
>                                   [[0,0,1],
>                                    [1,0,0],
>                                    [0,1,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 5,
>                                   OrderBound := 7,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 2^3
#I  Nuclear rank: 6
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^4 is 4
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^5 is 15
#I  # of capable immediate descendants is 13
#I  # of immediate descendants of order 2^6 is 28
#I  # of capable immediate descendants is 28
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 15
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 2^4
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^5 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1
#I  Order: 2^4
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^5 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2
#I  Order: 2^5
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 4
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^7 is 9
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #6;2
#I  Order: 2^5
#I  Nuclear rank: 3
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^6 is 16
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^7 is 106
#I  # of capable immediate descendants is 54
#I  **************************************************
#I  Starting group: [grp] #7;2
#I  Order: 2^5
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 9
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^7 is 13
#I  # of capable immediate descendants is 9
#I  **************************************************
#I  Starting group: [grp] #8;2
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #9;2
#I  Order: 2^5
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 11
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^7 is 20
#I  # of capable immediate descendants is 10
#I  **************************************************
#I  Starting group: [grp] #10;2
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #11;2
#I  Order: 2^5
#I  Nuclear rank: 3
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^6 is 12
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^7 is 70
#I  # of capable immediate descendants is 25
#I  **************************************************
#I  Starting group: [grp] #12;2
#I  Order: 2^5
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 15
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^7 is 40
#I  # of capable immediate descendants is 13
#I  **************************************************
#I  Starting group: [grp] #13;2
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #14;2
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #15;2
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #18;2
#I  Order: 2^5
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^7 is 19
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #19;2
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #20;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #21;3
#I  Order: 2^6
#I  Nuclear rank: 5
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^7 is 17
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #22;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #23;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 18
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #24;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 13
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #25;3
#I  Order: 2^6
#I  Nuclear rank: 6
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 18
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #26;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 20
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #27;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 14
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #28;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #29;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #30;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #31;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 19
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #32;3
#I  Order: 2^6
#I  Nuclear rank: 5
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^7 is 47
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #33;3
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 7
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #34;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 16
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #35;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #36;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 30
#I  # of capable immediate descendants is 8
#I  **************************************************
#I  Starting group: [grp] #37;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #38;3
#I  Order: 2^6
#I  Nuclear rank: 5
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^7 is 24
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #39;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #40;3
#I  Order: 2^6
#I  Nuclear rank: 4
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 24
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #41;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #42;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #43;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 12
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #44;3
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #45;3
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #46;3
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 5
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #47;3
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 1
#I  **************************************************
#I  Starting group: [grp] #48;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #48;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #49;4
#I  Order: 2^7
#I  Nuclear rank: 8
#I  2-multiplicator rank: 10
#I  Group [grp] #49;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #50;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #50;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #51;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #51;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #52;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #52;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #53;4
#I  Order: 2^7
#I  Nuclear rank: 7
#I  2-multiplicator rank: 9
#I  Group [grp] #53;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #54;4
#I  Order: 2^7
#I  Nuclear rank: 8
#I  2-multiplicator rank: 10
#I  Group [grp] #54;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #55;4
#I  Order: 2^7
#I  Nuclear rank: 7
#I  2-multiplicator rank: 9
#I  Group [grp] #55;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #56;4
#I  Order: 2^7
#I  Nuclear rank: 7
#I  2-multiplicator rank: 9
#I  Group [grp] #56;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #57;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #57;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #58;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #58;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #59;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #59;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #60;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #60;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #61;4
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 8
#I  Group [grp] #61;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #62;4
#I  Order: 2^7
#I  Nuclear rank: 7
#I  2-multiplicator rank: 9
#I  Group [grp] #62;4 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #5;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #5;2 #5;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #6;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #5;2 #6;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #7;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #7;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #8;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #9;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #10;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 7
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #4;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 18
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #6;2 #9;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 13
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #17;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #17;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #18;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #19;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #19;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #20;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #20;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #21;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #21;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #22;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #22;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #23;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #23;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #24;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #25;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #25;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #26;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #26;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #27;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #27;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #28;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #28;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #29;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #29;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #32;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #32;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #33;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #33;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #35;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #35;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #43;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #43;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #46;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #46;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #51;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #51;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #52;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #52;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #53;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #53;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #54;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #54;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #58;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #58;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #59;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #59;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #60;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #60;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #64;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #64;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #68;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #68;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #71;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #71;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #75;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #75;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #76;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #76;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #77;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #77;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #78;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #78;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #79;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #79;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #80;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #80;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #81;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #81;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #87;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #87;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #91;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #91;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #93;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #93;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #99;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #99;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #100;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #100;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #102;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #102;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #103;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #103;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #105;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #105;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #106;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #106;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #108;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #108;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #109;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #109;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #111;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #111;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #112;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #112;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #114;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #114;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #115;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #115;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #117;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #117;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #118;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #118;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #120;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #120;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #121;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #6;2 #121;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 5
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;2 #4;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #7;2 #5;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;2 #10;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #7;2 #10;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #11;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #7;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #12;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #13;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #14;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #15;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #16;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #18;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #19;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;2 #19;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #8;2 #2;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #9;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 5
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #9;2 #3;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #9;2 #8;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #9;2 #12;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #9;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #13;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #14;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #15;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #16;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #17;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #9;2 #17;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #18;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #23;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #23;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #24;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #29;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #9;2 #29;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #10;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #11;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #11;2 #11;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  # of immediate descendants of order 2^7 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #11;2 #13;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #11;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #14;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #15;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #16;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #11;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #17;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #17;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #18;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #19;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #19;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #20;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #20;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #21;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #21;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #22;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #22;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #23;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #23;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #24;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #25;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #25;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #28;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #28;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #31;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #31;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #34;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #34;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #41;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #11;2 #41;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #43;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #43;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #47;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #47;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #49;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #49;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #51;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #51;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #61;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #61;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #63;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #63;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #69;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #69;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #71;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #11;2 #71;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 9
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #12;2 #8;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 9
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #12;2 #16;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #18;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #12;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #19;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #19;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #20;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #20;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #24;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #25;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #25;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #26;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #26;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #28;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #28;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #32;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #32;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #33;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #33;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #36;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #36;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #38;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #38;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #40;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #12;2 #40;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #13;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #13;2 #3;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #14;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #14;2 #2;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #15;2 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #18;2 #2;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #18;2 #8;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #18;2 #8;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #11;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #18;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #14;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #18;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #15;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #18;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #23;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #18;2 #23;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #24;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #18;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #19;2 #3;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #20;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #20;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #20;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #20;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #20;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #20;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #21;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #21;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #21;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #21;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #21;3 #11;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #21;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #22;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #22;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #22;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #22;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #22;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #22;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #13;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #23;3 #15;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #23;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #9;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #10;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #24;3 #11;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #24;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #25;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #25;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #25;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #25;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #25;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #25;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #25;3 #8;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #25;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #25;3 #12;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #25;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #26;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #26;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #26;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;3 #8;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #26;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;3 #11;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #26;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;3 #12;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #26;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #27;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #27;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #27;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #27;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #27;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #27;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #27;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #27;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #27;3 #8;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #27;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #28;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #28;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #28;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #28;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #28;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #28;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #28;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #28;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #29;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #29;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #30;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #30;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #30;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #30;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #30;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #30;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #30;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #30;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #30;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #30;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #31;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #31;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #31;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #31;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #31;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #31;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #31;3 #10;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #31;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #31;3 #15;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #31;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #31;3 #17;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #31;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #32;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #32;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #32;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #32;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #32;3 #11;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #32;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #32;3 #21;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #32;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #32;3 #25;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #32;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #32;3 #30;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #32;3 #30;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #33;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #33;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #33;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #33;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #33;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #33;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #33;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #33;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #33;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #33;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #33;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #33;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #34;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #34;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #34;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #34;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #34;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #34;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #34;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #34;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #34;3 #12;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #34;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #34;3 #14;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #34;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #35;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #35;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #35;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #35;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #35;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #35;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #35;3 #9;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #35;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #35;3 #10;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #35;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #8;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #11;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #17;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #19;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #36;3 #29;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #36;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #10;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #37;3 #13;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #37;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #38;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #38;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #38;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #38;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #38;3 #13;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #38;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #39;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #39;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #39;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #39;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #40;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #40;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #40;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #40;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #40;3 #12;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #40;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #40;3 #15;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #40;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #40;3 #16;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #40;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #41;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #41;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #41;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #42;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #42;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #42;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #42;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #43;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #43;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #43;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #43;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #43;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #43;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #43;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #43;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #43;3 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #43;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #44;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #44;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #44;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #44;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #44;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #44;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #44;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #44;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #44;3 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #44;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #44;3 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #44;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #45;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #45;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #45;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #45;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #45;3 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #45;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #46;3 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #46;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #46;3 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #46;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #46;3 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #46;3 #3;1 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #5;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #5;2 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #4;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #4;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #4;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #4;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #4;1 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #4;1 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #4;1 #13;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #4;1 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #9;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #6;2 #9;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #5;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;2 #5;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #5;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;2 #5;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #8;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #1;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #8;2 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #9;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #3;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #9;2 #3;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #3;1 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #9;2 #3;1 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #10;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #10;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #11;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #11;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #11;2 #11;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #12;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #8;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #12;2 #8;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #13;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #13;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #13;2 #1;1 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #13;2 #1;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #14;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #14;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #14;2 #1;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #14;2 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #15;2 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #15;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #2;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #18;2 #2;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #19;2 #3;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #19;2 #3;1 #1;1 is an invalid starting group
#I  **************************************************
365
gap> ##Example: "2gp-PG-4-i" . . . based on: examples/pga_4-2.com
gap> ##All descendants of C2 x C2 x C2 x C2
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> procId := PqStart(F : Prime := 2);
3
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^4
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^14
gap> PqPGSupplyAutomorphisms(procId, [ [[1,1,0,0],
>                                    [0,1,0,0],
>                                    [0,0,1,0],
>                                    [0,0,0,1]],
> 
>                                   [[0,0,0,1],
>                                    [1,0,0,0],
>                                    [0,1,0,0],
>                                    [0,0,1,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 4,
>                                   OrderBound := 8,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 2^4
#I  Nuclear rank: 10
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^5 is 6
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^6 is 54
#I  # of capable immediate descendants is 29
#I  # of immediate descendants of order 2^7 is 604
#I  # of capable immediate descendants is 439
#I  # of immediate descendants of order 2^8 is 3566
#I  # of capable immediate descendants is 3458
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^6 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^6 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 7
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 11
#I  **************************************************
#I  Starting group: [grp] #8;2
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^7 is 26
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^8 is 440
#I  # of capable immediate descendants is 96
#I  **************************************************
#I  Starting group: [grp] #9;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^8 is 59
#I  # of capable immediate descendants is 16
#I  **************************************************
#I  Starting group: [grp] #10;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 9
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #11;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 32
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^8 is 252
#I  # of capable immediate descendants is 35
#I  **************************************************
#I  Starting group: [grp] #12;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #13;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #14;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #15;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #16;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #17;2
#I  Order: 2^6
#I  Nuclear rank: 3
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^7 is 33
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^8 is 1004
#I  # of capable immediate descendants is 107
#I  **************************************************
#I  Starting group: [grp] #18;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 41
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^8 is 494
#I  # of capable immediate descendants is 37
#I  **************************************************
#I  Starting group: [grp] #19;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #20;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #21;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 28
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #22;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 15
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #26;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 14
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^8 is 163
#I  # of capable immediate descendants is 14
#I  **************************************************
#I  Starting group: [grp] #27;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #28;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #30;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #31;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 21
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #32;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #33;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #41;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 30
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^8 is 570
#I  # of capable immediate descendants is 24
#I  **************************************************
#I  Starting group: [grp] #42;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 40
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #43;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 24
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #44;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #45;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 13
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #46;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^7 is 13
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #61;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #62;3
#I  Order: 2^7
#I  Nuclear rank: 5
#I  2-multiplicator rank: 12
#I  # of immediate descendants of order 2^8 is 29
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #63;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #64;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 52
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #65;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #66;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #67;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #68;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #69;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #70;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #71;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #72;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #73;3
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 13
#I  # of immediate descendants of order 2^8 is 47
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #74;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 56
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #75;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 34
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #76;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 36
#I  # of capable immediate descendants is 8
#I  **************************************************
#I  Starting group: [grp] #77;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #78;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #79;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #80;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #81;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #82;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 61
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #83;3
#I  Order: 2^7
#I  Nuclear rank: 5
#I  2-multiplicator rank: 12
#I  # of immediate descendants of order 2^8 is 125
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #84;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 26
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #85;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 57
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #86;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #87;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 44
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #88;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 79
#I  # of capable immediate descendants is 8
#I  **************************************************
#I  Starting group: [grp] #89;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 34
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #90;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 41
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #91;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #92;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #93;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 57
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #94;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #95;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 47
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #96;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 75
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #97;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #98;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #99;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 34
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #100;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #101;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #102;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 31
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #103;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 27
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #104;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 48
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #105;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #106;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 29
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #107;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 55
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #108;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #109;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #110;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #111;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 21
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #112;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 21
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #113;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #114;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #115;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #116;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #117;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #118;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #119;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #120;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #121;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #122;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 30
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #123;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #124;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 27
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #125;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #126;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #127;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #128;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #129;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #130;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #131;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #132;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #133;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #134;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #135;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 39
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #136;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #137;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #138;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 37
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #139;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #140;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 26
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #141;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #142;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #143;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #144;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #145;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #146;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #147;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #148;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 37
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #149;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #150;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 26
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #151;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #152;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #153;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #154;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 58
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #155;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 52
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #156;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #157;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 65
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #158;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #159;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #160;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #161;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #162;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 31
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #163;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #164;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #165;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #166;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #167;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #168;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 9
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #169;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #170;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #171;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #172;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #173;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #174;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #175;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 9
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #176;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #177;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #178;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #179;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #180;3
#I  Order: 2^7
#I  Nuclear rank: 5
#I  2-multiplicator rank: 12
#I  # of immediate descendants of order 2^8 is 64
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #181;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 28
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #182;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 73
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #183;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #184;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #185;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #186;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 30
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #187;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 23
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #188;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #189;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #190;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 13
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #191;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #192;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #193;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 21
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #194;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #195;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 11
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #196;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #199;3
#I  Order: 2^7
#I  Nuclear rank: 5
#I  2-multiplicator rank: 12
#I  # of immediate descendants of order 2^8 is 82
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #200;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #201;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 58
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #202;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 56
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #203;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 92
#I  # of capable immediate descendants is 7
#I  **************************************************
#I  Starting group: [grp] #204;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 90
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #205;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 56
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #206;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 122
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #207;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #208;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #209;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #210;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #211;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #212;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 85
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #213;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 50
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #214;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #215;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #216;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 25
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #217;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #218;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #219;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #220;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 124
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #221;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #222;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #223;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #224;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 85
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #225;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #226;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #227;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 78
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #228;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #229;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 94
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #230;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 44
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #231;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 47
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #232;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 48
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #233;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #234;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #235;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #236;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #237;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #238;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #239;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #240;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #241;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #242;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #243;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #244;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #245;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #246;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #247;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #248;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #249;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #250;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #251;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #252;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #253;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #254;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #255;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #256;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #257;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #258;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #259;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #260;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #261;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #262;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #263;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #264;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #265;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #266;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #267;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #268;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #269;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #270;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #271;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #272;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #273;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #274;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #275;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #276;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #277;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #278;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #279;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #280;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #281;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #282;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #283;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #284;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #285;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #286;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #287;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #288;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #291;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #292;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #294;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #295;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #296;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #297;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 43
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #298;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #299;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #300;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #302;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #308;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #315;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #317;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #330;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #331;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 45
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #332;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #333;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #334;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #335;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 68
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #336;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 68
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #337;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 71
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #338;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 43
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #339;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #340;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #341;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #342;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 29
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #343;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #344;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #345;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #346;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #347;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #348;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #349;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #350;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #351;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #352;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #353;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #354;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #355;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #356;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #357;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #358;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #359;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 17
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #360;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #361;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #362;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #363;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #364;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #367;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #368;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #375;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #376;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #377;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #378;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #379;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #380;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #387;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #388;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #394;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #395;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #396;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #397;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #398;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #399;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #400;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #405;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #406;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #407;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #409;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #410;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #411;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 27
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #412;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 25
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #413;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 42
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #414;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #415;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 30
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #416;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #417;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 26
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #418;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #419;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #420;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #421;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #422;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #423;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #424;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #425;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #426;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #427;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 21
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #428;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #429;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #430;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #431;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #432;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #433;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #434;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #435;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #436;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 15
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #437;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #438;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #440;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 3
#I  **************************************************
#I  Starting group: [grp] #452;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #453;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 18
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #454;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 15
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #455;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 13
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #456;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #457;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #458;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #459;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #460;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #464;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 48
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #465;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 75
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #466;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #467;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #468;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #469;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #470;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #471;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 29
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #472;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #473;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #474;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #475;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #476;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 38
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #477;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 72
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #478;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #479;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 26
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #480;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 26
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #481;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #482;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #483;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #484;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #485;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #486;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #487;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #488;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #489;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #490;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #491;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #492;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #493;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #494;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #495;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #496;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #497;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #498;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #499;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #500;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #501;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #502;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #503;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #504;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #505;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #506;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #507;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #508;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #509;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #510;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #511;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #512;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #513;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #514;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #515;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #516;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #517;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #518;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #519;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #520;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #521;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #522;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #523;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #524;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #525;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #526;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #527;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #528;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #529;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #530;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #545;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 4
#I  **************************************************
#I  Starting group: [grp] #556;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #578;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #600;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #608;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #609;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #610;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #611;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #612;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #613;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #614;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #615;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #616;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #617;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #618;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #619;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #622;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 3
#I  **************************************************
#I  Starting group: [grp] #636;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #638;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 3
#I  **************************************************
#I  Starting group: [grp] #642;3
#I  Order: 2^7
#I  Nuclear rank: 6
#I  2-multiplicator rank: 13
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #643;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 51
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #644;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 25
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #645;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 46
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #646;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 42
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #647;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #648;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #649;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #650;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #651;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #652;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 35
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #653;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 29
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #654;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 32
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #655;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 9
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #656;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #657;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #658;3
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  # of immediate descendants of order 2^8 is 11
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #663;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #664;3
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #665;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #665;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #666;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #666;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #667;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #667;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #668;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #668;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #669;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #669;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #670;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #670;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #671;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #671;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #672;4
#I  Order: 2^8
#I  Nuclear rank: 8
#I  2-multiplicator rank: 14
#I  Group [grp] #672;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #673;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #673;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #674;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #674;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #675;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #675;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #676;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #676;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #677;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #677;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #678;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #678;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #679;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #679;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #680;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #680;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #681;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #681;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #682;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #682;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #683;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #683;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #684;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #684;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #685;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #685;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #686;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #686;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #687;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #687;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #688;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #688;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #689;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #689;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #690;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #690;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #691;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #691;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #692;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #692;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #693;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #693;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #694;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #694;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #695;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #695;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #696;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #696;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #697;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #697;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #698;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #698;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #699;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #699;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #700;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #700;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #701;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #701;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #702;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #702;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #703;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #703;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #704;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #704;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #705;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #705;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #706;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #706;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #707;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #707;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #708;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #708;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #709;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #709;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #710;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #710;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #711;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #711;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #712;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #712;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #713;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #713;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #714;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #714;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #715;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #715;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #716;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #716;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #717;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #717;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #718;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #718;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #719;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #719;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #720;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #720;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #721;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #721;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #722;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #722;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #723;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #723;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #724;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #724;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #725;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #725;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #726;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #726;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #727;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #727;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #728;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #728;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #729;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #729;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #730;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #730;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #731;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #731;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #732;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #732;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #733;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #733;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #734;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #734;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #735;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #735;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #736;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #736;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #737;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #737;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #738;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #738;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #739;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #739;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #740;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #740;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #741;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #741;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #742;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #742;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #743;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #743;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #744;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #744;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #745;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #745;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #746;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #746;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #747;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #747;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #748;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #748;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #749;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #749;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #750;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #750;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #751;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #751;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #752;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #752;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #753;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #753;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #754;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #754;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #755;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #755;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #756;4
#I  Order: 2^8
#I  Nuclear rank: 8
#I  2-multiplicator rank: 14
#I  Group [grp] #756;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #757;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #757;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #758;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #758;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #759;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #759;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #760;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #760;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #761;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #761;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #762;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #762;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #763;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #763;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #764;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #764;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #765;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #765;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #766;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #766;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #767;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #767;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #768;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #768;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #769;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #769;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #770;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #770;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #771;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #771;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #772;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #772;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #773;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #773;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #774;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #774;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #775;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #775;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #776;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #776;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #777;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #777;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #778;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #778;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #779;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #779;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #780;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #780;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #781;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #781;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #782;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #782;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #783;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #783;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #784;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #784;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #785;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #785;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #786;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #786;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #787;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #787;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #788;4
#I  Order: 2^8
#I  Nuclear rank: 8
#I  2-multiplicator rank: 14
#I  Group [grp] #788;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #789;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #789;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #790;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #790;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #791;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #791;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #792;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #792;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #793;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #793;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #794;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #794;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #795;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #795;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #796;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #796;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #797;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #797;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #798;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #798;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #799;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #799;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #800;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #800;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #801;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #801;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #802;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #802;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #803;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #803;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #804;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #804;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #805;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #805;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #806;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #806;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #807;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #807;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #808;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #808;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #809;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #809;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #810;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #810;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #811;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #811;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #812;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #812;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #813;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #813;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #814;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #814;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #815;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #815;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #816;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #816;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #817;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #817;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #818;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #818;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #819;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #819;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #820;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #820;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #821;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #821;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #822;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #822;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #823;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #823;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #824;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #824;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #825;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #825;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #826;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #826;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #827;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #827;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #828;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #828;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #829;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #829;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #830;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #830;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #831;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #831;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #832;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #832;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #833;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #833;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #834;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #834;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #835;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #835;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #836;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #836;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #837;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #837;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #838;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #838;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #839;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #839;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #840;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #840;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #841;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #841;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #842;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #842;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #843;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #843;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #844;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #844;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #845;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #845;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #846;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #846;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #847;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #847;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #848;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #848;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #849;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #849;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #850;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #850;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #851;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #851;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #852;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #852;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #853;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #853;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #854;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #854;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #855;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #855;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #856;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #856;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #857;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #857;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #858;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #858;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #859;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #859;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #860;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #860;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #861;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #861;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #862;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #862;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #863;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #863;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #864;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #864;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #865;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #865;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #866;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #866;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #867;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #867;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #868;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #868;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #869;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #869;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #870;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #870;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #871;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #871;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #872;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #872;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #873;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #873;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #874;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #874;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #875;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #875;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #876;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #876;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #877;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #877;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #878;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #878;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #879;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #879;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #880;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #880;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #881;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #881;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #882;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #882;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #883;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #883;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #884;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #884;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #885;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #885;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #886;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #886;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #887;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #887;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #888;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #888;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #889;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #889;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #890;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #890;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #891;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #891;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #892;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #892;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #893;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #893;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #894;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #894;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #895;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #895;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #896;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #896;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #897;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #897;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #898;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #898;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #899;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #899;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #900;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #900;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #901;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #901;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #902;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #902;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #903;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #903;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #904;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #904;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #905;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #905;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #906;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #906;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #907;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #907;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #908;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #908;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #909;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #909;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #910;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #910;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #911;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #911;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #912;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #912;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #913;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #913;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #914;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #914;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #915;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #915;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #916;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #916;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #917;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #917;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #918;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #918;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #919;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #919;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #920;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #920;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #921;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #921;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #922;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #922;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #923;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #923;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #924;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #924;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #925;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #925;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #926;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #926;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #927;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #927;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #928;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #928;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #929;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #929;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #930;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #930;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #931;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #931;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #932;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #932;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #933;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #933;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #934;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #934;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #935;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #935;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #936;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #936;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #937;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #937;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #938;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #938;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #939;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #939;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #940;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #940;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #941;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #941;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #942;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #942;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #943;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #943;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #944;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #944;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #945;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #945;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #946;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #946;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #947;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #947;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #948;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #948;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #949;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #949;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #950;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #950;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #951;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #951;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #952;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #952;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #953;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #953;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #954;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #954;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #955;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #955;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #956;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #956;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #957;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #957;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #958;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #958;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #959;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #959;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #960;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #960;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #961;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #961;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #962;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #962;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #963;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #963;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #964;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #964;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #965;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #965;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #966;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #966;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #967;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #967;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #968;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #968;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #969;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #969;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #970;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #970;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #971;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #971;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #972;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #972;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #973;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #973;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #974;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #974;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #975;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #975;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #976;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #976;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #977;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #977;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #978;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #978;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #979;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #979;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #980;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #980;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #981;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #981;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #982;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #982;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #983;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #983;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #984;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #984;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #985;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #985;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #986;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #986;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #987;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #987;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #988;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #988;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #989;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #989;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #990;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #990;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #991;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #991;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #992;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #992;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #993;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #993;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #994;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #994;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #995;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #995;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #996;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #996;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #997;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #997;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #998;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #998;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #999;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #999;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1000;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1000;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1001;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1001;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1002;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1002;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1003;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1003;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1004;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1004;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1005;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1005;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1006;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1006;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1007;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1007;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1008;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1008;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1009;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1009;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1010;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1010;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1011;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1011;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1012;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1012;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1013;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1013;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1014;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1014;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1015;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1015;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1016;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1016;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1017;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1017;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1018;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1018;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1019;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1019;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1020;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1020;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1021;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1021;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1022;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1022;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1023;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1023;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1024;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1024;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1025;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1025;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1026;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1026;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1027;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1027;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1028;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1028;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1029;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1029;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1030;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1030;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1031;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1031;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1032;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1032;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1033;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1033;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1034;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1034;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1035;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1035;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1036;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1036;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1037;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1037;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1038;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1038;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1039;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1039;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1040;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1040;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1041;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1041;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1042;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1042;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1043;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1043;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1044;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1044;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1045;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1045;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1046;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1046;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1047;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1047;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1048;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1048;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1049;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1049;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1050;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1050;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1051;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1051;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1052;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1052;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1053;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1053;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1054;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1054;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1055;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1055;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1056;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1056;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1057;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1057;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1058;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1058;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1059;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1059;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1060;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1060;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1061;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1061;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1062;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1062;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1063;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1063;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1064;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1064;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1065;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1065;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1066;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1066;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1067;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1067;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1068;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1068;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1069;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1069;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1070;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1070;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1071;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1071;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1072;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1072;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1073;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1073;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1074;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1074;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1075;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1075;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1076;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1076;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1077;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1077;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1078;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1078;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1079;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1079;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1080;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1080;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1081;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1081;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1082;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1082;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1083;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1083;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1084;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1084;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1085;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1085;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1086;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1086;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1087;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1087;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1088;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1088;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1089;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1089;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1090;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1090;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1091;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1091;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1092;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1092;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1093;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1093;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1094;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1094;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1095;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1095;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1096;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1096;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1097;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1097;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1098;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1098;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1099;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1099;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1100;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1100;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1101;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1101;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1102;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1102;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1103;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1103;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1104;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1104;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1105;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1105;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1106;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1106;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1107;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1107;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1108;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1108;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1109;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1109;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1110;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1110;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1111;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1111;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1112;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1112;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1113;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1113;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1114;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1114;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1115;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1115;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1116;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1116;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1117;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1117;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1118;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1118;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1119;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1119;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1120;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1120;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1121;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1121;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1122;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1122;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1123;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1123;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1124;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1124;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1125;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1125;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1126;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1126;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1127;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1127;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1128;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1128;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1129;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1129;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1130;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1130;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1131;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1131;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1132;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1132;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1133;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1133;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1134;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1134;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1135;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1135;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1136;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1136;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1137;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1137;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1138;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1138;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1139;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1139;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1140;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1140;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1141;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1141;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1142;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1142;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1143;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1143;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1144;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1144;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1145;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1145;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1146;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1146;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1147;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1147;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1148;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1148;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1149;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1149;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1150;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1150;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1151;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1151;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1152;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1152;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1153;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1153;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1154;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1154;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1155;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1155;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1156;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1156;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1157;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1157;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1158;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1158;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1159;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1159;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1160;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1160;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1161;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1161;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1162;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1162;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1163;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1163;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1164;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1164;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1165;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1165;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1166;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1166;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1167;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1167;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1168;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1168;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1169;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1169;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1170;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1170;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1171;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1171;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1172;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1172;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1173;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1173;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1174;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1174;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1175;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1175;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1176;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1176;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1177;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1177;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1178;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1178;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1179;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1179;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1180;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1180;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1181;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1181;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1182;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1182;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1183;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1183;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1184;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1184;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1185;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1185;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1186;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1186;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1187;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1187;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1188;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1188;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1189;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1189;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1190;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1190;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1191;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1191;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1192;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1192;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1193;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1193;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1194;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1194;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1195;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1195;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1196;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1196;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1197;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1197;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1198;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1198;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1199;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1199;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1200;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1200;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1201;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1201;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1202;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1202;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1203;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1203;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1204;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1204;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1205;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1205;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1206;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1206;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1207;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1207;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1208;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1208;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1209;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1209;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1210;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1210;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1211;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1211;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1212;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1212;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1213;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1213;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1214;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1214;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1215;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1215;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1216;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1216;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1217;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1217;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1218;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1218;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1219;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1219;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1220;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1220;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1221;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1221;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1222;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1222;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1223;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1223;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1224;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1224;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1225;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1225;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1226;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1226;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1227;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1227;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1228;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1228;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1229;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1229;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1230;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1230;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1231;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1231;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1232;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1232;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1233;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1233;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1234;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1234;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1235;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1235;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1236;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1236;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1237;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1237;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1238;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1238;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1239;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1239;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1240;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1240;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1241;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1241;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1242;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1242;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1243;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1243;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1244;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1244;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1245;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1245;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1246;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1246;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1247;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1247;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1248;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1248;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1249;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1249;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1250;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1250;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1251;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1251;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1252;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1252;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1253;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1253;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1254;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1254;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1255;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1255;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1256;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1256;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1257;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1257;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1258;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1258;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1259;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1259;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1260;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1260;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1261;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1261;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1262;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1262;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1263;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1263;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1264;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1264;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1265;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1265;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1266;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1266;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1267;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1267;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1268;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1268;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1269;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1269;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1270;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1270;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1271;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1271;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1272;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1272;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1273;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1273;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1274;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1274;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1275;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1275;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1276;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1276;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1277;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1277;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1278;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1278;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1279;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1279;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1280;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1280;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1281;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1281;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1282;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1282;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1283;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1283;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1284;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1284;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1285;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1285;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1286;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1286;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1287;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1287;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1288;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1288;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1289;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1289;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1290;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1290;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1291;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1291;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1292;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1292;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1293;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1293;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1294;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1294;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1295;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1295;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1296;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1296;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1297;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1297;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1298;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1298;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1299;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1299;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1300;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1300;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1301;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1301;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1302;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1302;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1303;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1303;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1304;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1304;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1305;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1305;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1306;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1306;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1307;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1307;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1308;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1308;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1309;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1309;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1310;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1310;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1311;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1311;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1312;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1312;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1313;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1313;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1314;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1314;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1315;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1315;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1316;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1316;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1317;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1317;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1318;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1318;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1319;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1319;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1320;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1320;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1321;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1321;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1322;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1322;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1323;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1323;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1324;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1324;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1325;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1325;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1326;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1326;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1327;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1327;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1328;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1328;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1329;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1329;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1330;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1330;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1331;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1331;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1332;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1332;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1333;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1333;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1334;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1334;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1335;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1335;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1336;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1336;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1337;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1337;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1338;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1338;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1339;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1339;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1340;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1340;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1341;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1341;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1342;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1342;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1343;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1343;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1344;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1344;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1345;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1345;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1346;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1346;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1347;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1347;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1348;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1348;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1349;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1349;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1350;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1350;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1351;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1351;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1352;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1352;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1353;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1353;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1354;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1354;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1355;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1355;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1356;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1356;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1357;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1357;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1358;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1358;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1359;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1359;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1360;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1360;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1361;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1361;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1362;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1362;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1363;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1363;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1364;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1364;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1365;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1365;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1366;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1366;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1367;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1367;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1368;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1368;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1369;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1369;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1370;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1370;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1371;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1371;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1372;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1372;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1373;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1373;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1374;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1374;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1375;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1375;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1376;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1376;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1377;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1377;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1378;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1378;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1379;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1379;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1380;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1380;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1381;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1381;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1382;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1382;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1383;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1383;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1384;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1384;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1385;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1385;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1386;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1386;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1387;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1387;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1388;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1388;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1389;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1389;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1390;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1390;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1391;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1391;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1392;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1392;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1393;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1393;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1394;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1394;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1395;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1395;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1396;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1396;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1397;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1397;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1398;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1398;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1399;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1399;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1400;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1400;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1401;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1401;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1402;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1402;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1403;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1403;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1404;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1404;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1405;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1405;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1406;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1406;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1407;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1407;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1408;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1408;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1409;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1409;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1410;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1410;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1411;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1411;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1412;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1412;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1413;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1413;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1414;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1414;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1415;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1415;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1416;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1416;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1417;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1417;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1418;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1418;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1419;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1419;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1420;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1420;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1421;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1421;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1422;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1422;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1423;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1423;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1424;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1424;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1425;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1425;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1426;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1426;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1427;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1427;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1428;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1428;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1429;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1429;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1430;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1430;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1431;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1431;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1432;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1432;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1433;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1433;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1434;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1434;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1435;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1435;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1436;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1436;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1437;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1437;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1438;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1438;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1439;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1439;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1440;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1440;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1441;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1441;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1442;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1442;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1443;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1443;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1444;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1444;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1445;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1445;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1446;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1446;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1447;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1447;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1448;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1448;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1449;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1449;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1450;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1450;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1451;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1451;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1452;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1452;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1453;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1453;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1454;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1454;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1455;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1455;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1456;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1456;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1457;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1457;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1458;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1458;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1459;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1459;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1460;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1460;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1461;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1461;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1462;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1462;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1463;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1463;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1464;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1464;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1465;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1465;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1466;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1466;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1467;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1467;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1468;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1468;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1469;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1469;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1470;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1470;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1471;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1471;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1472;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1472;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1473;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1473;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1474;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1474;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1475;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1475;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1476;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1476;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1477;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1477;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1478;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1478;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1479;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1479;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1480;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1480;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1481;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1481;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1482;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1482;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1483;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1483;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1484;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1484;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1485;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1485;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1486;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1486;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1487;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1487;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1488;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1488;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1489;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1489;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1490;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1490;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1491;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1491;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1492;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1492;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1493;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1493;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1494;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1494;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1495;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1495;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1496;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1496;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1497;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1497;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1498;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1498;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1499;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1499;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1500;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1500;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1501;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1501;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1502;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1502;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1503;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1503;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1504;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1504;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1505;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1505;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1506;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1506;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1507;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1507;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1508;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1508;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1509;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1509;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1510;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1510;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1511;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1511;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1512;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1512;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1513;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1513;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1514;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1514;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1515;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1515;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1516;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1516;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1517;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1517;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1518;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1518;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1519;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1519;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1520;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1520;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1521;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1521;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1522;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1522;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1523;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1523;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1524;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1524;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1525;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1525;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1526;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1526;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1527;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1527;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1528;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1528;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1529;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1529;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1530;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1530;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1531;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1531;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1532;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1532;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1533;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1533;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1534;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1534;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1535;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1535;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1536;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1536;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1537;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1537;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1538;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1538;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1539;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1539;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1540;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1540;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1541;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1541;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1542;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1542;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1543;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1543;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1544;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1544;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1545;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1545;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1546;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1546;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1547;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1547;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1548;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1548;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1549;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1549;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1550;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1550;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1551;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1551;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1552;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1552;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1553;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1553;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1554;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1554;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1555;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1555;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1556;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1556;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1557;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1557;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1558;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1558;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1559;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1559;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1560;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1560;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1561;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1561;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1562;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1562;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1563;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1563;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1564;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1564;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1565;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1565;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1566;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1566;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1567;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1567;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1568;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1568;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1569;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1569;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1570;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1570;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1571;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1571;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1572;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1572;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1573;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1573;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1574;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1574;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1575;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1575;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1576;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1576;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1577;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1577;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1578;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1578;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1579;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1579;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1580;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1580;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1581;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1581;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1582;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1582;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1583;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1583;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1584;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1584;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1585;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1585;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1586;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1586;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1587;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1587;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1588;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1588;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1589;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1589;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1590;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1590;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1591;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1591;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1592;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1592;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1593;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1593;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1594;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1594;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1595;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1595;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1596;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1596;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1597;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1597;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1598;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1598;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1599;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1599;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1600;4
#I  Order: 2^8
#I  Nuclear rank: 10
#I  2-multiplicator rank: 16
#I  Group [grp] #1600;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1601;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1601;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1602;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1602;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1603;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1603;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1604;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1604;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1605;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1605;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1606;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1606;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1607;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1607;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1608;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1608;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1609;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1609;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1610;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1610;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1611;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1611;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1612;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1612;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1613;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1613;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1614;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1614;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1615;4
#I  Order: 2^8
#I  Nuclear rank: 8
#I  2-multiplicator rank: 14
#I  Group [grp] #1615;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1616;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1616;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1617;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1617;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1618;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1618;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1619;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1619;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1620;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1620;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1621;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1621;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1622;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1622;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1623;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1623;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1624;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1624;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1625;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1625;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1626;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1626;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1627;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1627;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1628;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1628;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1629;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1629;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1630;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1630;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1631;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1631;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1632;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1632;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1633;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1633;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1634;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1634;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1635;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1635;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1636;4
#I  Order: 2^8
#I  Nuclear rank: 8
#I  2-multiplicator rank: 14
#I  Group [grp] #1636;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1637;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1637;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1638;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1638;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1639;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1639;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1640;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1640;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1641;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1641;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1642;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1642;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1643;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1643;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1644;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1644;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1645;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1645;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1646;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1646;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1647;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1647;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1648;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1648;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1649;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1649;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1650;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1650;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1651;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1651;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1652;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1652;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1653;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1653;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1654;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1654;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1655;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1655;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1656;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1656;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1657;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1657;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1658;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1658;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1659;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1659;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1660;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1660;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1661;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1661;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1662;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1662;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1663;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1663;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1664;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1664;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1665;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1665;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1666;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1666;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1667;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1667;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1668;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1668;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1669;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1669;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1670;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1670;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1671;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1671;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1672;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1672;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1673;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1673;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1674;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1674;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1675;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1675;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1676;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1676;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1677;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1677;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1678;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1678;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1679;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1679;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1680;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1680;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1681;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1681;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1682;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1682;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1683;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1683;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1684;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1684;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1685;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1685;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1686;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1686;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1687;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1687;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1688;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1688;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1689;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1689;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1690;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1690;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1691;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1691;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1692;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1692;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1693;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1693;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1694;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1694;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1695;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1695;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1696;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1696;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1697;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1697;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1698;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1698;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1699;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1699;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1700;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1700;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1701;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1701;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1702;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1702;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1703;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1703;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1704;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1704;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1705;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1705;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1706;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1706;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1707;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1707;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1708;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1708;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1709;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1709;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1710;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1710;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1711;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1711;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1712;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1712;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1713;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1713;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1714;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1714;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1715;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1715;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1716;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1716;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1717;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1717;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1718;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1718;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1719;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1719;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1720;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1720;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1721;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1721;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1722;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1722;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1723;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1723;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1724;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1724;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1725;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1725;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1726;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1726;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1727;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1727;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1728;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1728;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1729;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1729;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1730;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1730;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1731;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1731;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1732;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1732;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1733;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1733;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1734;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1734;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1735;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1735;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1736;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1736;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1737;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1737;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1738;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1738;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1739;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1739;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1740;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1740;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1741;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1741;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1742;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1742;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1743;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1743;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1744;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1744;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1745;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1745;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1746;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1746;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1747;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1747;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1748;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1748;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1749;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1749;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1750;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1750;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1751;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1751;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1752;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1752;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1753;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1753;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1754;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1754;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1755;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1755;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1756;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1756;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1757;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1757;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1758;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1758;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1759;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1759;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1760;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1760;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1761;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1761;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1762;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1762;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1763;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1763;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1764;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1764;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1765;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1765;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1766;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1766;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1767;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1767;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1768;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1768;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1769;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1769;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1770;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1770;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1771;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1771;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1772;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1772;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1773;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1773;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1774;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1774;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1775;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1775;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1776;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1776;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1777;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1777;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1778;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1778;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1779;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1779;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1780;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1780;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1781;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1781;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1782;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1782;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1783;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1783;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1784;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1784;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1785;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1785;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1786;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1786;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1787;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1787;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1788;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1788;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1789;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1789;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1790;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1790;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1791;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1791;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1792;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1792;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1793;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1793;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1794;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1794;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1795;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1795;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1796;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1796;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1797;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1797;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1798;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1798;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1799;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1799;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1800;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1800;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1801;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1801;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1802;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1802;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1803;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1803;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1804;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1804;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1805;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1805;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1806;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #1806;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1807;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1807;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1808;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1808;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1809;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1809;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1810;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1810;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1811;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1811;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1812;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1812;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1813;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1813;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1814;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1814;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1815;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1815;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1816;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1816;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1817;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1817;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1818;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1818;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1819;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1819;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1820;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1820;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1821;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1821;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1822;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1822;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1823;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1823;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1824;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1824;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1825;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1825;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1826;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1826;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1827;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1827;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1828;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1828;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1829;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1829;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1830;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1830;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1831;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1831;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1832;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #1832;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1833;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1833;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1834;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1834;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1835;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1835;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1836;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1836;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1837;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1837;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1838;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1838;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1839;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1839;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1840;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1840;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1841;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1841;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1842;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1842;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1843;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1843;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1844;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1844;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1845;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1845;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1846;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1846;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1847;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1847;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1848;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1848;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1849;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1849;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1850;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1850;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1851;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1851;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1852;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1852;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1853;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1853;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1854;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1854;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1855;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1855;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1856;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1856;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1857;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1857;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1858;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1858;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1859;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1859;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1860;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1860;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1861;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1861;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1862;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1862;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1863;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1863;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1864;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1864;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1865;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1865;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1866;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1866;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1867;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1867;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1868;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1868;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1869;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1869;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1870;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1870;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1871;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1871;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1872;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1872;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1873;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1873;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1874;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1874;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1875;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1875;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1876;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1876;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1877;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1877;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1878;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1878;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1879;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1879;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1880;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1880;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1881;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1881;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1882;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1882;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1883;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1883;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1884;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1884;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1885;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1885;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1886;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1886;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1887;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1887;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1888;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1888;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1889;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1889;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1890;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1890;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1891;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1891;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1892;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1892;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1893;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1893;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1894;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1894;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1895;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1895;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1896;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1896;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1897;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1897;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1898;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1898;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1899;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1899;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1900;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1900;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1901;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1901;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1902;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1902;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1903;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1903;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1904;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1904;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1905;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1905;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1906;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1906;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1907;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1907;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1908;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1908;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1909;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1909;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1910;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1910;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1911;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1911;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1912;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1912;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1913;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1913;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1914;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1914;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1915;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1915;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1916;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1916;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1917;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1917;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1918;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1918;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1919;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1919;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1920;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1920;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1921;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1921;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1922;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1922;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1923;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1923;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1924;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1924;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1925;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1925;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1926;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1926;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1927;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1927;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1928;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1928;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1929;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1929;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1930;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1930;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1931;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1931;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1932;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1932;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1933;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1933;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1934;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1934;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1935;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1935;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1936;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1936;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1937;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1937;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1938;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1938;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1939;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1939;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1940;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1940;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1941;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1941;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1942;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1942;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1943;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1943;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1944;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1944;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1945;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1945;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1946;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1946;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1947;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1947;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1948;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1948;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1950;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1950;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1952;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1952;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1954;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1954;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1956;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1956;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1959;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1959;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1961;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1961;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1962;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1962;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1964;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1964;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1965;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1965;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1966;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1966;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1967;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1967;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1968;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1968;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1969;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1969;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1970;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1970;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1971;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1971;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1972;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1972;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1973;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1973;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1974;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1974;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1975;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1975;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1976;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1976;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1978;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1978;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1980;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1980;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1981;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1981;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1982;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1982;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1983;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1983;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1984;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1984;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1985;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1985;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1986;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1986;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1987;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1987;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1988;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1988;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1989;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #1989;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1990;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #1990;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1991;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1991;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1992;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1992;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1993;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1993;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1994;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #1994;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1995;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1995;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1996;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1996;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1997;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #1997;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1998;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1998;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #1999;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #1999;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2000;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2000;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2001;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2001;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2002;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2002;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2003;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2003;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2004;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2004;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2005;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2005;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2006;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2006;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2007;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2007;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2008;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2008;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2009;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2009;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2010;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2010;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2011;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2011;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2012;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2012;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2013;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2013;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2014;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2014;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2015;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2015;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2016;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2016;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2017;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2017;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2018;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2018;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2019;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2019;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2020;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2020;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2021;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2021;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2022;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2022;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2023;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2023;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2024;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2024;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2025;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2025;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2026;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2026;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2027;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2027;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2028;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2028;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2029;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2029;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2030;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2030;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2031;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2031;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2032;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2032;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2033;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2033;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2034;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2034;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2035;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2035;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2036;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2036;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2037;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2037;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2038;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2038;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2039;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2039;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2040;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2040;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2041;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2041;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2042;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2042;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2043;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2043;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2044;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2044;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2045;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2045;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2046;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2046;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2047;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2047;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2048;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2048;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2049;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2049;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2050;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2050;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2051;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2051;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2052;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2052;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2053;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2053;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2054;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2054;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2055;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2055;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2056;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2056;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2057;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2057;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2058;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2058;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2059;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2059;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2060;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2060;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2061;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2061;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2062;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2062;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2063;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2063;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2064;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2064;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2065;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2065;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2066;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2066;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2067;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2067;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2068;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2068;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2069;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2069;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2070;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2070;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2071;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2071;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2072;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2072;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2073;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2073;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2074;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2074;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2075;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2075;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2076;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2076;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2077;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2077;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2078;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2078;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2079;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2079;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2080;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2080;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2081;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2081;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2082;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2082;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2083;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2083;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2084;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2084;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2085;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2085;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2086;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2086;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2087;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2087;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2088;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2088;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2089;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2089;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2090;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2090;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2091;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2091;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2092;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2092;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2093;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2093;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2094;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2094;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2095;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2095;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2096;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2096;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2097;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2097;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2098;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2098;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2099;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2099;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2100;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2100;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2101;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2101;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2102;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2102;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2103;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2103;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2104;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2104;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2105;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2105;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2106;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2106;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2107;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2107;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2108;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2108;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2109;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2109;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2110;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2110;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2111;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2111;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2112;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2112;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2113;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2113;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2114;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2114;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2115;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2115;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2116;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2116;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2117;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2117;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2118;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2118;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2119;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2119;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2120;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2120;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2121;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2121;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2122;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2122;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2123;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2123;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2124;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2124;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2125;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2125;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2126;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2126;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2127;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2127;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2128;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2128;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2130;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2130;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2132;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2132;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2133;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2133;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2134;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2134;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2135;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2135;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2136;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2136;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2137;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2137;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2138;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2138;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2139;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2139;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2140;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2140;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2141;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2141;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2142;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2142;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2143;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2143;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2144;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2144;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2145;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2145;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2146;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2146;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2147;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2147;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2148;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2148;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2149;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2149;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2150;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2150;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2151;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2151;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2152;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2152;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2153;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2153;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2154;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2154;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2155;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2155;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2156;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2156;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2157;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2157;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2158;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2158;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2159;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2159;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2160;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2160;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2161;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2161;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2162;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2162;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2163;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2163;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2164;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2164;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2165;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2165;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2166;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2166;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2167;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2167;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2168;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2168;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2169;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2169;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2170;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2170;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2171;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2171;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2172;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2172;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2173;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2173;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2174;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2174;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2175;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2175;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2176;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2176;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2177;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2177;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2178;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #2178;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2179;4
#I  Order: 2^8
#I  Nuclear rank: 7
#I  2-multiplicator rank: 13
#I  Group [grp] #2179;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2180;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2180;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2181;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2181;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2182;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2182;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2183;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2183;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2184;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2184;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2185;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2185;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2186;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2186;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2187;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2187;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2188;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2188;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2189;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2189;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2190;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2190;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2191;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2191;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2192;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2192;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2193;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2193;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2194;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2194;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2195;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2195;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2196;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2196;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2197;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2197;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2198;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2198;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2199;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2199;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2200;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2200;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2201;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2201;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2202;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2202;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2203;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2203;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2204;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2204;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2205;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2205;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2206;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2206;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2207;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2207;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2208;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2208;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2209;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2209;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2210;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2210;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2211;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2211;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2212;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2212;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2213;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2213;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2214;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2214;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2215;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2215;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2216;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2216;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2217;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2217;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2218;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2218;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2219;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2219;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2220;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2220;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2221;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2221;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2222;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2222;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2223;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2223;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2224;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2224;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2225;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2225;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2226;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2226;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2227;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2227;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2228;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2228;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2229;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2229;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2230;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2230;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2231;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2231;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2232;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2232;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2233;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2233;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2234;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2234;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2235;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2235;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2236;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2236;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2237;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2237;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2238;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2238;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2239;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2239;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2240;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2240;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2241;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2241;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2242;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2242;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2243;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2243;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2244;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2244;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2245;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2245;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2246;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2246;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2247;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2247;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2248;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2248;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2249;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2249;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2250;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2250;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2251;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2251;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2252;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2252;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2253;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2253;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2254;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2254;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2255;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2255;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2256;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2256;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2257;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2257;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2258;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2258;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2259;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2259;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2260;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2260;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2261;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2261;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2262;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2262;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2263;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2263;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2264;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2264;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2265;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2265;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2266;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2266;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2267;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2267;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2268;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2268;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2269;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2269;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2270;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2270;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2271;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2271;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2272;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2272;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2273;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2273;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2274;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2274;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2275;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2275;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2276;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2276;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2277;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2277;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2278;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2278;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2279;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2279;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2280;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2280;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2281;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2281;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2282;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2282;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2283;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2283;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2284;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2284;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2285;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2285;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2286;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2286;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2287;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2287;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2288;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2288;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2289;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2289;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2290;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2290;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2291;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2291;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2292;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2292;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2293;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2293;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2294;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2294;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2295;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2295;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2296;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2296;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2297;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2297;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2298;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2298;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2299;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2299;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2300;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2300;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2301;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2301;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2302;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2302;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2303;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2303;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2304;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2304;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2305;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2305;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2306;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2306;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2307;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2307;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2308;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2308;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2309;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2309;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2310;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2310;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2311;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2311;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2312;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2312;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2313;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2313;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2314;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2314;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2315;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2315;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2316;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2316;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2317;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2317;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2318;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2318;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2319;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2319;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2320;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2320;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2321;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2321;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2322;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2322;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2323;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2323;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2324;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2324;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2325;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2325;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2326;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2326;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2327;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2327;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2328;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2328;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2329;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2329;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2330;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2330;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2331;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2331;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2332;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2332;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2333;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2333;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2334;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2334;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2335;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2335;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2336;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2336;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2337;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2337;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2338;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2338;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2339;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2339;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2340;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2340;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2341;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2341;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2342;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2342;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2343;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2343;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2344;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2344;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2345;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2345;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2346;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2346;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2347;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2347;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2348;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2348;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2349;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2349;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2350;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2350;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2351;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2351;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2352;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2352;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2353;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2353;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2354;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2354;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2355;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2355;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2356;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2356;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2357;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2357;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2358;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2358;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2359;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2359;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2360;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2360;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2361;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2361;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2362;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2362;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2363;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2363;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2364;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2364;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2365;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2365;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2366;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2366;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2367;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2367;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2368;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2368;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2369;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2369;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2370;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2370;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2371;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2371;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2372;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2372;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2373;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2373;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2374;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2374;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2375;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2375;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2376;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2376;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2377;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2377;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2378;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2378;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2379;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2379;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2380;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2380;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2381;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2381;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2382;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2382;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2383;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2383;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2384;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2384;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2385;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2385;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2386;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2386;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2387;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2387;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2388;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2388;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2389;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2389;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2390;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2390;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2391;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2391;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2392;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2392;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2393;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2393;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2394;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2394;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2395;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2395;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2396;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2396;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2397;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2397;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2398;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2398;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2399;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2399;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2400;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2400;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2401;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2401;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2402;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2402;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2403;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2403;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2404;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2404;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2405;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2405;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2406;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2406;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2407;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2407;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2408;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2408;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2409;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2409;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2410;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2410;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2411;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2411;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2412;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2412;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2413;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2413;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2414;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2414;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2415;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2415;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2416;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2416;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2417;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2417;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2418;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2418;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2419;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2419;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2420;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2420;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2421;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2421;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2422;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2422;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2423;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2423;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2424;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2424;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2425;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2425;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2426;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2426;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2427;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2427;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2428;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2428;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2429;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2429;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2430;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2430;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2431;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2431;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2432;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2432;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2433;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2433;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2434;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2434;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2435;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2435;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2436;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2436;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2437;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2437;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2438;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2438;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2439;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2439;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2440;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2440;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2441;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2441;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2442;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2442;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2443;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2443;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2444;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2444;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2445;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2445;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2446;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2446;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2447;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2447;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2448;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2448;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2449;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2449;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2450;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2450;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2451;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2451;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2452;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2452;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2453;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2453;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2454;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2454;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2455;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2455;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2456;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2456;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2457;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2457;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2458;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2458;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2459;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2459;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2460;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2460;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2461;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2461;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2462;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2462;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2463;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2463;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2464;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2464;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2465;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2465;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2466;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2466;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2467;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2467;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2468;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2468;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2469;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2469;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2470;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2470;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2471;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2471;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2472;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2472;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2473;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2473;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2474;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2474;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2475;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2475;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2476;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2476;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2477;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2477;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2478;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2478;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2479;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2479;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2480;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2480;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2481;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2481;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2482;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2482;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2483;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2483;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2484;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2484;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2485;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2485;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2486;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2486;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2487;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2487;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2488;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2488;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2489;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2489;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2490;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2490;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2491;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2491;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2492;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2492;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2493;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2493;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2494;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2494;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2495;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2495;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2496;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2496;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2497;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2497;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2498;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2498;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2499;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2499;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2500;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2500;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2501;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2501;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2502;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2502;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2503;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2503;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2504;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2504;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2505;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2505;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2506;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2506;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2507;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2507;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2508;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2508;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2509;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2509;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2510;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2510;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2511;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2511;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2512;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2512;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2513;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2513;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2514;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2514;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2515;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2515;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2516;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2516;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2517;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2517;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2518;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2518;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2519;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2519;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2520;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2520;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2521;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2521;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2522;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2522;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2523;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2523;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2524;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2524;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2525;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2525;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2526;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2526;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2527;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2527;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2528;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2528;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2529;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2529;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2530;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2530;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2531;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2531;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2532;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2532;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2533;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2533;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2534;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2534;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2535;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2535;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2536;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2536;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2537;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2537;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2538;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2538;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2539;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2539;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2540;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2540;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2541;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2541;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2542;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2542;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2543;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2543;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2544;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2544;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2545;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2545;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2546;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2546;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2547;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2547;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2548;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2548;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2549;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2549;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2550;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2550;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2551;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2551;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2552;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2552;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2553;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2553;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2554;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2554;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2555;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2555;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2556;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2556;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2557;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2557;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2558;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2558;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2559;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2559;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2560;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2560;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2561;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2561;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2562;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2562;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2563;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2563;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2564;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2564;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2565;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2565;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2566;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2566;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2567;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2567;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2568;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2568;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2569;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2569;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2570;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2570;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2571;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2571;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2572;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2572;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2573;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2573;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2574;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2574;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2575;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2575;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2576;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2576;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2577;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2577;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2578;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2578;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2579;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2579;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2580;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2580;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2581;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2581;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2582;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2582;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2583;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2583;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2584;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2584;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2585;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2585;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2586;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2586;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2587;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2587;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2588;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2588;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2589;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2589;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2590;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2590;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2591;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2591;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2592;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2592;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2593;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2593;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2594;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2594;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2595;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2595;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2596;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2596;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2597;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2597;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2598;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2598;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2599;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2599;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2600;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2600;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2601;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2601;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2602;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2602;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2603;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2603;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2604;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2604;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2605;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2605;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2606;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2606;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2607;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2607;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2608;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2608;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2609;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2609;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2610;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2610;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2611;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2611;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2612;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2612;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2613;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2613;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2614;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2614;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2615;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2615;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2616;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2616;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2617;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2617;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2618;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2618;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2619;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2619;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2620;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2620;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2621;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2621;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2622;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2622;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2623;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2623;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2624;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2624;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2625;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2625;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2626;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2626;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2627;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2627;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2628;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2628;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2629;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2629;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2630;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2630;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2631;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2631;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2632;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2632;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2633;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2633;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2634;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2634;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2635;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2635;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2636;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2636;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2637;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2637;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2638;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2638;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2639;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2639;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2640;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2640;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2641;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2641;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2642;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2642;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2643;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2643;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2644;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2644;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2645;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2645;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2646;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2646;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2647;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2647;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2648;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2648;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2649;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2649;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2650;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2650;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2651;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2651;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2652;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2652;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2653;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2653;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2654;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2654;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2655;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2655;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2656;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2656;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2657;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2657;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2658;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2658;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2659;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2659;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2660;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2660;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2661;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2661;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2662;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2662;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2663;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2663;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2664;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2664;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2665;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2665;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2666;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2666;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2667;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2667;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2668;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2668;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2669;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2669;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2670;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2670;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2671;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2671;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2672;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2672;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2673;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2673;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2674;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2674;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2675;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2675;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2676;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2676;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2677;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2677;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2678;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2678;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2679;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2679;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2680;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2680;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2681;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2681;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2682;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2682;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2683;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2683;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2684;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2684;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2685;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2685;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2686;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2686;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2687;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2687;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2688;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2688;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2689;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2689;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2690;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2690;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2691;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2691;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2692;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2692;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2693;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2693;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2694;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2694;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2695;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2695;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2696;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2696;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2697;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2697;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2698;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2698;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2699;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2699;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2700;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2700;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2701;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2701;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2702;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2702;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2703;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2703;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2704;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2704;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2705;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2705;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2706;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2706;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2707;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2707;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2708;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2708;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2709;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2709;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2710;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2710;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2711;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2711;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2712;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2712;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2713;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2713;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2714;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2714;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2715;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2715;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2716;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2716;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2717;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2717;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2718;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2718;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2719;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2719;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2720;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2720;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2721;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2721;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2722;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2722;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2723;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2723;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2724;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2724;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2725;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2725;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2726;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2726;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2727;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2727;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2728;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2728;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2729;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2729;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2730;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2730;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2731;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2731;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2732;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2732;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2733;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2733;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2734;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2734;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2735;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2735;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2736;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2736;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2737;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2737;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2738;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2738;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2739;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2739;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2740;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2740;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2741;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2741;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2742;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2742;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2743;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2743;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2744;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2744;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2745;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2745;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2746;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2746;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2747;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2747;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2748;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2748;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2749;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2749;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2750;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2750;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2751;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2751;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2752;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2752;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2753;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2753;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2754;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2754;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2755;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2755;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2756;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2756;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2757;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2757;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2758;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2758;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2759;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2759;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2760;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2760;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2761;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2761;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2762;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2762;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2763;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2763;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2764;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2764;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2765;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2765;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2766;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2766;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2767;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2767;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2768;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2768;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2769;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2769;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2770;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2770;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2771;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2771;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2772;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2772;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2773;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2773;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2774;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2774;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2775;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2775;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2776;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2776;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2777;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2777;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2778;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2778;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2779;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2779;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2780;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2780;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2781;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2781;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2782;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2782;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2783;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2783;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2784;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2784;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2785;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2785;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2786;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2786;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2787;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2787;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2788;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2788;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2789;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2789;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2790;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2790;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2791;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2791;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2792;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2792;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2793;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2793;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2794;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2794;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2795;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2795;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2796;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2796;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2797;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2797;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2798;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2798;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2799;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2799;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2800;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2800;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2801;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2801;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2802;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2802;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2803;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2803;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2804;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2804;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2805;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2805;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2806;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2806;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2807;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2807;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2808;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2808;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2809;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2809;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2810;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2810;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2811;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2811;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2812;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2812;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2813;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2813;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2814;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2814;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2815;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2815;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2816;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2816;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2817;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2817;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2818;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2818;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2819;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2819;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2820;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2820;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2821;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2821;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2822;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2822;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2823;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2823;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2824;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2824;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2825;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2825;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2826;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2826;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2827;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2827;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2828;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2828;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2829;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2829;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2830;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2830;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2831;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2831;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2832;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2832;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2833;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2833;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2834;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2834;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2835;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2835;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2836;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2836;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2837;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2837;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2838;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2838;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2839;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2839;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2840;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2840;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2841;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2841;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2842;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2842;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2843;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2843;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2844;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2844;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2845;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2845;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2846;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2846;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2847;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2847;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2848;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2848;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2849;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2849;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2850;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2850;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2851;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2851;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2852;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2852;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2853;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2853;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2854;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2854;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2855;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2855;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2856;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2856;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2857;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2857;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2858;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2858;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2859;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2859;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2860;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2860;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2861;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2861;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2862;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2862;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2863;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2863;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2864;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2864;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2865;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2865;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2866;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2866;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2867;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2867;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2868;4
#I  Order: 2^8
#I  Nuclear rank: 8
#I  2-multiplicator rank: 14
#I  Group [grp] #2868;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2869;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #2869;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2870;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2870;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2871;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2871;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2872;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2872;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2873;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2873;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2874;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2874;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2875;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2875;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2876;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2876;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2877;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2877;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2878;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2878;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2879;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2879;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2880;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2880;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2881;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2881;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2882;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2882;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2883;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2883;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2884;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2884;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2885;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2885;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2886;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2886;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2887;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2887;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2888;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2888;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2889;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2889;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2890;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2890;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2891;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2891;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2892;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2892;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2893;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2893;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2894;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2894;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2895;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2895;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2896;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2896;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2897;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2897;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2898;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2898;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2899;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2899;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2900;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2900;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2901;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2901;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2902;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2902;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2903;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2903;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2904;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2904;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2905;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2905;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2906;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2906;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2907;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #2907;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2908;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2908;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2909;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2909;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2910;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2910;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2911;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2911;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2912;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2912;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2913;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2913;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2914;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2914;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2915;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2915;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2916;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2916;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2917;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2917;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2918;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2918;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2919;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2919;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2920;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2920;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2921;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2921;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2922;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2922;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2923;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2923;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2924;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2924;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2925;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2925;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2926;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2926;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2927;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2927;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2928;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2928;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2929;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2929;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2930;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2930;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2931;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2931;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2932;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2932;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2933;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2933;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2934;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2934;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2935;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2935;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2936;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2936;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2937;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2937;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2938;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2938;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2939;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2939;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2940;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2940;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2941;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2941;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2942;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2942;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2943;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2943;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2944;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2944;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2945;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2945;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2946;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2946;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2947;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2947;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2948;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2948;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2949;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2949;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2950;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2950;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2951;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2951;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2952;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2952;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2953;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2953;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2954;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2954;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2955;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2955;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2956;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2956;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2957;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2957;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2958;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2958;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2959;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2959;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2960;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2960;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2961;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2961;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2962;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2962;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2963;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2963;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2964;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2964;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2965;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2965;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2966;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2966;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2967;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2967;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2968;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2968;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2969;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2969;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2970;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2970;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2971;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2971;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2972;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #2972;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2973;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2973;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2974;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2974;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2975;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2975;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2976;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2976;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2977;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2977;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2978;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2978;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2979;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2979;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2980;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2980;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2981;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2981;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2982;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2982;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2983;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2983;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2984;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2984;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2985;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2985;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2986;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2986;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2987;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2987;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2988;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2988;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2989;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2989;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2990;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2990;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2991;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #2991;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2992;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2992;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2993;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2993;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2994;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2994;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2995;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2995;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2996;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2996;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2997;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #2997;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2998;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2998;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2999;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #2999;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3000;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3000;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3001;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3001;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3002;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3002;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3003;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3003;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3004;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3004;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3005;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3005;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3006;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3006;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3007;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3007;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3008;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3008;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3009;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3009;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3010;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3010;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3011;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3011;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3012;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3012;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3013;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3013;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3014;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3014;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3015;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3015;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3016;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3016;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3017;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3017;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3018;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3018;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3019;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3019;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3020;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3020;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3021;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3021;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3022;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3022;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3023;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3023;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3024;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3024;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3025;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3025;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3026;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3026;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3027;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3027;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3028;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3028;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3029;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3029;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3030;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3030;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3031;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3031;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3032;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3032;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3033;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3033;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3034;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3034;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3035;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3035;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3036;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3036;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3037;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3037;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3038;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3038;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3039;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3039;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3041;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3041;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3042;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3042;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3043;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3043;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3045;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3045;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3046;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3046;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3049;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3049;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3050;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3050;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3052;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3052;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3053;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3053;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3054;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3054;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3055;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3055;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3056;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3056;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3058;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3058;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3062;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3062;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3064;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3064;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3065;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3065;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3066;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3066;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3067;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3067;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3068;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3068;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3070;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3070;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3072;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3072;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3073;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3073;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3075;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3075;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3077;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3077;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3079;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3079;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3080;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3080;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3081;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3081;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3082;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3082;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3083;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3083;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3084;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3084;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3087;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3087;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3096;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3096;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3098;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3098;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3102;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3102;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3103;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3103;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3104;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3104;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3105;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3105;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3106;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3106;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3111;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3111;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3113;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3113;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3114;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3114;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3115;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3115;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3116;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3116;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3117;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3117;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3118;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3118;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3120;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3120;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3121;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3121;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3122;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3122;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3125;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3125;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3126;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3126;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3127;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3127;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3128;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3128;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3129;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3129;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3130;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3130;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3131;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3131;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3132;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3132;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3134;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3134;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3135;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3135;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3136;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3136;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3137;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3137;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3138;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3138;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3140;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3140;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3141;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3141;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3143;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3143;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3144;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3144;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3145;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3145;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3146;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3146;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3149;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3149;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3150;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3150;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3153;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3153;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3154;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3154;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3156;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3156;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3157;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3157;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3159;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3159;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3161;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3161;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3163;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3163;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3164;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3164;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3165;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3165;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3166;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3166;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3167;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3167;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3168;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3168;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3169;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3169;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3170;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #3170;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3171;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3171;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3172;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3172;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3173;4
#I  Order: 2^8
#I  Nuclear rank: 6
#I  2-multiplicator rank: 12
#I  Group [grp] #3173;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3174;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3174;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3175;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3175;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3176;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3176;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3177;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3177;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3178;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3178;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3179;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3179;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3180;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3180;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3181;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3181;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3182;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3182;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3183;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3183;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3184;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3184;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3185;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3185;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3186;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3186;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3187;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3187;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3188;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3188;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3189;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3189;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3190;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3190;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3191;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3191;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3192;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3192;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3193;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3193;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3194;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3194;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3195;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3195;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3196;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3196;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3197;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3197;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3198;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3198;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3199;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3199;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3200;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3200;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3201;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3201;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3202;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3202;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3203;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3203;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3204;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3204;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3205;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3205;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3206;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3206;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3207;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3207;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3208;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3208;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3209;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3209;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3210;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3210;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3211;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3211;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3212;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3212;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3213;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3213;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3214;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3214;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3215;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3215;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3216;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3216;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3217;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3217;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3218;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3218;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3219;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3219;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3220;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3220;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3221;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3221;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3222;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3222;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3223;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3223;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3224;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3224;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3225;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3225;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3226;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3226;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3227;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3227;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3228;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3228;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3229;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3229;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3230;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3230;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3231;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3231;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3232;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3232;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3233;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3233;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3234;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3234;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3235;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3235;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3236;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3236;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3237;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3237;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3238;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3238;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3239;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3239;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3240;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3240;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3241;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3241;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3242;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3242;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3243;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3243;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3244;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3244;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3245;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3245;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3246;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3246;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3247;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3247;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3248;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3248;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3249;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3249;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3250;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3250;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3251;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3251;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3252;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3252;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3253;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3253;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3254;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3254;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3255;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3255;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3256;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3256;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3257;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3257;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3258;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3258;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3259;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3259;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3260;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3260;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3261;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3261;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3262;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3262;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3263;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3263;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3264;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3264;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3265;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3265;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3266;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3266;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3267;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3267;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3268;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3268;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3269;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3269;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3270;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3270;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3271;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3271;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3272;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3272;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3273;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3273;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3274;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3274;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3275;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3275;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3276;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3276;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3277;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3277;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3278;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3278;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3279;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3279;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3280;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3280;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3281;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3281;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3282;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3282;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3283;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3283;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3284;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3284;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3285;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3285;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3286;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3286;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3287;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3287;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3288;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3288;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3289;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3289;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3290;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3290;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3291;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3291;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3292;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3292;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3293;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3293;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3294;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3294;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3295;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3295;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3296;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3296;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3297;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3297;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3298;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3298;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3299;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3299;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3300;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3300;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3301;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3301;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3302;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3302;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3303;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3303;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3304;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3304;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3305;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3305;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3306;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3306;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3307;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3307;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3308;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3308;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3309;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3309;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3310;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3310;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3311;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3311;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3312;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3312;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3313;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3313;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3314;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3314;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3315;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3315;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3316;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3316;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3317;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3317;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3318;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3318;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3319;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3319;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3320;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3320;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3321;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3321;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3322;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3322;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3323;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3323;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3324;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3324;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3325;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3325;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3326;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3326;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3327;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3327;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3328;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3328;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3329;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3329;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3330;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3330;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3331;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3331;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3332;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3332;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3333;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3333;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3334;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3334;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3335;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3335;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3336;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3336;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3337;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3337;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3338;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3338;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3339;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3339;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3340;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3340;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3341;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3341;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3342;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3342;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3343;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3343;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3344;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3344;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3345;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3345;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3346;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3346;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3347;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3347;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3348;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3348;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3349;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3349;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3350;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3350;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3351;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3351;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3352;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3352;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3353;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3353;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3354;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3354;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3355;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3355;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3356;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3356;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3357;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3357;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3358;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3358;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3359;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3359;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3360;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3360;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3361;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3361;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3362;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3362;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3363;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3363;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3364;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3364;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3365;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3365;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3366;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3366;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3367;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3367;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3368;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3368;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3369;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3369;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3370;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3370;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3371;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3371;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3372;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3372;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3373;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3373;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3374;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3374;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3375;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3375;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3376;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3376;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3377;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3377;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3378;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3378;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3379;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3379;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3380;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3380;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3381;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3381;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3382;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3382;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3383;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3383;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3384;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3384;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3385;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3385;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3386;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3386;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3387;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3387;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3388;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3388;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3389;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3389;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3390;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3390;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3391;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3391;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3392;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3392;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3393;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3393;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3394;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3394;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3395;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3395;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3396;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3396;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3397;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3397;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3398;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3398;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3399;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3399;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3400;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3400;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3401;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3401;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3402;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3402;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3403;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3403;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3404;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3404;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3405;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3405;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3406;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3406;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3407;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3407;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3408;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3408;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3409;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3409;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3410;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3410;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3411;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3411;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3412;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3412;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3413;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3413;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3414;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3414;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3415;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3415;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3416;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3416;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3417;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3417;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3418;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3418;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3419;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3419;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3420;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3420;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3421;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3421;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3422;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3422;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3423;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3423;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3424;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3424;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3425;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3425;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3426;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3426;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3427;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3427;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3428;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3428;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3429;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3429;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3430;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #3430;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3431;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3431;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3432;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3432;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3433;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3433;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3434;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3434;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3435;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3435;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3436;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3436;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3437;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3437;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3438;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3438;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3439;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3439;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3440;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3440;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3441;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3441;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3442;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3442;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3443;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3443;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3444;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3444;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3445;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3445;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3446;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3446;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3447;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3447;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3448;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3448;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3449;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3449;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3450;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3450;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3451;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3451;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3452;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3452;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3453;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3453;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3454;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3454;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3455;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3455;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3456;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3456;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3457;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3457;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3458;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3458;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3459;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3459;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3460;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3460;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3461;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3461;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3462;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3462;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3463;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3463;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3464;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3464;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3465;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3465;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3466;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3466;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3467;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3467;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3468;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3468;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3469;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3469;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3470;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3470;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3471;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3471;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3472;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3472;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3473;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3473;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3474;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3474;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3475;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3475;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3476;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3476;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3477;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3477;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3478;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3478;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3479;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3479;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3480;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3480;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3481;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3481;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3482;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3482;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3483;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3483;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3484;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3484;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3485;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3485;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3486;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3486;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3487;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3487;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3488;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3488;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3489;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3489;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3490;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3490;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3491;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3491;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3492;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3492;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3493;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3493;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3494;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3494;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3495;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3495;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3496;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3496;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3497;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3497;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3498;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3498;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3499;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3499;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3500;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3500;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3501;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3501;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3502;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3502;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3503;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3503;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3504;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3504;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3505;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3505;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3506;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3506;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3507;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3507;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3508;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3508;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3509;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3509;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3510;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3510;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3511;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3511;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3512;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3512;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3513;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3513;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3514;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3514;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3515;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3515;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3516;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3516;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3517;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3517;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3518;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3518;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3519;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3519;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3520;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3520;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3521;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3521;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3522;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3522;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3523;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3523;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3524;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3524;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3525;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3525;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3526;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3526;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3527;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3527;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3528;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3528;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3529;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3529;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3530;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3530;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3531;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3531;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3532;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3532;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3533;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3533;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3534;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3534;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3535;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3535;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3536;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3536;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3537;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3537;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3538;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3538;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3539;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3539;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3540;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3540;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3541;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3541;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3542;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3542;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3543;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3543;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3544;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3544;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3545;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3545;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3546;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3546;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3547;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3547;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3548;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3548;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3549;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3549;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3550;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3550;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3551;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3551;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3552;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3552;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3553;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3553;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3554;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3554;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3555;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3555;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3556;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3556;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3557;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3557;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3558;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3558;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3559;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3559;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3560;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3560;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3561;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3561;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3562;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3562;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3563;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3563;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3564;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3564;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3565;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3565;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3566;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3566;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3567;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3567;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3568;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3568;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3569;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3569;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3570;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3570;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3571;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3571;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3572;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3572;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3573;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3573;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3574;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3574;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3575;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3575;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3576;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3576;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3577;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3577;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3578;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3578;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3579;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3579;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3580;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3580;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3581;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3581;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3582;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3582;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3583;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3583;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3584;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3584;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3585;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3585;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3586;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3586;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3587;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3587;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3588;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3588;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3589;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3589;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3590;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3590;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3591;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3591;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3592;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3592;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3593;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3593;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3594;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3594;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3595;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3595;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3596;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3596;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3597;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3597;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3598;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3598;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3599;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3599;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3600;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3600;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3601;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3601;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3602;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3602;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3603;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3603;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3604;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3604;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3605;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3605;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3606;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3606;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3607;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3607;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3608;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3608;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3609;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3609;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3610;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3610;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3611;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3611;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3612;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3612;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3613;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3613;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3614;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3614;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3615;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3615;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3616;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3616;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3617;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3617;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3618;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3618;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3619;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3619;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3620;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3620;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3621;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3621;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3622;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3622;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3623;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3623;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3624;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3624;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3625;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3625;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3626;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3626;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3627;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3627;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3628;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3628;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3629;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3629;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3630;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3630;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3631;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3631;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3632;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3632;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3637;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3637;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3638;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3638;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3639;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3639;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3640;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3640;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3641;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3641;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3643;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3643;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3644;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3644;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3646;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3646;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3647;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3647;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3648;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3648;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3649;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3649;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3650;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3650;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3651;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3651;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3652;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3652;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3653;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3653;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3654;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3654;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3655;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3655;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3656;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3656;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3657;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3657;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3658;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3658;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3659;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3659;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3660;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3660;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3662;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3662;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3663;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3663;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3664;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3664;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3665;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3665;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3666;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3666;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3667;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3667;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3668;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3668;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3669;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3669;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3670;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3670;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3671;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3671;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3672;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3672;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3673;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3673;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3674;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3674;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3675;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3675;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3676;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3676;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3677;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3677;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3679;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3679;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3680;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3680;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3681;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3681;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3682;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3682;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3683;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3683;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3684;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3684;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3685;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3685;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3686;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3686;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3687;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3687;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3688;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3688;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3689;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3689;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3690;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3690;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3692;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3692;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3693;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3693;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3694;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3694;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3695;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3695;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3697;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3697;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3699;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3699;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3700;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3700;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3701;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3701;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3702;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3702;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3703;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3703;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3706;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3706;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3707;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3707;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3708;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3708;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3709;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3709;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3710;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3710;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3711;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3711;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3712;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3712;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3713;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3713;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3714;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3714;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3715;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3715;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3716;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3716;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3717;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3717;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3718;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3718;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3719;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3719;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3720;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3720;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3721;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3721;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3722;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3722;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3723;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3723;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3724;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3724;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3725;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3725;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3726;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3726;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3727;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3727;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3728;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3728;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3729;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3729;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3730;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3730;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3731;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3731;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3732;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3732;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3733;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3733;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3734;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3734;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3735;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3735;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3736;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3736;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3737;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3737;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3738;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3738;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3739;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3739;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3740;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3740;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3741;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3741;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3742;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3742;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3743;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3743;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3744;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3744;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3745;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3745;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3746;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3746;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3747;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3747;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3748;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3748;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3749;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3749;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3750;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3750;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3751;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3751;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3752;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3752;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3753;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3753;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3754;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3754;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3755;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3755;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3756;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3756;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3757;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3757;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3758;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3758;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3759;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3759;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3760;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3760;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3761;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3761;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3762;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3762;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3763;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3763;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3764;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3764;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3765;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3765;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3766;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3766;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3767;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3767;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3768;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3768;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3769;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3769;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3770;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3770;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3771;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3771;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3772;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3772;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3773;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3773;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3774;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3774;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3775;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3775;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3776;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3776;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3777;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3777;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3778;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3778;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3779;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3779;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3780;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3780;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3781;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3781;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3782;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3782;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3783;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3783;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3784;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3784;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3785;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3785;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3786;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3786;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3787;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3787;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3788;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3788;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3789;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3789;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3790;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3790;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3791;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3791;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3792;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3792;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3793;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3793;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3794;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3794;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3795;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3795;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3796;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3796;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3797;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3797;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3798;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3798;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3799;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3799;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3800;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3800;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3801;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3801;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3802;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3802;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3803;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3803;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3804;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3804;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3805;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3805;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3806;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3806;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3807;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3807;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3808;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3808;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3809;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3809;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3810;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3810;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3811;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3811;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3812;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3812;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3813;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3813;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3814;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3814;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3815;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3815;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3816;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3816;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3817;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3817;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3818;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3818;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3819;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3819;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3820;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3820;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3821;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3821;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3822;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3822;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3823;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3823;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3824;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3824;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3825;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3825;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3826;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3826;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3827;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3827;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3828;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3828;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3829;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3829;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3830;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3830;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3832;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3832;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3833;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3833;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3834;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3834;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3836;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3836;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3837;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3837;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3838;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3838;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3839;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3839;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3840;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3840;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3841;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3841;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3842;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3842;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3844;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3844;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3846;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3846;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3847;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3847;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3848;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3848;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3849;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3849;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3850;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3850;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3851;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3851;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3852;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3852;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3853;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3853;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3854;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3854;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3855;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3855;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3856;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3856;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3857;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3857;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3859;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3859;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3861;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3861;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3863;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3863;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3868;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3868;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3870;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3870;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3871;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3871;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3872;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3872;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3873;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3873;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3874;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3874;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3875;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3875;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3876;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3876;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3877;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3877;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3878;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3878;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3881;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3881;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3882;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3882;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3883;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3883;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3884;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3884;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3885;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3885;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3886;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3886;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3887;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3887;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3888;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3888;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3892;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3892;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3893;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3893;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3897;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3897;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3898;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3898;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3899;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3899;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3900;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3900;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3901;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3901;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3902;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3902;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3903;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3903;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3904;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3904;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3905;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3905;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3906;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3906;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3907;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3907;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3908;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3908;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3909;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3909;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3910;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3910;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3911;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3911;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3912;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3912;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3913;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3913;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3914;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3914;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3915;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3915;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3916;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3916;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3918;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3918;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3920;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3920;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3922;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3922;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3924;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3924;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3925;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3925;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3926;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3926;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3927;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3927;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3928;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3928;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3929;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3929;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3930;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3930;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3931;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3931;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3932;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3932;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3933;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3933;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3934;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3934;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3935;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3935;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3936;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3936;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3937;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3937;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3938;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3938;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3939;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3939;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3940;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3940;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3941;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3941;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3942;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3942;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3943;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3943;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3944;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3944;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3945;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3945;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3946;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3946;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3947;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3947;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3948;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3948;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3949;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3949;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3950;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3950;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3951;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3951;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3952;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3952;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3953;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3953;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3955;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3955;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3956;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3956;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3957;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3957;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3958;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3958;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3959;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3959;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3960;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3960;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3961;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3961;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3962;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3962;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3963;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3963;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3964;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3964;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3965;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3965;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3966;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3966;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3967;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3967;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3968;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3968;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3969;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3969;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3970;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3970;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3971;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3971;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3972;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3972;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3973;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3973;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3974;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3974;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3975;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3975;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3976;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3976;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3977;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #3977;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3978;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3978;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3979;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3979;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3980;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3980;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3981;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3981;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3982;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3982;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3983;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3983;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3984;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3984;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3985;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3985;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3986;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3986;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3987;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3987;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3988;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3988;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3989;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #3989;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3990;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3990;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3991;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3991;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3992;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3992;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3993;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3993;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3994;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3994;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3995;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3995;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3996;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3996;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3997;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3997;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3998;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #3998;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3999;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #3999;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4000;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4000;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4001;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4001;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4002;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4002;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4003;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4003;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4004;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4004;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4005;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4005;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4006;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4006;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4007;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4007;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4008;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4008;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4009;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4009;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4010;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4010;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4011;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4011;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4012;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4012;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4013;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4013;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4014;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4014;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4015;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4015;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4016;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4016;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4017;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4017;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4018;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4018;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4019;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4019;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4020;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4020;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4021;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4021;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4022;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4022;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4023;4
#I  Order: 2^8
#I  Nuclear rank: 5
#I  2-multiplicator rank: 11
#I  Group [grp] #4023;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4024;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4024;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4025;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4025;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4026;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4026;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4027;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4027;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4028;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4028;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4029;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4029;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4030;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4030;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4031;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4031;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4032;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4032;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4033;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4033;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4034;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4034;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4035;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4035;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4036;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4036;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4037;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4037;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4038;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4038;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4039;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4039;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4040;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4040;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4041;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4041;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4042;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4042;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4043;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4043;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4044;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4044;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4045;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4045;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4046;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4046;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4047;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4047;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4048;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4048;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4049;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4049;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4050;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4050;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4051;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4051;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4052;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4052;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4053;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4053;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4054;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4054;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4055;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4055;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4056;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4056;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4057;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4057;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4058;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4058;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4059;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4059;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4060;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4060;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4061;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4061;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4062;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4062;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4063;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4063;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4064;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4064;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4065;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4065;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4066;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4066;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4067;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4067;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4068;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4068;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4069;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4069;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4070;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4070;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4071;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4071;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4072;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4072;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4073;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4073;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4074;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4074;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4075;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4075;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4076;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4076;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4077;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4077;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4078;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4078;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4079;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4079;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4080;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4080;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4081;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4081;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4082;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4082;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4083;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4083;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4084;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4084;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4085;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4085;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4086;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4086;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4087;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4087;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4088;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4088;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4089;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4089;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4090;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4090;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4091;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4091;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4092;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4092;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4093;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4093;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4094;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4094;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4095;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4095;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4096;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4096;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4097;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4097;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4098;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4098;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4099;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4099;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4100;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4100;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4101;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4101;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4102;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4102;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4103;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4103;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4104;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4104;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4105;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4105;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4106;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4106;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4107;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4107;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4108;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4108;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4109;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4109;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4110;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4110;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4111;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4111;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4112;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4112;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4113;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4113;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4114;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4114;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4115;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4115;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4116;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4116;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4117;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4117;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4118;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4118;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4119;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4119;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4120;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4120;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4121;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4121;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4122;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4122;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4123;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4123;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4124;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4124;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4125;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4125;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4126;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4126;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4127;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4127;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4128;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4128;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4129;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4129;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4130;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4130;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4131;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4131;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4132;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4132;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4133;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4133;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4134;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4134;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4135;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4135;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4136;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4136;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4137;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4137;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4138;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4138;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4139;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4139;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4140;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4140;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4141;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4141;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4142;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4142;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4143;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4143;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4144;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4144;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4145;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4145;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4146;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4146;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4147;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4147;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4148;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4148;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4149;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4149;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4150;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4150;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4151;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4151;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4152;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4152;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4153;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4153;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4154;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4154;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4155;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4155;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4156;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4156;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4157;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4157;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4158;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4158;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4159;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4159;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4160;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4160;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4161;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4161;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4162;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4162;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4163;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4163;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4164;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4164;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4165;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4165;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4167;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4167;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4168;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4168;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4169;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4169;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4170;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4170;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4171;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4171;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4172;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4172;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4173;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4173;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4174;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4174;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4175;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4175;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4176;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4176;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4177;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4177;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4180;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4180;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4181;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4181;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4182;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4182;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4183;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4183;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4184;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4184;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4185;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4185;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4186;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4186;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4188;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4188;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4189;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4189;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4190;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4190;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4191;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4191;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4192;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4192;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4193;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4193;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4194;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4194;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4195;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4195;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4196;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4196;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4197;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4197;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4198;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4198;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4199;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4199;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4200;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4200;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4201;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4201;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4202;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4202;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4203;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4203;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4204;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4204;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4205;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4205;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4209;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4209;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4210;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4210;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4211;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4211;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4212;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4212;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4214;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4214;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4215;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4215;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4216;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4216;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4217;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4217;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4218;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4218;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4219;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4219;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4221;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4221;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4222;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4222;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4224;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4224;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4225;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4225;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4226;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4226;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4227;4
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 7
#I  Group [grp] #4227;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4228;4
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 8
#I  Group [grp] #4228;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4229;4
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 9
#I  Group [grp] #4229;4 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4230;4
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 10
#I  Group [grp] #4230;4 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^7 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 3
#I  **************************************************
#I  Starting group: [grp] #7;2 #8;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #7;2 #8;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #9;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #7;2 #9;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #10;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #10;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #11;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #12;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #13;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #14;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #15;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #16;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #30;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #30;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;2 #31;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #7;2 #31;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 11
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #8;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #8;2 #14;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #8;2 #27;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #27;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #28;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #28;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #29;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #29;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #30;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #30;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #31;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #31;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #32;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #32;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #33;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #33;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #34;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #34;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #35;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #35;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #36;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #36;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #37;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #37;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #38;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #38;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #39;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #39;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #40;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #40;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #42;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #42;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #43;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #43;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #44;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #44;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #46;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #46;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #47;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #47;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #57;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #57;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #61;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #61;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #62;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #62;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #65;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #65;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #76;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #76;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #80;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #80;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #81;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #81;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #84;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #84;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #95;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #95;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #96;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #96;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #97;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #97;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #98;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #98;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #99;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #99;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #100;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #100;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #101;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #101;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #102;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #102;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #103;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #103;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #104;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #104;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #108;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #108;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #119;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #119;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #134;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #134;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #149;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #149;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #171;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #171;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #172;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #172;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #174;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #174;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #175;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #175;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #178;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #178;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #179;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #179;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #182;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #182;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #183;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #183;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #186;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #186;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #195;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #195;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #196;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #196;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #199;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #199;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #200;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #200;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #203;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #203;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #208;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #208;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #209;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #209;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #212;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #212;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #213;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #213;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #216;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #216;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #218;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #218;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #221;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #221;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #222;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #222;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #226;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #226;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #231;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #231;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #236;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #236;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #239;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #239;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #247;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #247;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #252;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #252;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #267;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #267;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #277;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #277;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #290;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #290;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #309;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #309;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #319;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #8;2 #319;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #320;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #320;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #321;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #321;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #322;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #322;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #327;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #327;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #328;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #328;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #329;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #329;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #330;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #330;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #335;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #335;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #346;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #346;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #347;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #347;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #354;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #354;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #358;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #358;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #380;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #380;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #396;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #396;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #399;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #399;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #401;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #401;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #404;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #404;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #416;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #416;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #436;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #436;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #439;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #439;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #441;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #441;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #8;2 #444;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #8;2 #444;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #9;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 3
#I  **************************************************
#I  Starting group: [grp] #9;2 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #9;2 #16;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #9;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #17;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #9;2 #17;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #18;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #19;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #19;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #20;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #20;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #21;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #21;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #22;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #22;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #23;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #23;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #29;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #29;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #30;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #30;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #40;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #40;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #41;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #41;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #51;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #51;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #52;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #52;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #59;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #59;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #9;2 #61;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #9;2 #61;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #10;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 9
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #10;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 5
#I  **************************************************
#I  Starting group: [grp] #11;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 15
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #11;2 #9;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 22
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #11;2 #15;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 6
#I  **************************************************
#I  Starting group: [grp] #11;2 #33;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #11;2 #33;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #34;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #34;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #35;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #35;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #36;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #36;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #37;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #37;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #38;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #38;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #39;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #11;2 #39;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #40;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #40;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #41;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #41;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #42;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #42;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #43;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #43;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #44;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #44;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #45;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #45;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #46;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #46;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #47;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #47;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #48;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #48;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #49;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #49;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #50;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #50;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #57;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #57;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #74;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #74;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #80;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #80;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #91;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #91;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #98;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #98;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #119;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #119;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #126;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #126;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #147;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #147;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #153;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #153;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #166;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #166;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #172;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #172;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #185;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #185;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #192;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #192;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #205;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #205;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #212;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #212;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #229;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #229;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #11;2 #239;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #11;2 #239;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #12;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #13;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #14;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #15;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #16;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #17;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 33
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #17;2 #28;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  # of immediate descendants of order 2^8 is 30
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #17;2 #34;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #17;2 #34;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #35;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #35;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #36;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #36;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #37;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #37;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #38;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #38;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #39;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #39;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #40;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #40;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #41;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #41;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #42;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #42;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #48;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #17;2 #48;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #49;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #49;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #50;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #50;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #51;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #51;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #52;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #52;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #53;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #53;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #54;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #54;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #55;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #55;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #56;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #56;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #62;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #62;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #72;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #72;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #73;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #73;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #74;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #74;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #75;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #75;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #76;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #76;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #82;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #82;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #83;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #83;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #84;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #84;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #85;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #85;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #86;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #86;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #92;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #92;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #102;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #102;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #103;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #103;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #104;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #104;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #105;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #105;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #106;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #106;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #107;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #107;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #108;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #108;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #118;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #118;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #126;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #126;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #132;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #132;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #133;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #133;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #134;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #17;2 #134;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #135;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #135;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #136;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #136;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #137;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #137;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #138;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #138;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #139;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #139;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #142;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #142;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #143;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #143;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #150;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #150;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #158;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #158;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #164;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #164;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #165;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #165;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #166;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #166;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #167;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #167;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #168;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #168;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #169;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #169;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #170;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #170;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #171;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #171;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #192;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #192;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #193;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #193;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #194;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #194;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #195;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #195;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #196;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #196;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #197;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #197;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #198;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #198;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #199;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #199;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #200;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #200;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #201;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #201;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #224;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #224;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #232;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #232;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #240;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #240;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #248;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #248;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #320;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #320;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #327;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #327;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #334;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #334;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #341;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #341;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #348;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #348;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #414;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 11
#I  Group [grp] #17;2 #414;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #415;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #415;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #421;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #421;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #426;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #426;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #428;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #428;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #435;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #435;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #438;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #438;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #444;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #444;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #451;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #451;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #456;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #456;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #464;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #464;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #560;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #560;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #570;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #570;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #580;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #580;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #590;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #590;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #602;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #602;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #614;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #614;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #686;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #686;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #696;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #696;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #706;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #706;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #716;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #716;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #726;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #726;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #738;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #738;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #810;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #810;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #822;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #822;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #832;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #832;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #842;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #842;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #852;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #852;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #17;2 #862;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #17;2 #862;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #18;2 #20;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 23
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #18;2 #42;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #42;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #43;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #18;2 #43;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #44;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #44;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #45;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #45;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #47;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #47;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #48;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #48;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #50;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #50;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #52;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #52;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #56;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #56;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #60;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #60;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #65;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #65;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #69;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #69;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #74;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #74;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #75;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #75;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #76;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #76;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #77;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #77;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #106;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #106;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #107;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #107;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #130;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #130;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #131;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #131;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #132;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #132;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #133;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #133;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #134;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #134;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #174;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #174;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #175;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #175;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #199;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #199;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #202;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #202;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #205;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #205;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #211;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #211;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #215;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #215;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #220;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #220;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #330;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #330;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #336;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #336;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #342;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #342;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #348;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #348;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #354;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #354;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #18;2 #360;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #18;2 #360;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #19;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 15
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #19;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 4
#I  **************************************************
#I  Starting group: [grp] #20;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 15
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #20;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 6
#I  **************************************************
#I  Starting group: [grp] #21;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 28
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #21;2 #10;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 6
#I  **************************************************
#I  Starting group: [grp] #22;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 15
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #26;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #26;2 #16;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #20;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #20;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #24;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #26;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #25;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #25;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #26;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #26;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #37;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #37;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #38;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #38;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #39;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #39;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #52;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #52;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #53;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #53;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #54;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #54;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #83;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #83;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #84;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #84;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #26;2 #85;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #26;2 #85;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #27;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #28;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #30;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #31;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 21
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #32;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #33;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 19
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #41;2 #16;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  # of immediate descendants of order 2^8 is 31
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #41;2 #31;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #31;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #32;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #32;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #33;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #33;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #34;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #34;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #35;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #35;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #36;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #36;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #37;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #37;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #38;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #38;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #159;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #159;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #160;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #160;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #161;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #161;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #162;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #162;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #163;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #163;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #164;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #164;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #165;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #165;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #243;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 10
#I  Group [grp] #41;2 #243;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #244;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #244;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #245;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #245;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #246;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #246;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #247;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #247;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #248;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #248;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #249;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #249;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #250;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #250;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #41;2 #251;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #41;2 #251;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #42;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 40
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #43;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 24
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #44;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 20
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #45;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 13
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #46;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  # of immediate descendants of order 2^8 is 13
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #61;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #61;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #61;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #61;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #61;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #61;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #62;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #62;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #62;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #62;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #62;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #62;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #63;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #63;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #63;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #63;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #63;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #63;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #64;3 #36;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #64;3 #36;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #65;3 #24;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #65;3 #24;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #66;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #66;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #66;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #66;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #66;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #66;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #66;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #66;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #66;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #66;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #66;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #66;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #67;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #67;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #67;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #67;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #67;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #67;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #67;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #67;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #68;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #68;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #68;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #68;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #69;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #69;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #69;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #69;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #70;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #70;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #70;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #70;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #70;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #70;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #70;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #70;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #71;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #71;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #71;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #71;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #72;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #72;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #72;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #72;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #31;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #31;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #73;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #73;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #74;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #74;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #74;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #74;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #74;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #74;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #74;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #74;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #74;3 #32;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #74;3 #32;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #74;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #74;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #75;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #75;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #75;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #75;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #75;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #75;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #75;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #75;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #75;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #75;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #20;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #20;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #76;3 #26;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #76;3 #26;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #77;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #77;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #77;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #77;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #77;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #77;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #77;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #77;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #78;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #78;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #79;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #79;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #79;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #79;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #80;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #80;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #80;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #80;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #80;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #80;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #80;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #80;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #80;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #80;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #81;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #81;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #81;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #81;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #81;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #81;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #81;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #81;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #81;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #81;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #82;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #82;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #82;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #82;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #82;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #82;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #82;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #82;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #82;3 #38;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #82;3 #38;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #82;3 #56;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #82;3 #56;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #83;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #83;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #83;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #83;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #83;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #83;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #83;3 #65;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #83;3 #65;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #83;3 #74;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #83;3 #74;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #83;3 #89;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #83;3 #89;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #84;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #84;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #84;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #84;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #84;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #84;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #84;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #84;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #84;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #84;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #84;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #84;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #85;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #85;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #85;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #85;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #85;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #85;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #85;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #85;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #85;3 #39;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #85;3 #39;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #85;3 #42;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #85;3 #42;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #86;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #86;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #86;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #86;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #86;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #86;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #86;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #86;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #86;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #86;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #87;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #87;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #87;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #87;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #87;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #87;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #87;3 #26;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #87;3 #26;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #87;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #87;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #39;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #39;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #42;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #42;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #88;3 #71;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #88;3 #71;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #89;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #89;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #89;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #89;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #89;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #89;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #89;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #89;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #90;3 #34;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #90;3 #34;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #91;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #91;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #91;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #91;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #92;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #92;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #92;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #92;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #92;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #92;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #92;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #92;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #28;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #28;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #35;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #35;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #93;3 #47;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #93;3 #47;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #94;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #94;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #94;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #94;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #95;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #95;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #95;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #95;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #95;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #95;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #95;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #95;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #96;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #96;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #96;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #96;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #96;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #96;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #96;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #96;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #96;3 #42;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #96;3 #42;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #97;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #97;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #97;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #97;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #97;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #97;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #97;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #97;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #97;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #97;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #98;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #98;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #98;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #98;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #98;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #98;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #98;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #98;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #98;3 #20;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #98;3 #20;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #99;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #99;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #99;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #99;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #99;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #99;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #99;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #99;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #100;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #100;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #101;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #101;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #102;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #102;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #102;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #102;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #102;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #102;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #102;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #102;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #102;3 #26;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #102;3 #26;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #103;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #103;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #103;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #103;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #103;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #103;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #104;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #104;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #104;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #104;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #104;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #104;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #104;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #104;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #105;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #105;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #106;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #106;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #106;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #106;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #106;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #106;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #106;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #106;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #106;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #106;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #107;3 #41;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #107;3 #41;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #108;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #108;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #108;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #108;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #109;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #109;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #109;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #109;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #110;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #110;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #110;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #110;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #111;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #111;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #111;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #111;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #111;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #111;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #112;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #112;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #112;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #112;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #112;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #112;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #113;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #113;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #113;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #113;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #113;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #113;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #113;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #113;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #114;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #114;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #114;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #114;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #115;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #115;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #115;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #115;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #116;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #116;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #116;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #116;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #117;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #117;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #117;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #117;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #117;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #117;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #117;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #117;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #118;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #118;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #118;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #118;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #118;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #118;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #119;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #119;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #119;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #119;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #119;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #119;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #120;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #120;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #120;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #120;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #120;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #120;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #120;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #120;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #121;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #121;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #121;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #121;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #121;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #121;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #122;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #122;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #122;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #122;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #122;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #122;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #122;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #122;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #123;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #123;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #123;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #123;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #123;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #123;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #124;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #124;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #124;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #124;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #124;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #124;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #124;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #124;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #125;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #125;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #126;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #126;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #127;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #127;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #127;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #127;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #128;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #128;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #129;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #129;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #129;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #129;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #130;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #130;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #131;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #131;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #132;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #132;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #133;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #133;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #133;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #133;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #134;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #134;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #134;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #134;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #134;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #134;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #134;3 #28;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #134;3 #28;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #134;3 #38;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #134;3 #38;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #135;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #135;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #135;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #135;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #135;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #135;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #136;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #136;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #136;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #136;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #136;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #136;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #137;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #137;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #137;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #137;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #137;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #137;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #138;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #138;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #138;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #138;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #138;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #138;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #138;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #138;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #139;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #139;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #139;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #139;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #140;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #140;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #140;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #140;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #140;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #140;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #140;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #140;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #141;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #141;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #141;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #141;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #142;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #142;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #142;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #142;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #142;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #142;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #143;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #143;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #143;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #143;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #144;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #144;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #144;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #144;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #144;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #144;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #144;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #144;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #144;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #144;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #145;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #145;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #145;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #145;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #145;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #145;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #146;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #146;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #146;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #146;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #146;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #146;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #147;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #147;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #147;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #147;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #147;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #147;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #148;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #148;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #148;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #148;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #148;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #148;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #148;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #148;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #149;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #149;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #149;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #149;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #150;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #150;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #150;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #150;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #150;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #150;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #150;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #150;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #151;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #151;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #151;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #151;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #152;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #152;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #152;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #152;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #153;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #153;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #153;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #153;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #154;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #154;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #154;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #154;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #154;3 #39;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #154;3 #39;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #155;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #155;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #155;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #155;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #155;3 #39;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #155;3 #39;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #156;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #156;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #156;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #156;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #156;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #156;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #157;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #157;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #157;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #157;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #157;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #157;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #157;3 #26;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #157;3 #26;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #157;3 #30;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #157;3 #30;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #158;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #158;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #158;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #158;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #158;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #158;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #159;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #159;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #159;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #159;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #159;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #159;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #159;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #159;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #160;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #160;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #160;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #160;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #160;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #160;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #161;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #161;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #161;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #161;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #161;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #161;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #162;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #162;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #162;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #162;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #162;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #162;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #163;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #163;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #163;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #163;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #163;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #163;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #164;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #164;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #165;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #165;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #165;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #165;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #166;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #166;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #167;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #167;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #168;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #168;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #168;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #168;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #168;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #168;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #169;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #169;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #170;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #170;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #171;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #171;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #171;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #171;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #172;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #172;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #173;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #173;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #173;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #173;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #174;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #174;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #175;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #175;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #175;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #175;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #175;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #175;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #176;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #176;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #177;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #177;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #177;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #177;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #178;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #178;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #179;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #179;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #180;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #180;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #180;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #180;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #180;3 #35;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #180;3 #35;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #181;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #181;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #181;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #181;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #182;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #182;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #182;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #182;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #182;3 #32;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #182;3 #32;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #182;3 #44;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #182;3 #44;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #182;3 #46;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #182;3 #46;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #183;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #183;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #183;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #183;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #183;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #183;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #184;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #184;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #184;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #184;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #184;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #184;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #185;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #185;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #185;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #185;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #186;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #186;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #186;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #186;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #186;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #186;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #186;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #186;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #186;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #186;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #187;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #187;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #187;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #187;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #187;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #187;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #187;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #187;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #187;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #187;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #187;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #187;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #188;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #188;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #188;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #188;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #188;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #188;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #188;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #188;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #188;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #188;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #189;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #189;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #189;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #189;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #189;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #189;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #190;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #190;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #190;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #190;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #190;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #190;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #191;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #191;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #191;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #191;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #191;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #191;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #191;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #191;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #191;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #191;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #191;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #191;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #192;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #192;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #192;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #192;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #192;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #192;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #192;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #192;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #193;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #193;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #193;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #193;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #193;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #193;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #194;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #194;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #194;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #194;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #195;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #195;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #195;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #195;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #195;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #195;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #199;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #199;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #199;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #199;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #199;3 #65;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 12
#I  Group [grp] #199;3 #65;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #200;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #200;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #200;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #200;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #200;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #200;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #201;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #201;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #201;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #201;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #201;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #201;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #201;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #201;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #201;3 #35;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #201;3 #35;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #201;3 #36;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #201;3 #36;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #202;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #202;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #202;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #202;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #202;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #202;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #202;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #202;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #202;3 #34;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #202;3 #34;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #35;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #35;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #69;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #69;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #203;3 #81;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #203;3 #81;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #204;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #204;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #204;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #204;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #204;3 #41;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #204;3 #41;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #204;3 #53;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #204;3 #53;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #204;3 #60;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #204;3 #60;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #205;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #205;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #205;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #205;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #205;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #205;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #205;3 #41;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #205;3 #41;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #205;3 #53;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #205;3 #53;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #206;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #206;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #206;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #206;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #206;3 #43;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #206;3 #43;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #206;3 #71;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #206;3 #71;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #207;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #207;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #207;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #207;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #207;3 #31;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #207;3 #31;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #208;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #208;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #208;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #208;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #208;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #208;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #208;3 #28;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #208;3 #28;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #209;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #209;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #209;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #209;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #209;3 #32;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #209;3 #32;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #210;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #210;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #210;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #210;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #210;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #210;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #211;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #211;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #211;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #211;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #211;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #211;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #212;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #212;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #212;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #212;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #212;3 #47;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #212;3 #47;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #213;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #213;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #213;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #213;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #213;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #213;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #213;3 #26;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #213;3 #26;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #214;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #214;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #214;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #214;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #214;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #214;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #214;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #214;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #214;3 #49;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #214;3 #49;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #214;3 #53;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #214;3 #53;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #215;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #215;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #215;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #215;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #216;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #216;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #216;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #216;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #217;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #217;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #217;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #217;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #218;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #218;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #218;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #218;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #218;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #218;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #218;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #218;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #219;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #219;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #219;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #219;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #220;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #220;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #220;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #220;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #220;3 #73;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #220;3 #73;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #220;3 #93;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #220;3 #93;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #221;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #221;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #221;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #221;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #222;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #222;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #222;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #222;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #223;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #223;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #223;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #223;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #223;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #223;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #223;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #223;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #224;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #224;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #224;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #224;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #224;3 #43;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #224;3 #43;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #225;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #225;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #225;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #225;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #226;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #226;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #226;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #226;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #226;3 #43;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #226;3 #43;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #227;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #227;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #227;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #227;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #227;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #227;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #227;3 #65;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #227;3 #65;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #228;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #228;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #228;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #228;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #228;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #228;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #228;3 #59;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #228;3 #59;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #229;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #229;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #229;3 #41;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #229;3 #41;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #229;3 #55;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #229;3 #55;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #229;3 #63;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #229;3 #63;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #230;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #230;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #230;3 #28;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #230;3 #28;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #230;3 #42;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #230;3 #42;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #231;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #231;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #231;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #231;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #231;3 #31;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #231;3 #31;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #231;3 #45;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #231;3 #45;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #231;3 #46;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #231;3 #46;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #232;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #232;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #232;3 #32;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #232;3 #32;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #232;3 #46;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #232;3 #46;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #233;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #233;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #233;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #233;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #234;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #234;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #234;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #234;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #235;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #235;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #235;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #235;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #236;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #236;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #236;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #236;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #236;3 #26;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #236;3 #26;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #237;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #237;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #237;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #237;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #238;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #238;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #238;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #238;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #238;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #238;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #238;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #238;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #239;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #239;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #239;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #239;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #239;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #239;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #239;3 #49;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #239;3 #49;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #240;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #240;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #240;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #240;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #241;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #241;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #241;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #241;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #241;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #241;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #242;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #242;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #242;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #242;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #243;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #243;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #243;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #243;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #243;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #243;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #243;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #243;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #244;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #244;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #244;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #244;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #245;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #245;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #245;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #245;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #245;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #245;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #245;3 #57;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #245;3 #57;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #246;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #246;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #246;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #246;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #247;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #247;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #247;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #247;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #248;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #248;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #248;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #248;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #248;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #248;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #249;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #249;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #249;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #249;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #250;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #250;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #250;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #250;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #250;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #250;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #250;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #250;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #251;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #251;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #251;3 #24;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #251;3 #24;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #252;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #252;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #252;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #252;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #252;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #252;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #252;3 #50;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #252;3 #50;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #253;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #253;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #253;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #253;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #253;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #253;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #253;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #253;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #254;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #254;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #254;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #254;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #255;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #255;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #255;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #255;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #256;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #256;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #256;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #256;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #257;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #257;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #257;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #257;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #257;3 #35;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #257;3 #35;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #258;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #258;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #258;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #258;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #259;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #259;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #259;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #259;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #260;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #260;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #260;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #260;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #260;3 #36;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #260;3 #36;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #260;3 #52;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #260;3 #52;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #261;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #261;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #261;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #261;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #262;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #262;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #262;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #262;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #263;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #263;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #263;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #263;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #264;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #264;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #264;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #264;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #264;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #264;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #264;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #264;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #265;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #265;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #265;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #265;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #266;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #266;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #266;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #266;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #267;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #267;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #267;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #267;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #268;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #268;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #268;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #268;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #268;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #268;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #269;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #269;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #269;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #269;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #270;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #270;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #270;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #270;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #271;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #271;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #271;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #271;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #272;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #272;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #272;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #272;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #273;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #273;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #273;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #273;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #274;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #274;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #274;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #274;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #275;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #275;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #275;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #275;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #276;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #276;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #276;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #276;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #276;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #276;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #276;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #276;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #277;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #277;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #277;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #277;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #278;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #278;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #278;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #278;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #279;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #279;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #279;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #279;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #279;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #279;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #280;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #280;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #280;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #280;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #280;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #280;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #280;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #280;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #281;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #281;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #281;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #281;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #281;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #281;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #281;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #281;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #282;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #282;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #282;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #282;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #282;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #282;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #282;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #282;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #283;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #283;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #283;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #283;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #284;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #284;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #285;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #285;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #285;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #285;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #286;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #286;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #286;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #286;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #287;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #287;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #287;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #287;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #288;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #288;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #288;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #288;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #292;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #292;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #292;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #292;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #294;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #294;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #294;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #294;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #295;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #295;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #295;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #295;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #296;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #296;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #296;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #296;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #297;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #297;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #297;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #297;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #297;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #297;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #297;3 #28;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #297;3 #28;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #298;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #298;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #298;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #298;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #299;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #299;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #299;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #299;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #300;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #300;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #302;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #302;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #308;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #308;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #315;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #315;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #317;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #317;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #317;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #317;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #330;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #330;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #330;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #330;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #330;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #330;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #330;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #330;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #331;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #331;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #331;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #331;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #331;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #331;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #331;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #331;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #332;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #332;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #332;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #332;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #332;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #332;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #332;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #332;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #333;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #333;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #333;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #333;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #333;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #333;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #333;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #333;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #334;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #334;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #334;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #334;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #334;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #334;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #334;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #334;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #335;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #335;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #335;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #335;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #335;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #335;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #335;3 #41;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #335;3 #41;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #336;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #336;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #336;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #336;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #336;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #336;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #336;3 #45;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #336;3 #45;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #337;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #337;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #337;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #337;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #337;3 #41;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #337;3 #41;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #338;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #338;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #338;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #338;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #338;3 #20;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #338;3 #20;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #338;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #338;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #339;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #339;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #339;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #339;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #340;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #340;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #340;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #340;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #340;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #340;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #341;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #341;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #341;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #341;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #342;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #342;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #342;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #342;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #343;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #343;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #343;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #343;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #343;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #343;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #344;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #344;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #344;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #344;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #345;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #345;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #346;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #346;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #346;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #346;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #347;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #347;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #348;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #348;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #348;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #348;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #349;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #349;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #349;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #349;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #350;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #350;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #351;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #351;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #352;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #352;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #353;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #353;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #353;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #353;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #354;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #354;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #355;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #355;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #355;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #355;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #356;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #356;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #357;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #357;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #357;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #357;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #358;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #358;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #358;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #358;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #359;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #359;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #360;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #360;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #360;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #360;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #361;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #361;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #361;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #361;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #362;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #362;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #362;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #362;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #363;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #363;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #363;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #363;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #364;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #364;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #364;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #364;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #375;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #375;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #375;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #375;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #375;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #375;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #375;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #375;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #376;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #376;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #376;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #376;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #377;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #377;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #377;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #377;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #377;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #377;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #377;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #377;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #378;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #378;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #378;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #378;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #378;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #378;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #379;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #379;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #379;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #379;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #380;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #380;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #380;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #380;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #387;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #387;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #387;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #387;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #388;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #388;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #388;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #388;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #394;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #394;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #394;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #394;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #395;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #395;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #395;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #395;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #395;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #395;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #395;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #395;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #396;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #396;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #396;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #396;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #396;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #396;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #397;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #397;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #397;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #397;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #398;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #398;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #398;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #398;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #399;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #399;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #399;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #399;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #399;3 #14;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #399;3 #14;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #400;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #400;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #400;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #400;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #405;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #405;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #405;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #405;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #405;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #405;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #406;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #406;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #406;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #406;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #407;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #407;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #407;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #407;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #409;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #409;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #409;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #409;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #409;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #409;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #409;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #409;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #409;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #409;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #410;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #410;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #410;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #410;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #411;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #411;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #411;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #411;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #412;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #412;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #412;3 #23;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #412;3 #23;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #413;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #413;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #413;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #413;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #413;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #413;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #414;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #414;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #414;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #414;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #415;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #415;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #415;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #415;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #416;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #416;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #416;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #416;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #417;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #417;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #417;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #417;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #418;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #418;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #418;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #418;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #419;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #419;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #419;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #419;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #419;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #419;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #420;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #420;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #420;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #420;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #420;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #420;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #421;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #421;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #421;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #421;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #421;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #421;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #422;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #422;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #422;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #422;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #423;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #423;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #423;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #423;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #424;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #424;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #424;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #424;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #425;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #425;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #425;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #425;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #426;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #426;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #426;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #426;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #427;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #427;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #428;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #428;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #428;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #428;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #429;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #429;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #429;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #429;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #430;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #430;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #430;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #430;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #431;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #431;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #431;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #431;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #432;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #432;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #432;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #432;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #432;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #432;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #433;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #433;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #433;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #433;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #434;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #434;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #434;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #434;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #435;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #435;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #436;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #436;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #437;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #437;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #438;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #438;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #452;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #452;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #453;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #453;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #454;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #454;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #455;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #455;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #456;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #456;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #457;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #457;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #458;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #458;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #459;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #459;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #460;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #460;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #464;3 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #464;3 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #464;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #464;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #465;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #465;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #465;3 #36;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #465;3 #36;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #466;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #466;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #467;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #467;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #467;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #467;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #467;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #467;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #467;3 #37;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #467;3 #37;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #468;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #468;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #468;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #468;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #469;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #469;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #469;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #469;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #470;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #470;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #470;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #470;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #471;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #471;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #472;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #472;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #472;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #472;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #473;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #473;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #474;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #474;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #474;3 #17;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #474;3 #17;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #474;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #474;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #474;3 #49;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #474;3 #49;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #475;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #475;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #475;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #475;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #476;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #476;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #477;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #477;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #477;3 #33;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #477;3 #33;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #478;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #478;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #478;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #478;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #479;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #479;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #480;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #480;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #480;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #480;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #481;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #481;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #481;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #481;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #482;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #482;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #482;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #482;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #483;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #483;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #483;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #483;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #484;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #484;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #484;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #484;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #485;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #485;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #485;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #485;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #486;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #486;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #486;3 #30;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #486;3 #30;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #487;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #487;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #487;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #487;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #488;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #488;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #488;3 #30;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #488;3 #30;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #489;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #489;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #489;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #489;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #489;3 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #489;3 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #490;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #490;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #490;3 #32;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #490;3 #32;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #491;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #491;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #491;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #491;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #492;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #492;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #492;3 #32;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #492;3 #32;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #493;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #493;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #493;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #493;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #493;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #493;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #493;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #493;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #494;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #494;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #495;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #495;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #495;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #495;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #496;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #496;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #497;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #497;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #497;3 #27;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #497;3 #27;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #498;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #498;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #499;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #499;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #499;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #499;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #500;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #500;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #501;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #501;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #502;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #502;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #502;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #502;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #503;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #503;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #504;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #504;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #504;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #504;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #504;3 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #504;3 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #504;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #504;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #505;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #505;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #505;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #505;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #506;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #506;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #507;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #507;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #508;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #508;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #509;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #509;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #510;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #510;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #511;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #511;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #512;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #512;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #513;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #513;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #513;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #513;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #514;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #514;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #515;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #515;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #516;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #516;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #517;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #517;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #518;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #518;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #519;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #519;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #520;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #520;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #520;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #520;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #520;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #520;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #521;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #521;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #522;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #522;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #523;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #523;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #524;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #524;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #524;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #524;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #524;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #524;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #525;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #525;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #525;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #525;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #526;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #526;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #526;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #526;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #527;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #527;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #528;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #528;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #528;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #528;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #528;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #528;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #529;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #529;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #530;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #530;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #608;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #608;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #608;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #608;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #609;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #609;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #610;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #610;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #611;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #611;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #612;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #612;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #613;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #613;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #614;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #614;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #615;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #615;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #616;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #616;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #617;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #617;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #618;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #618;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #619;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #619;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #619;3 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #619;3 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #642;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #642;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #642;3 #21;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #642;3 #21;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #642;3 #22;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 13
#I  Group [grp] #642;3 #22;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #643;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #643;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #643;3 #25;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #643;3 #25;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #643;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 11
#I  Group [grp] #643;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #644;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #644;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #644;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #644;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #644;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #644;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #644;3 #20;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #644;3 #20;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #645;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #645;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #645;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #645;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #645;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #645;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #645;3 #16;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #645;3 #16;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #645;3 #29;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #645;3 #29;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #645;3 #31;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #645;3 #31;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #646;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #646;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #646;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #646;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #646;3 #19;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #646;3 #19;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #647;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #647;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #647;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #647;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #647;3 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #647;3 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #648;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #648;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #648;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #648;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #649;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #649;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #649;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #649;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #649;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #649;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #650;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #650;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #650;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #650;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #650;3 #24;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #650;3 #24;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #651;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #651;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #651;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #651;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #651;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #651;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #651;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #651;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #652;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #652;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #652;3 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #652;3 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #653;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #653;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #653;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #653;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #653;3 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #653;3 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #654;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #654;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #654;3 #18;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #654;3 #18;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #655;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #655;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #656;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #656;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #656;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #656;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #657;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #657;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #658;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 8
#I  Group [grp] #658;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #663;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 10
#I  Group [grp] #663;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #664;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 9
#I  Group [grp] #664;3 #3;1 is an invalid starting group
#I  **************************************************
5473
gap> PqQuitAll();
gap> ##Example: "2gp-PG-e4-i" . . . based on: examples/pga_exp4
gap> ##All 2-generator exponent 4 2-groups
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 2);
1
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
gap> PqPGSupplyAutomorphisms(procId, [ [[0,1],
>                                    [1,1]],
> 
>                                   [[0,1],
>                                    [1,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 12,
>                                   PcgsAutomorphisms,
>                                   Exponent := 4);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 2^2
#I  Nuclear rank: 3
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^3 is 3
#I  # of immediate descendants of order 2^4 is 3
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^5 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 2^3
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2;1
#I  Order: 2^3
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #3;1
#I  Order: 2^3
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2
#I  Order: 2^4
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #4;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2
#I  Order: 2^4
#I  Nuclear rank: 1
#I  2-multiplicator rank: 2
#I  # of immediate descendants of order 2^5 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2
#I  Order: 2^4
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #6;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3
#I  Order: 2^5
#I  Nuclear rank: 2
#I  2-multiplicator rank: 2
#I  # of immediate descendants of order 2^6 is 1
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^7 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 2
#I  # of immediate descendants of order 2^6 is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 2
#I  # of immediate descendants of order 2^7 is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 3
#I  # of immediate descendants of order 2^9 is 3
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^10 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #5;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #2;1
#I  Order: 2^6
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #5;2 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #2;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #2;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #2;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #4;2
#I  Order: 2^9
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #7;3 #2;2 #4;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #5;2
#I  Order: 2^9
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #7;3 #2;2 #5;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #6;2
#I  Order: 2^9
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^10 is 2
#I  # of immediate descendants of order 2^11 is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #7;3
#I  Order: 2^10
#I  Nuclear rank: 2
#I  2-multiplicator rank: 2
#I  # of immediate descendants of order 2^11 is 1
#I  # of immediate descendants of order 2^12 is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #6;2 #1;1
#I  Order: 2^10
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #2;2 #6;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #6;2 #2;1
#I  Order: 2^10
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #2;2 #6;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #6;2 #3;2
#I  Order: 2^11
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #7;3 #2;2 #6;2 #3;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #6;2 #4;2
#I  Order: 2^11
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #7;3 #2;2 #6;2 #4;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #7;3 #1;1
#I  Order: 2^11
#I  Nuclear rank: 0
#I  2-multiplicator rank: 1
#I  Group [grp] #7;3 #2;2 #7;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #2;2 #7;3 #2;2
#I  Order: 2^12
#I  Nuclear rank: 0
#I  2-multiplicator rank: 2
#I  Group [grp] #7;3 #2;2 #7;3 #2;2 is an invalid starting group
26
gap> ##Example: "2gp-PG-i" . . . based on: examples/pga_2gp
gap> ##All descendants of C2 x C2 up to order 2^8
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 2);
2
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
gap> PqPGSupplyAutomorphisms(procId, [ [[0,1],
>                                    [1,1]],
> 
>                                   [[0,1],
>                                    [1,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 7,
>                                   OrderBound := 8,
>                                   PcgsAutomorphisms,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 2^2
#I  Nuclear rank: 3
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^3 is 3
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^4 is 3
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^5 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 2^3
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^4 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1
#I  Order: 2^3
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^4 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #4;2
#I  Order: 2^4
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^5 is 2
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2
#I  Order: 2^4
#I  Nuclear rank: 3
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^5 is 7
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^6 is 11
#I  # of capable immediate descendants is 10
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #6;2
#I  Order: 2^4
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^5 is 4
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3
#I  Order: 2^5
#I  Nuclear rank: 5
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^6 is 9
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^7 is 37
#I  # of capable immediate descendants is 28
#I  # of immediate descendants of order 2^8 is 37
#I  # of capable immediate descendants is 37
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1
#I  Order: 2^4
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^5 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1
#I  Order: 2^4
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^5 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #4;2 #2;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #4;2 #4;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^8 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^6 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #5;2 #5;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^6 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 5
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 8
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 7
#I  # of capable immediate descendants is 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 8
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 6
#I  **************************************************
#I  Starting group: [grp] #5;2 #11;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 12
#I  # of capable immediate descendants is 4
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 16
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #14;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 2^8 is 7
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #5;2 #17;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #18;2
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #19;3
#I  Order: 2^7
#I  Nuclear rank: 4
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #20;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #5;2 #21;3
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #6;2 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #2;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #6;2 #5;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 2^8 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2
#I  Order: 2^6
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #5;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #10;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #11;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #12;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #13;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #14;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #15;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #16;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #17;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #18;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #19;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #7;3 #20;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #21;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #23;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #25;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #26;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #27;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #29;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #30;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #31;2
#I  Order: 2^7
#I  Nuclear rank: 3
#I  2-multiplicator rank: 6
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #7;3 #32;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #7;3 #33;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #37;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #7;3 #38;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #39;2
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 14
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #40;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #41;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #44;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #46;2
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #47;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #47;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #48;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #48;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #49;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #49;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #50;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #50;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #51;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #51;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #52;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #52;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #53;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #53;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #54;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #54;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #55;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #55;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #56;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #56;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #57;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #57;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #58;3
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #58;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #59;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #59;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #60;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #60;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #61;3
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #61;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #62;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #62;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #63;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #63;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #64;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #64;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #65;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #65;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #66;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #66;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #67;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #67;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #68;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #68;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #69;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #69;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #70;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #70;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #71;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #71;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #72;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #72;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #73;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #73;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #74;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #74;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #75;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #75;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #76;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #76;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #77;3
#I  Order: 2^8
#I  Nuclear rank: 4
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #77;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #78;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #78;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #79;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #79;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #80;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #80;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #81;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #81;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #82;3
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #82;3 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #83;3
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #83;3 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1 #1;1
#I  Order: 2^5
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^6 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1 #2;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #3;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #3;2 #3;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #4;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #3;2 #4;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #4;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #4;2 #4;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #4;2 #3;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #4;2 #3;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #3;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #4;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 3
#I  **************************************************
#I  Starting group: [grp] #5;2 #5;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^7 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #9;2
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #8;2 #9;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #10;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #10;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #11;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #12;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #13;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #14;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #8;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #15;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #16;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #8;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 5
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #8;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #8;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #9;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #9;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #10;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #10;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #11;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #9;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #12;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #13;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #9;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #9;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #10;2 #9;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #10;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #10;2 #10;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #11;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #10;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #12;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #10;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #13;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #10;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #15;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #10;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #11;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #6;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #7;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #9;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #13;2
#I  Order: 2^8
#I  Nuclear rank: 3
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #12;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #14;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #14;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #15;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #15;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #16;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #16;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #17;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #17;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #18;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #18;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #19;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #19;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #20;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #20;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #21;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #21;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #22;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #22;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #23;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #23;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #24;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #24;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #25;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #25;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #26;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #26;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #27;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #27;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #28;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #12;2 #28;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #5;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #7;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #16;2 #7;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #9;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #16;2 #9;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #11;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #16;2 #11;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #12;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #16;2 #12;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #13;2
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #16;2 #13;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #17;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #17;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #19;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #19;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #19;3 #2;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #5;2 #19;3 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #19;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #19;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #19;3 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #19;3 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #20;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #20;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #20;3 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #20;3 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #20;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #20;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #20;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #20;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #20;3 #6;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #20;3 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #21;3 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #21;3 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #21;3 #4;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #21;3 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #21;3 #5;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #21;3 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1 #2;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #5;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #5;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #5;2 #4;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #5;2 #4;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #5;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #6;2 #5;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #6;2
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #6;2 #6;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #5;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 10
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 16
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  # of immediate descendants of order 2^8 is 12
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #7;3 #10;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #10;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #10;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #10;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #11;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #11;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #11;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #11;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #11;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #11;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #12;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #12;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #12;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #12;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #13;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #13;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #13;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #13;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #14;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #14;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #14;2 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #14;2 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #14;2 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #14;2 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #15;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #15;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #16;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #16;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #16;2 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #16;2 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #16;2 #12;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #16;2 #12;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #17;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #17;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #17;2 #5;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #17;2 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #17;2 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #17;2 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #18;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #18;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #18;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #18;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #19;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #19;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #19;2 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #19;2 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #19;2 #6;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #19;2 #6;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #19;2 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #19;2 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #20;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #20;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #21;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #21;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #21;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #21;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #21;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #21;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #23;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #23;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #23;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #23;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #26;2 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #26;2 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #26;2 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #26;2 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #26;2 #11;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #26;2 #11;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #29;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #29;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #29;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #29;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #30;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #30;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #30;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #30;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #30;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #30;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #31;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #31;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #31;2 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #31;2 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #31;2 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #31;2 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #31;2 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #31;2 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #31;2 #15;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 7
#I  Group [grp] #7;3 #31;2 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #32;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #32;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #32;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #32;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #32;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #32;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #32;2 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #32;2 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #33;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #33;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #33;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #33;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #37;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 6
#I  Group [grp] #7;3 #37;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #37;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #37;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #37;2 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #37;2 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #39;2 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #39;2 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #40;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #40;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #40;2 #2;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #40;2 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #41;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #41;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #41;2 #3;1
#I  Order: 2^8
#I  Nuclear rank: 2
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #41;2 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #44;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #44;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #44;2 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #44;2 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #46;2 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #46;2 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #46;2 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #7;3 #46;2 #7;1 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1 #1;1 #1;1
#I  Order: 2^6
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^7 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1 #1;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #3;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #1;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #3;2 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #3;2 #2;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #3;2 #2;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #4;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #4;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #4;2 #1;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #4;2 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 8
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #1;1 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #3;1 #3;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #3;1 #4;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 4
#I  **************************************************
#I  Starting group: [grp] #5;2 #5;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  # of immediate descendants of order 2^8 is 6
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #3;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #8;2 #3;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #3;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #8;2 #3;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #3;1 #7;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #8;2 #3;1 #7;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #3;1 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #8;2 #3;1 #8;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #4;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #4;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #4;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #4;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #5;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #5;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #5;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #5;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #5;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #5;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #8;2 #5;1 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #8;2 #5;1 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #3;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #3;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #4;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #4;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #4;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #4;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #5;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #5;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #9;2 #5;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #9;2 #5;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #10;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #3;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #10;2 #3;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #10;2 #3;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #10;2 #3;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #11;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #11;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #5;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #12;2 #5;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #5;1 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #12;2 #5;1 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #5;1 #10;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #12;2 #5;1 #10;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #5;1 #15;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #5;2 #12;2 #5;1 #15;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #6;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #6;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #7;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #7;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #7;1 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #7;1 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #12;2 #9;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #12;2 #9;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #13;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #1;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #13;2 #1;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #1;1 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #13;2 #1;1 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #2;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #13;2 #2;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #2;1 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #13;2 #2;1 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #4;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #13;2 #4;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #13;2 #4;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #13;2 #4;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #16;2 #5;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #16;2 #5;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #17;2 #2;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #17;2 #2;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #17;2 #2;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #5;2 #17;2 #2;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #1;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1 #1;1 #2;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 1
#I  **************************************************
#I  Starting group: [grp] #6;2 #5;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #5;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #6;2 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #2;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #6;2 #2;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #2;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #6;2 #2;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #6;2 #3;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #6;2 #3;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #5;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #5;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #7;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #1;1 #5;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #7;1 #1;1 #5;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #1;1 #9;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #7;1 #1;1 #9;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #1;1 #13;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #7;1 #1;1 #13;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #4;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #7;1 #4;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #7;3 #7;1 #4;1 #8;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 5
#I  Group [grp] #7;3 #7;1 #4;1 #8;1 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1 #1;1 #1;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1 #1;1 #1;1 #1;1 #1;1
#I  Order: 2^7
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  # of immediate descendants of order 2^8 is 3
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #1;1 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #4;2 #1;1 #1;1 #1;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #4;2 #1;1 #1;1 #1;1 #2;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #1;1 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #1;1 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #2;1 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #3;1 #3;1 #3;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #2;1 #3;1 #3;1 #3;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #2;1 #3;1 #3;1 #4;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #2;1 #3;1 #3;1 #4;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #5;2 #5;1 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 4
#I  Group [grp] #5;2 #5;1 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #1;1 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #1;1 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1 #1;1 #1;1 #1;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #3;1 #1;1 #1;1 #1;1 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;1 #1;1 #1;1 #2;1
#I  Order: 2^8
#I  Nuclear rank: 1
#I  2-multiplicator rank: 3
#I  Group [grp] #6;2 #3;1 #1;1 #1;1 #2;1 is an invalid starting group
#I  **************************************************
355
gap> ##Example: "2gp-Rel" . . . based on: examples/2gp
gap> ##Construction of largest quotient of class 6 of a 2-group
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "[b, a, a]", "(a * b * a)^4"];
[ "[b, a, a]", "(a * b * a)^4" ]
gap> Pq( F : Prime := 2, ClassBound := 6, Relators := rels, 
>         OutputLevel := 1 );
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^8
#I  Group: [grp] to lower exponent-2 central class 4 has order 2^11
#I  Group: [grp] to lower exponent-2 central class 5 has order 2^15
#I  Group: [grp] to lower exponent-2 central class 6 has order 2^19
<pc group of size 524288 with 19 generators>
gap> # Executing interactive variant of example: "2gp-Rel"
gap> ##Example: "2gp-Rel" . . . based on: examples/2gp
gap> ##Construction of largest quotient of class 6 of a 2-group
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "[b, a, a]", "(a * b * a)^4"];
[ "[b, a, a]", "(a * b * a)^4" ]
gap> procId := PqStart( F );
3
gap> Pq( procId : Prime := 2, ClassBound := 6, Relators := rels, 
>         OutputLevel := 1 );
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^8
#I  Group: [grp] to lower exponent-2 central class 4 has order 2^11
#I  Group: [grp] to lower exponent-2 central class 5 has order 2^15
#I  Group: [grp] to lower exponent-2 central class 6 has order 2^19
<pc group of size 524288 with 19 generators>
gap> PqQuitAll();
gap> ##Example: "2gp-Rel-i" . . . based on: examples/2gp
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["[b, a, a]", "(a * b * a)^4"];
[ "[b, a, a]", "(a * b * a)^4" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 6, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^8
#I  Group: [grp] to lower exponent-2 central class 4 has order 2^11
#I  Group: [grp] to lower exponent-2 central class 5 has order 2^15
#I  Group: [grp] to lower exponent-2 central class 6 has order 2^19
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "2gp-SP-1-Rel-i" . . . based on: isom/2gp.exam
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^4", "b^2*[b, a, b]^-1", "b*(a^2 * b^-1 * a^2)^-1" ];
[ "a^4", "b^2*[b, a, b]^-1", "b*(a^2 * b^-1 * a^2)^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[1,0],
>                                     [1,1]],
> 
>                                    [[0,1],
>                                     [1,0]] ]
> 
>                          : # options
>                            ClassBound := 7,
>                            PcgsAutomorphisms);
gap> ##Example: "2gp-SP-2-Rel-i" . . . based on: isom/2gp.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^4", "b^2*[b, a, b]^-1" ];
[ "a^4", "b^2*[b, a, b]^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,1,1,0],
>                                     [0,1,1,1]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> PqDisplayPcPresentation(procId);
gap> PqQuitAll();
gap> ##Example: "2gp-SP-3-Rel-i" . . . based on: isom/example.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^4", "b^2*[b, a, b]^-1", "b * (a^2 * b^-1 * a^2)^-1" ];
[ "a^4", "b^2*[b, a, b]^-1", "b * (a^2 * b^-1 * a^2)^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,1,1,0],
>                                     [0,1,1,1]] ]
> 
>                          : # options
>                            ClassBound := 9,
>                            PcgsAutomorphisms);
gap> ##Example: "2gp-SP-4-Rel-i" . . . based on: isom/isom_example.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^4", "b^2*[b, a, a]^-1", "b * (a^2 * b^-1 * a^2)^-1" ];
[ "a^4", "b^2*[b, a, a]^-1", "b * (a^2 * b^-1 * a^2)^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,1,1,0],
>                                     [0,1,1,1]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "2gp-SP-Rel-i" . . . based on: isom/2gp
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^4", "b^4", "[b, a, a]" ];
[ "a^4", "b^4", "[b, a, a]" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1, OutputLevel := 3);
#I  Lower exponent-2 central series for [grp]
#I  Defining relation was 1
#I  Defining relation was 2
#I  Defining relation was 3
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
#I  Group defining relations:
#I  4 1
#I  0
#I  4 2
#I  0
#I  1 [ 2 1 1 ]
#I  0
#I  Relationship between group defining generators and consistent
#I  power-commutator presentation generators:
#I  1    = .1
#I  1^-1 = .1
#I  2    = .2
#I  2^-1 = .2
#I  Values of power-commutator presentation generators
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  The number of new generators introduced for weight 2 is 3
gap> PqSPStandardPresentation(procId, [ [[0,1],
>                                     [1,1]],
> 
>                                    [[0,1],
>                                     [1,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
#I  Starting group has order 2^2; its automorphism group order is 6 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 
#I  2 ---> 0 1 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 2 2-quotient is
#I  Group: [grp] #1;3 to lower exponent-2 central class 2 has order 2^5
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  Subset of automorphism group to check has order bound 96
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 
#I  2 ---> 0 1 0 0 0 
#I  Non-standard label is 3
#I  Required step size is 2
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 3 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 0 1 0 0 0 
#I  2 ---> 1 1 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 
#I  2 ---> 0 1 0 0 0 
#I  Non-standard label is 6
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 6 has orbit representative 1
#I  The standard presentation for the class 3 2-quotient is
#I  Group: G #1;2 to lower exponent-2 central class 3 has order 2^7
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .6 .7
#I  Subset of automorphism group to check has order bound 32
#I  The standard automorphism is:
#I  1 ---> 1 1 1 0 0 0 0 
#I  2 ---> 0 1 1 1 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 1
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 
#I  Non-standard label is 2
#I  Required step size is 1
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 2 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 
#I  2 ---> 0 1 0 1 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 1
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 1
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 4 2-quotient is
#I  Group: G #1;1 to lower exponent-2 central class 4 has order 2^8
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8
#I  [ .5, .1 ] = .6 .7
#I  [ .6, .1 ] = .8
#I  [ .7, .1 ] = .8
#I  Subset of automorphism group to check has order bound 32
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 5 2-quotient is
#I  Group: G #1;2 to lower exponent-2 central class 5 has order 2^10
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .6^2 = .9
#I   .7^2 = .9
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .9
#I  [ .5, .1 ] = .6 .7
#I  [ .5, .4 ] = .9 .10
#I  [ .6, .1 ] = .8
#I  [ .6, .2 ] = .10
#I  [ .6, .3 ] = .10
#I  [ .6, .4 ] = .9
#I  [ .7, .1 ] = .8 .9 .10
#I  [ .7, .2 ] = .10
#I  [ .7, .3 ] = .10
#I  [ .7, .4 ] = .9
#I  [ .8, .1 ] = .9
#I  [ .8, .2 ] = .10
#I  Subset of automorphism group to check has order bound 256
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 
#I  Non-standard label is 4
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 4 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 1 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 6 2-quotient is
#I  Group: G #1;2 to lower exponent-2 central class 6 has order 2^12
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .6^2 = .9 .12
#I   .7^2 = .9 .12
#I   .8^2 = .11
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .9 .11 .12
#I  [ .5, .1 ] = .6 .7
#I  [ .5, .4 ] = .9 .10 .12
#I  [ .6, .1 ] = .8
#I  [ .6, .2 ] = .10
#I  [ .6, .3 ] = .10
#I  [ .6, .4 ] = .9 .11
#I  [ .7, .1 ] = .8 .9 .10 .11 .12
#I  [ .7, .2 ] = .10
#I  [ .7, .3 ] = .10
#I  [ .7, .4 ] = .9 .12
#I  [ .8, .1 ] = .9
#I  [ .8, .2 ] = .10
#I  [ .8, .3 ] = .12
#I  [ .8, .4 ] = .11
#I  [ .9, .1 ] = .11
#I  [ .10, .1 ] = .12
#I  Subset of automorphism group to check has order bound 256
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 7 2-quotient is
#I  Group: G #1;2 to lower exponent-2 central class 7 has order 2^14
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .6^2 = .9 .12 .14
#I   .7^2 = .9 .12
#I   .8^2 = .11 .13
#I   .9^2 = .13
#I   .10^2 = .13
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .9 .11 .12 .14
#I  [ .5, .1 ] = .6 .7
#I  [ .5, .4 ] = .9 .10 .12 .13 .14
#I  [ .6, .1 ] = .8
#I  [ .6, .2 ] = .10 .14
#I  [ .6, .3 ] = .10 .13
#I  [ .6, .4 ] = .9 .11 .13
#I  [ .6, .5 ] = .13 .14
#I  [ .7, .1 ] = .8 .9 .10 .11 .12 .14
#I  [ .7, .2 ] = .10 .13 .14
#I  [ .7, .3 ] = .10
#I  [ .7, .4 ] = .9 .12 .13
#I  [ .7, .5 ] = .13 .14
#I  [ .7, .6 ] = .13 .14
#I  [ .8, .1 ] = .9
#I  [ .8, .2 ] = .10
#I  [ .8, .3 ] = .12
#I  [ .8, .4 ] = .11 .13
#I  [ .8, .5 ] = .13 .14
#I  [ .9, .1 ] = .11
#I  [ .9, .2 ] = .13 .14
#I  [ .9, .3 ] = .13
#I  [ .9, .4 ] = .13
#I  [ .10, .1 ] = .12
#I  [ .10, .2 ] = .14
#I  [ .10, .3 ] = .14
#I  [ .10, .4 ] = .13
#I  [ .11, .1 ] = .13
#I  [ .11, .2 ] = .13
#I  [ .12, .1 ] = .13
#I  [ .12, .2 ] = .14
#I  Subset of automorphism group to check has order bound 512
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 4
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 4 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 1 1 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 2
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 8 2-quotient is
#I  Group: G #1;2 to lower exponent-2 central class 8 has order 2^16
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .6^2 = .9 .12 .14 .15 .16
#I   .7^2 = .9 .12 .15 .16
#I   .8^2 = .11 .13
#I   .9^2 = .13 .15
#I   .10^2 = .13
#I   .11^2 = .15
#I   .12^2 = .15
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .9 .11 .12 .14 .16
#I  [ .5, .1 ] = .6 .7
#I  [ .5, .4 ] = .9 .10 .12 .13 .14 .15 .16
#I  [ .6, .1 ] = .8
#I  [ .6, .2 ] = .10 .14
#I  [ .6, .3 ] = .10 .13
#I  [ .6, .4 ] = .9 .11 .13
#I  [ .6, .5 ] = .13 .14
#I  [ .7, .1 ] = .8 .9 .10 .11 .12 .14 .16
#I  [ .7, .2 ] = .10 .13 .14
#I  [ .7, .3 ] = .10
#I  [ .7, .4 ] = .9 .12 .13 .15 .16
#I  [ .7, .5 ] = .13 .14
#I  [ .7, .6 ] = .13 .14
#I  [ .8, .1 ] = .9
#I  [ .8, .2 ] = .10
#I  [ .8, .3 ] = .12 .16
#I  [ .8, .4 ] = .11 .13 .15
#I  [ .8, .5 ] = .13 .14
#I  [ .8, .6 ] = .15 .16
#I  [ .9, .1 ] = .11
#I  [ .9, .2 ] = .13 .14
#I  [ .9, .3 ] = .13 .16
#I  [ .9, .4 ] = .13 .15
#I  [ .10, .1 ] = .12
#I  [ .10, .2 ] = .14
#I  [ .10, .3 ] = .14
#I  [ .10, .4 ] = .13
#I  [ .11, .1 ] = .13
#I  [ .11, .2 ] = .13
#I  [ .11, .3 ] = .15
#I  [ .11, .4 ] = .15
#I  [ .12, .1 ] = .13 .15
#I  [ .12, .2 ] = .14
#I  [ .12, .3 ] = .16
#I  [ .12, .4 ] = .15
#I  [ .13, .1 ] = .15
#I  [ .14, .1 ] = .16
#I  Subset of automorphism group to check has order bound 512
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 9 2-quotient is
#I  Group: G #1;3 to lower exponent-2 central class 9 has order 2^19
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .6^2 = .9 .12 .14 .15 .16
#I   .7^2 = .9 .12 .15 .16
#I   .8^2 = .11 .13 .17 .18
#I   .9^2 = .13 .15
#I   .10^2 = .13 .19
#I   .11^2 = .15 .17
#I   .12^2 = .15
#I   .13^2 = .17
#I   .14^2 = .17
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .9 .11 .12 .14 .16 .18
#I  [ .5, .1 ] = .6 .7
#I  [ .5, .4 ] = .9 .10 .12 .13 .14 .15 .16 .18 .19
#I  [ .6, .1 ] = .8
#I  [ .6, .2 ] = .10 .14 .19
#I  [ .6, .3 ] = .10 .13
#I  [ .6, .4 ] = .9 .11 .13 .17 .18
#I  [ .6, .5 ] = .13 .14
#I  [ .7, .1 ] = .8 .9 .10 .11 .12 .14 .16 .17 .18 .19
#I  [ .7, .2 ] = .10 .13 .14 .17 .18
#I  [ .7, .3 ] = .10 .18 .19
#I  [ .7, .4 ] = .9 .12 .13 .15 .16 .17
#I  [ .7, .5 ] = .13 .14
#I  [ .7, .6 ] = .13 .14
#I  [ .8, .1 ] = .9
#I  [ .8, .2 ] = .10
#I  [ .8, .3 ] = .12 .16
#I  [ .8, .4 ] = .11 .13 .15
#I  [ .8, .5 ] = .13 .14 .17 .19
#I  [ .8, .6 ] = .15 .16 .18 .19
#I  [ .8, .7 ] = .17 .18
#I  [ .9, .1 ] = .11
#I  [ .9, .2 ] = .13 .14 .17 .18
#I  [ .9, .3 ] = .13 .16 .18
#I  [ .9, .4 ] = .13 .15 .17
#I  [ .9, .5 ] = .17 .19
#I  [ .9, .6 ] = .17 .18
#I  [ .9, .7 ] = .17 .18
#I  [ .10, .1 ] = .12
#I  [ .10, .2 ] = .14 .17
#I  [ .10, .3 ] = .14 .19
#I  [ .10, .4 ] = .13 .18 .19
#I  [ .10, .5 ] = .17 .19
#I  [ .10, .6 ] = .18 .19
#I  [ .10, .7 ] = .17 .18
#I  [ .10, .8 ] = .17 .18
#I  [ .11, .1 ] = .13
#I  [ .11, .2 ] = .13 .18 .19
#I  [ .11, .3 ] = .15 .17
#I  [ .11, .4 ] = .15 .17
#I  [ .12, .1 ] = .13 .15 .18 .19
#I  [ .12, .2 ] = .14
#I  [ .12, .3 ] = .16 .17 .19
#I  [ .12, .4 ] = .15
#I  [ .12, .5 ] = .17 .19
#I  [ .12, .6 ] = .17 .18
#I  [ .12, .7 ] = .17 .18
#I  [ .13, .1 ] = .15
#I  [ .13, .2 ] = .17
#I  [ .13, .3 ] = .17
#I  [ .13, .4 ] = .17
#I  [ .14, .1 ] = .16
#I  [ .14, .2 ] = .19
#I  [ .14, .3 ] = .19
#I  [ .14, .4 ] = .18
#I  [ .15, .1 ] = .17
#I  [ .15, .2 ] = .17
#I  [ .16, .1 ] = .18
#I  [ .16, .2 ] = .19
#I  Subset of automorphism group to check has order bound 8192
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 1
#I  Rank of characteristic subgroup is 1
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 2
#I  Rank of characteristic subgroup is 2
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 3
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 5
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 5 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Non-standard label is 4
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 4 has orbit representative 1
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Non-standard label is 1
#I  Required step size is 3
#I  Relative step size is 3
#I  Rank of characteristic subgroup is 4
#I  The non-standard subgroup 1 has orbit representative 1
#I  The standard presentation for the class 10 2-quotient is
#I  Group: G #1;3 to lower exponent-2 central class 10 has order 2^22
#I  Non-trivial powers:
#I   .1^2 = .4
#I   .2^2 = .5
#I   .3^2 = .6 .7
#I   .6^2 = .9 .12 .14 .15 .16 .20 .21
#I   .7^2 = .9 .12 .15 .16 .20 .21
#I   .8^2 = .11 .13 .17 .18
#I   .9^2 = .13 .15
#I   .10^2 = .13 .19 .20 .22
#I   .11^2 = .15 .17
#I   .12^2 = .15 .22
#I   .13^2 = .17 .20
#I   .14^2 = .17 .20
#I   .15^2 = .20
#I   .16^2 = .20
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .9 .11 .12 .14 .16 .18 .21 .22
#I  [ .5, .1 ] = .6 .7
#I  [ .5, .4 ] = .9 .10 .12 .13 .14 .15 .16 .18 .19 .20 .21 .22
#I  [ .6, .1 ] = .8
#I  [ .6, .2 ] = .10 .14 .19
#I  [ .6, .3 ] = .10 .13 .20 .22
#I  [ .6, .4 ] = .9 .11 .13 .17 .18
#I  [ .6, .5 ] = .13 .14 .20 .22
#I  [ .7, .1 ] = .8 .9 .10 .11 .12 .14 .16 .17 .18 .19 .20 .21 .22
#I  [ .7, .2 ] = .10 .13 .14 .17 .18 .20 .22
#I  [ .7, .3 ] = .10 .18 .19 .20
#I  [ .7, .4 ] = .9 .12 .13 .15 .16 .17 .20 .21 .22
#I  [ .7, .5 ] = .13 .14 .20 .22
#I  [ .7, .6 ] = .13 .14 .20 .22
#I  [ .8, .1 ] = .9
#I  [ .8, .2 ] = .10
#I  [ .8, .3 ] = .12 .16 .22
#I  [ .8, .4 ] = .11 .13 .15
#I  [ .8, .5 ] = .13 .14 .17 .19 .22
#I  [ .8, .6 ] = .15 .16 .18 .19 .20
#I  [ .8, .7 ] = .17 .18
#I  [ .9, .1 ] = .11
#I  [ .9, .2 ] = .13 .14 .17 .18 .20 .22
#I  [ .9, .3 ] = .13 .16 .18 .20
#I  [ .9, .4 ] = .13 .15 .17
#I  [ .9, .5 ] = .17 .19 .20
#I  [ .9, .6 ] = .17 .18 .20 .22
#I  [ .9, .7 ] = .17 .18
#I  [ .9, .8 ] = .20 .21
#I  [ .10, .1 ] = .12
#I  [ .10, .2 ] = .14 .17 .20
#I  [ .10, .3 ] = .14 .19
#I  [ .10, .4 ] = .13 .18 .19 .22
#I  [ .10, .5 ] = .17 .19 .20
#I  [ .10, .6 ] = .18 .19 .20
#I  [ .10, .7 ] = .17 .18
#I  [ .10, .8 ] = .17 .18
#I  [ .11, .1 ] = .13
#I  [ .11, .2 ] = .13 .18 .19 .22
#I  [ .11, .3 ] = .15 .17 .20 .21 .22
#I  [ .11, .4 ] = .15 .17 .20
#I  [ .12, .1 ] = .13 .15 .18 .19 .20
#I  [ .12, .2 ] = .14
#I  [ .12, .3 ] = .16 .17 .19
#I  [ .12, .4 ] = .15 .21 .22
#I  [ .12, .5 ] = .17 .19 .20
#I  [ .12, .6 ] = .17 .18 .20 .22
#I  [ .12, .7 ] = .17 .18
#I  [ .12, .8 ] = .20 .21
#I  [ .13, .1 ] = .15
#I  [ .13, .2 ] = .17 .20
#I  [ .13, .3 ] = .17 .20
#I  [ .13, .4 ] = .17 .20
#I  [ .14, .1 ] = .16
#I  [ .14, .2 ] = .19
#I  [ .14, .3 ] = .19
#I  [ .14, .4 ] = .18 .20
#I  [ .15, .1 ] = .17
#I  [ .15, .2 ] = .17 .20
#I  [ .15, .3 ] = .20
#I  [ .15, .4 ] = .20
#I  [ .16, .1 ] = .18
#I  [ .16, .2 ] = .19
#I  [ .16, .3 ] = .22
#I  [ .16, .4 ] = .21
#I  [ .17, .1 ] = .20
#I  [ .18, .1 ] = .21
#I  [ .19, .1 ] = .22
#I  Subset of automorphism group to check has order bound 8192
#I  The standard automorphism is:
#I  1 ---> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  0 
#I  2 ---> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  0 
gap> PqQuitAll();
gap> ##Example: "2gp-SP-d-Rel-i" . . . based on: isom/red1.com
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> rels := ["b^4", "b^2 * [b, a, a]^-1", "d^16", "a^16 * (c * d)^-1",
>          "b^8 * (d * c^4)^-1", "b * (a^2 * b^-1 * a^2)^-1"];
[ "b^4", "b^2 * [b, a, a]^-1", "d^16", "a^16 * (c * d)^-1", 
  "b^8 * (d * c^4)^-1", "b * (a^2 * b^-1 * a^2)^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[0,1],
>                                     [1,1]],
> 
>                                    [[0,1],
>                                     [1,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "2gp-a-Rel-i" . . . based on: examples/keyword_2gp
gap> F := FreeGroup(3, "x");
<free group on the generators [ x1, x2, x3 ]>
gap> rels := ["x1^x2 * x3", "[x2, x1, x1]", 
>          "[x2 * [x2, x1] * x1^2, x1 * x2 ]"];
[ "x1^x2 * x3", "[x2, x1, x1]", "[x2 * [x2, x1] * x1^2, x1 * x2 ]" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
2
gap> PqPcPresentation(procId : ClassBound := 3, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^4
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^6
gap> ##Example: "3gp-PG-4-i" . . . based on: examples/pga_4-3.com
gap> ##All descendants of C3 x C3 x C3 x C3
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> procId := PqStart(F : Prime := 3);
3
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^4
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^14
gap> PqPGSupplyAutomorphisms(procId, [ [[2,0,0,0],
>                                    [0,1,0,0],
>                                    [0,0,1,0],
>                                    [0,0,0,1]],
> 
>                                   [[2,0,0,1],
>                                    [2,0,0,0],
>                                    [0,2,0,0],
>                                    [0,0,2,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 2,
>                                   OrderBound := 5);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 3^4
#I  Nuclear rank: 10
#I  3-multiplicator rank: 10
#I  # of immediate descendants of order 3^5 is 6
#I  # of capable immediate descendants is 2
#I  **************************************************
6
gap> PqQuitAll();
gap> ##Example: "3gp-PG-i" . . . based on: examples/pga_3gp
gap> ##All groups with lower exponent-3 series of shape 2-2-3-1
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 3);
1
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^5
gap> PqPGSupplyAutomorphisms(procId, [ [[2,0],
>                                    [0,2]],
> 
>                                   [[0,2],
>                                    [1,0]],
> 
>                                   [[1,2],
>                                    [2,2]],
> 
>                                   [[1,0],
>                                    [2,1]],
> 
>                                   [[2,0],
>                                    [0,1]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 4,
>                                   CapableDescendants,
>                                   StepSize := [2, 3, 1],
>                                   PcgsAutomorphisms,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 3^2
#I  Nuclear rank: 3
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^4 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;2
#I  Order: 3^4
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  Group [grp] #1;2 is an invalid starting group
#I  **************************************************
#I  Starting group: [grp] #2;2
#I  Order: 3^4
#I  Nuclear rank: 3
#I  3-multiplicator rank: 4
#I  # of immediate descendants of order 3^7 is 4
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #3;2
#I  Order: 3^4
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  Group [grp] #3;2 is an invalid starting group
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #2;2 #1;3
#I  Order: 3^7
#I  Nuclear rank: 4
#I  3-multiplicator rank: 5
#I  # of immediate descendants of order 3^8 is 16
#I  # of capable immediate descendants is 11
#I  **************************************************
#I  Starting group: [grp] #2;2 #2;3
#I  Order: 3^7
#I  Nuclear rank: 3
#I  3-multiplicator rank: 4
#I  # of immediate descendants of order 3^8 is 13
#I  # of capable immediate descendants is 9
#I  **************************************************
#I  Starting group: [grp] #2;2 #3;3
#I  Order: 3^7
#I  Nuclear rank: 3
#I  3-multiplicator rank: 4
#I  # of immediate descendants of order 3^8 is 13
#I  # of capable immediate descendants is 9
#I  **************************************************
#I  Starting group: [grp] #2;2 #4;3
#I  Order: 3^7
#I  Nuclear rank: 3
#I  3-multiplicator rank: 4
#I  # of immediate descendants of order 3^8 is 7
#I  # of capable immediate descendants is 5
#I  **************************************************
41
gap> ##Example: "3gp-PG-x-i" . . . based on example from Werner
gap> ##A `by hand' descendants example
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 3);
2
gap> Pq(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
<pc group of size 9 with 2 generators>
gap> PqPCover(procId);
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^5
<pc group of size 243 with 5 generators>
gap> PqPGSupplyAutomorphisms(procId, [ [[1,1],
>                                    [0,1]],
> 
>                                   [[0,1],
>                                    [1,0]] ]);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 
#I  Generator  4 --> 0 0 0 1 1 
#I  Generator  5 --> 0 0 0 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 0 1 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 
#I  Generator  4 --> 0 0 0 0 1 
#I  Generator  5 --> 0 0 0 1 0 
gap> PqPGConstructDescendants(procId : ClassBound := 3,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 3^2
#I  Nuclear rank: 3
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^3 is 3
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 3^4 is 3
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 3^5 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 3^3
#I  Nuclear rank: 1
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^4 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #2;1
#I  Order: 3^3
#I  Nuclear rank: 2
#I  3-multiplicator rank: 4
#I  # of immediate descendants of order 3^4 is 4
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 3^5 is 7
#I  # of capable immediate descendants is 5
#I  **************************************************
#I  Starting group: [grp] #4;2
#I  Order: 3^4
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^5 is 2
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 3^6 is 2
#I  # of capable immediate descendants is 2
#I  **************************************************
#I  Starting group: [grp] #5;2
#I  Order: 3^4
#I  Nuclear rank: 3
#I  3-multiplicator rank: 4
#I  # of immediate descendants of order 3^5 is 9
#I  # of capable immediate descendants is 5
#I  # of immediate descendants of order 3^6 is 18
#I  # of capable immediate descendants is 16
#I  # of immediate descendants of order 3^7 is 4
#I  # of capable immediate descendants is 4
#I  **************************************************
#I  Starting group: [grp] #6;2
#I  Order: 3^4
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^5 is 2
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 3^6 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
#I  Starting group: [grp] #7;3
#I  Order: 3^5
#I  Nuclear rank: 5
#I  3-multiplicator rank: 5
#I  # of immediate descendants of order 3^6 is 11
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 3^7 is 58
#I  # of capable immediate descendants is 32
#I  # of immediate descendants of order 3^8 is 58
#I  # of capable immediate descendants is 58
#I  # of immediate descendants of order 3^9 is 11
#I  # of capable immediate descendants is 11
#I  # of immediate descendants of order 3^10 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
148
gap> ##Example: "3gp-PG-x-i" . . . based on example from Werner
gap> ##Iterated descendants example
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 3);
3
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^5
gap> PqPGSupplyAutomorphisms(procId, [ [[1,1],
>                                    [0,1]],
> 
>                                   [[0,1],
>                                    [1,0]] ]);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 
#I  Generator  4 --> 0 0 0 1 1 
#I  Generator  5 --> 0 0 0 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 0 1 0 0 0 
#I  Generator  2 --> 1 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 
#I  Generator  4 --> 0 0 0 0 1 
#I  Generator  5 --> 0 0 0 1 0 
gap> PqPGConstructDescendants(procId : ClassBound := 2,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 3^2
#I  Nuclear rank: 3
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^3 is 3
#I  # of capable immediate descendants is 2
#I  # of immediate descendants of order 3^4 is 3
#I  # of capable immediate descendants is 3
#I  # of immediate descendants of order 3^5 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
6
gap> PqPGSetDescendantToPcp(procId, 2, 5);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 1 0 0 
#I  Generator  4 --> 0 0 0 1 0 1 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 
#I  Generator  7 --> 0 0 0 0 2 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 
#I  Generator  7 --> 0 0 0 0 0 1 1 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 
#I  Generator  4 --> 0 0 1 1 2 0 1 
#I  Generator  5 --> 0 0 0 0 1 0 0 
#I  Generator  6 --> 0 0 0 0 1 1 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 2 0 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 
#I  Generator  5 --> 0 0 0 0 2 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 
#I  Generator  7 --> 0 0 0 0 0 0 2 
gap> PqPGConstructDescendants(procId : ClassBound := 3,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp] #6;2
#I  Order: 3^4
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^5 is 2
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 3^6 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
2
gap> PqPGSetDescendantToPcp(procId, 3, 2);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 1 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 1 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 
#I  Generator  9 --> 0 0 0 0 0 0 2 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 1 1 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 1 1 2 0 2 0 1 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 1 1 2 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 1 1 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 2 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 2 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 2 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 2 
gap> PqPGConstructDescendants(procId : ClassBound := 4,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;2
#I  Order: 3^6
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^7 is 2
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 3^8 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
2
gap> PqPGSetDescendantToPcp(procId, 4, 2);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 1 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 1 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 2 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 1 1 
#I  Automorphism 3:
#I  Generator  1 --> 1 1 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 1 1 2 0 2 0 0 0 1 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 1 1 2 0 1 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 1 1 2 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 1 1 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 2 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 2 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 2 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 2 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 2 
gap> PqPGConstructDescendants(procId : ClassBound := 5,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;2 #3;2
#I  Order: 3^8
#I  Nuclear rank: 2
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^9 is 2
#I  # of capable immediate descendants is 1
#I  # of immediate descendants of order 3^10 is 1
#I  # of capable immediate descendants is 1
#I  **************************************************
2
gap> PqPGSetDescendantToPcp(procId, 5, 1);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 1 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 1 0 1 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 1 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 1 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 1 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 2 1 
#I  Automorphism 4:
#I  Generator  1 --> 1 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 1 1 2 0 2 0 0 0 0 1 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 1 1 2 0 0 0 1 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 1 1 0 0 2 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 1 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 2 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 2 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 2 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 2 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 2 
gap> PqPGConstructDescendants(procId : ClassBound := 6,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;2 #3;2 #1;1
#I  Order: 3^9
#I  Nuclear rank: 1
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^10 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
1
gap> PqPGSetDescendantToPcp(procId, 6, 1);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 1 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 1 0 1 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 1 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 1 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 1 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 1 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 1 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 1 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 2 1 
#I  Automorphism 5:
#I  Generator  1 --> 1 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 1 1 2 0 2 0 0 0 0 0 1 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 1 1 2 0 0 0 0 1 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 1 1 0 0 0 2 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 1 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 6:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 2 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 2 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 2 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 2 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 2 
gap> PqPGConstructDescendants(procId : ClassBound := 7,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;2 #3;2 #1;1 #1;1
#I  Order: 3^10
#I  Nuclear rank: 1
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^11 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
1
gap> PqPGSetDescendantToPcp(procId, 7, 1);
gap> PqPGExtendAutomorphisms(procId);
#I  Automorphism 1:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 1 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 2:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 1 
#I  Automorphism 3:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 1 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 1 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 4:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 1 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 1 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 1 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 5:
#I  Generator  1 --> 1 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 1 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 1 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 1 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 1 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 0 2 1 
#I  Automorphism 6:
#I  Generator  1 --> 1 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 1 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 1 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 1 1 2 0 2 0 0 0 0 0 0 1 
#I  Generator  5 --> 0 0 0 0 1 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 1 1 2 0 0 0 0 0 1 0 
#I  Generator  7 --> 0 0 0 0 0 0 1 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 1 1 0 0 0 0 2 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 1 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 1 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 0 0 1 
#I  Automorphism 7:
#I  Generator  1 --> 1 0 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  2 --> 0 2 0 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  3 --> 0 0 2 0 0 0 0 0 0 0 0 0 0 0 
#I  Generator  4 --> 0 0 0 1 0 0 0 0 0 0 0 0 0 0 
#I  Generator  5 --> 0 0 0 0 2 0 0 0 0 0 0 0 0 0 
#I  Generator  6 --> 0 0 0 0 0 1 0 0 0 0 0 0 0 0 
#I  Generator  7 --> 0 0 0 0 0 0 2 0 0 0 0 0 0 0 
#I  Generator  8 --> 0 0 0 0 0 0 0 1 0 0 0 0 0 0 
#I  Generator  9 --> 0 0 0 0 0 0 0 0 1 0 0 0 0 0 
#I  Generator 10 --> 0 0 0 0 0 0 0 0 0 1 0 0 0 0 
#I  Generator 11 --> 0 0 0 0 0 0 0 0 0 0 1 0 0 0 
#I  Generator 12 --> 0 0 0 0 0 0 0 0 0 0 0 1 0 0 
#I  Generator 13 --> 0 0 0 0 0 0 0 0 0 0 0 0 2 0 
#I  Generator 14 --> 0 0 0 0 0 0 0 0 0 0 0 0 0 2 
gap> PqPGConstructDescendants(procId : ClassBound := 8,
>                                   CapableDescendants,
>                                   BasicAlgorithm);
#I  **************************************************
#I  Starting group: [grp] #6;2 #3;2 #3;2 #1;1 #1;1 #1;1
#I  Order: 3^11
#I  Nuclear rank: 1
#I  3-multiplicator rank: 3
#I  # of immediate descendants of order 3^12 is 2
#I  # of capable immediate descendants is 1
#I  **************************************************
1
gap> PqQuitAll();
gap> ##Example: "3gp-Rel-i" . . . based on: examples/3gp
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> rels := ["(b * c^-1 * d)^-3", "(c * d^-1)^3", 
>          "[b, a] * c", "[c, a]", "[c, b] * d"];
[ "(b * c^-1 * d)^-3", "(c * d^-1)^3", "[b, a] * c", "[c, a]", "[c, b] * d" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 19, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^4
#I  Group: [grp] to lower exponent-3 central class 3 has order 3^6
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
#I  Group: [grp] to lower exponent-3 central class 5 has order 3^8
#I  Group: [grp] to lower exponent-3 central class 6 has order 3^9
#I  Group: [grp] to lower exponent-3 central class 7 has order 3^10
#I  Group: [grp] to lower exponent-3 central class 8 has order 3^11
#I  Group: [grp] to lower exponent-3 central class 9 has order 3^12
#I  Group: [grp] to lower exponent-3 central class 10 has order 3^13
#I  Group: [grp] to lower exponent-3 central class 11 has order 3^14
#I  Group: [grp] to lower exponent-3 central class 12 has order 3^15
#I  Group: [grp] to lower exponent-3 central class 13 has order 3^16
#I  Group: [grp] to lower exponent-3 central class 14 has order 3^17
#I  Group: [grp] to lower exponent-3 central class 15 has order 3^18
#I  Group: [grp] to lower exponent-3 central class 16 has order 3^19
#I  Group: [grp] to lower exponent-3 central class 17 has order 3^20
#I  Group: [grp] to lower exponent-3 central class 18 has order 3^21
#I  Group: [grp] to lower exponent-3 central class 19 has order 3^22
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "3gp-SP-1-Rel-i" . . . based on: isom/3gp.ex1
gap> F := FreeGroup("x", "y");
<free group on the generators [ x, y ]>
gap> rels := ["(x * y * x)^3", "[x, y, y]"];
[ "(x * y * x)^3", "[x, y, y]" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[2,0],
>                                     [0,2]],
> 
>                                    [[0,2],
>                                     [1,0]],
> 
>                                    [[1,2],
>                                     [2,2]],
> 
>                                    [[1,0],
>                                     [2,1]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "3gp-SP-2-Rel-i" . . . based on: isom/3gp.ex2
gap> F := FreeGroup("x", "y");
<free group on the generators [ x, y ]>
gap> rels := ["(x * y * x)^3", "[y, x, x]"];
[ "(x * y * x)^3", "[y, x, x]" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[2,0],
>                                     [0,2]],
> 
>                                    [[0,2],
>                                     [1,0]],
> 
>                                    [[1,2],
>                                     [2,2]],
> 
>                                    [[1,0],
>                                     [2,1]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 9,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "3gp-SP-3-Rel-i" . . . based on: isom/3gp.exam
gap> F := FreeGroup("x", "y");
<free group on the generators [ x, y ]>
gap> rels := ["(x * y * x)^3", "[x, y, x]"];
[ "(x * y * x)^3", "[x, y, x]" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[2,0],
>                                     [0,2]],
> 
>                                    [[0,2],
>                                     [1,0]],
> 
>                                    [[1,2],
>                                     [2,2]],
> 
>                                    [[1,0],
>                                     [2,1]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 6,
>                            PcgsAutomorphisms);
gap> ##Example: "3gp-SP-4-Rel-i" . . . based on: isom/3gp.test
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["(b * [b, a] * [b, a, b])^3", "([b, a] * [b, a, b]^2)^3",
>          "[b, a, a]"];
[ "(b * [b, a] * [b, a, b])^3", "([b, a] * [b, a, b]^2)^3", "[b, a, a]" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[2,0],
>                                     [0,2]],
> 
>                                    [[0,2],
>                                     [1,0]],
> 
>                                    [[1,2],
>                                     [2,2]],
> 
>                                    [[1,0],
>                                     [2,1]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 7,
>                            PcgsAutomorphisms);
gap> ##Example: "3gp-SP-Rel-i" . . . based on: isom/3gp
gap> F := FreeGroup("x", "y");
<free group on the generators [ x, y ]>
gap> rels := ["[x, y, y, y]", "[x, y, x]"];
[ "[x, y, y, y]", "[x, y, x]" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[2,0],
>                                     [0,2]],
> 
>                                    [[0,2],
>                                     [1,0]],
> 
>                                    [[1,2],
>                                     [2,2]],
> 
>                                    [[1,0],
>                                     [2,1]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "3gp-a-Rel" . . . based on: examples/3gpA
gap> ##(exponent 9 group with 2 generators of order 3)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^3", "b^3"];
[ "a^3", "b^3" ]
gap> Pq( F : Prime := 3, Exponent := 9, Relators := rels, ClassBound := 6,
>         OutputLevel := 1 );
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^3
#I  Group: [grp] to lower exponent-3 central class 3 has order 3^5
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
#I  Group: [grp] to lower exponent-3 central class 5 has order 3^9
#I  Group: [grp] to lower exponent-3 central class 6 has order 3^11
<pc group of size 177147 with 11 generators>
gap> # Executing interactive variant of example: "3gp-a-Rel"
gap> ##Example: "3gp-a-Rel" . . . based on: examples/3gpA
gap> ##(exponent 9 group with 2 generators of order 3)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^3", "b^3"];
[ "a^3", "b^3" ]
gap> procId := PqStart( F );
1
gap> Pq( procId : Prime := 3, Exponent := 9, Relators := rels, ClassBound := 6,
>         OutputLevel := 1 );
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^3
#I  Group: [grp] to lower exponent-3 central class 3 has order 3^5
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
#I  Group: [grp] to lower exponent-3 central class 5 has order 3^9
#I  Group: [grp] to lower exponent-3 central class 6 has order 3^11
<pc group of size 177147 with 11 generators>
gap> ##Example: "3gp-a-Rel-i" . . . based on: examples/3gpA
gap> ##(exponent 9 group with 2 generators of order 3)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^3", "b^3"];
[ "a^3", "b^3" ]
gap> procId := PqStart(F : Prime := 3, Exponent := 9, Relators := rels);
2
gap> PqPcPresentation(procId : ClassBound := 12, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^3
#I  Group: [grp] to lower exponent-3 central class 3 has order 3^5
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
#I  Group: [grp] to lower exponent-3 central class 5 has order 3^9
#I  Group: [grp] to lower exponent-3 central class 6 has order 3^11
#I  Group: [grp] to lower exponent-3 central class 7 has order 3^15
#I  Group: [grp] to lower exponent-3 central class 8 has order 3^20
#I  Group: [grp] to lower exponent-3 central class 9 has order 3^28
#I  Group: [grp] to lower exponent-3 central class 10 has order 3^38
#I  Group: [grp] to lower exponent-3 central class 11 has order 3^52
#I  Group: [grp] to lower exponent-3 central class 12 has order 3^73
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "3gp-a-x-Rel-i" . . . based on: examples/3gpA
gap> ##(exponent 9 group with 2 generators of order 3, extended)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^3", "b^3"];
[ "a^3", "b^3" ]
gap> procId := PqStart(F : Prime := 3, Exponent := 9, Relators := rels);
3
gap> PqPcPresentation(procId : ClassBound := 4, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^3
#I  Group: [grp] to lower exponent-3 central class 3 has order 3^5
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
gap> PqDisplayPcPresentation(procId : OutputLevel := 2);
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
#I  Non-trivial powers:
#I   .3^3 = .6^2
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .3, .2 ] = .5
#I  [ .4, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .5, .1 ] = .7
#I  [ .5, .2 ] = .6
gap> PqNextClass(procId);
#I  Processing tails for generators of weight 4 and 1
#I  Processing tails for generators of weight 3 and 2
#I  The number of new generators introduced for weight 5 is 4
#I  Processing tails for generators of weight 3 and 1
#I  Processing tails for generators of weight 2 and 2
#I  The number of new generators introduced for weight 4 is 2
#I  Generator 12 is redundant
#I  Jacobi was 3 2 1
#I  Processing tails for generators of weight 2 and 1
#I  The number of new generators introduced for weight 3 is 0
#I  Generator 13 is redundant
#I  Jacobi was 2 1 1
#I  The number of new generators introduced for weight 2 is 2
#I  Generator 14 is trivial
#I  Defining relation was 1
#I  Generator 15 is trivial
#I  Defining relation was 2
#I  2 relations of class 1 collected
#I  Generator 9 is redundant
#I  Collected power 9 of the following word: 1^1 2^1 
#I  2 relations of class 2 collected
#I  Generator 8 is trivial
#I  Collected power 9 of the following word: 1^1 2^2 
#I  3 relations of class 3 collected
#I  2 relations of class 4 collected
#I  1 relation of class 5 collected
#I  Group: [grp] to lower exponent-3 central class 5 has order 3^9
#I  Non-trivial powers:
#I   .3^3 = .6^2 .8^2
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .3, .2 ] = .5
#I  [ .4, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8
#I  [ .5, .1 ] = .7 .8 .9^2
#I  [ .5, .2 ] = .6 .8 .9^2
#I  [ .5, .3 ] = .9^2
#I  [ .7, .1 ] = .8
#I  [ .7, .2 ] = .9
gap> PqSetOutputLevel(procId, 1);
gap> PqSetupTablesForNextClass(procId);
#I  Setup performed for class 6
gap> PqTails(procId, 0);
gap> PqDoConsistencyChecks(procId, 0, 0);
gap> PqCollectDefiningRelations(procId);
gap> PqDoExponentChecks(procId);
gap> PqEliminateRedundantGenerators(procId);
gap> PqDisplayPcPresentation(procId : OutputLevel := 2);
#I  Group: [grp] to lower exponent-3 central class 6 has order 3^11
#I  Non-trivial powers:
#I   .3^3 = .6^2 .8^2 .10 .11
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .3, .2 ] = .5
#I  [ .4, .1 ] = .6
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .8 .10^2 .11^2
#I  [ .5, .1 ] = .7 .8 .9^2 .10^2 .11^2
#I  [ .5, .2 ] = .6 .8 .9^2 .10^2
#I  [ .5, .3 ] = .9^2 .11
#I  [ .5, .4 ] = .10 .11
#I  [ .6, .2 ] = .10^2
#I  [ .7, .1 ] = .8
#I  [ .7, .2 ] = .9
#I  [ .7, .3 ] = .10^2 .11
#I  [ .8, .2 ] = .10
#I  [ .9, .1 ] = .11
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "3gp-maxoccur-Rel-i" . . . based on: examples/maxoccur
gap> ##Demonstrates usage of `PqSetMaximalOccurrences'.
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^3", "b^3"];
[ "a^3", "b^3" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-3 central series for [grp]
#I  Group: [grp] to lower exponent-3 central class 1 has order 3^2
gap> PqSetMaximalOccurrences(procId, [6, 4]);
gap> for class in [2 .. 9] do
>   PqNextClass(procId);
> od;
#I  Group: [grp] to lower exponent-3 central class 2 has order 3^3
#I  Group: [grp] to lower exponent-3 central class 3 has order 3^5
#I  Group: [grp] to lower exponent-3 central class 4 has order 3^7
#I  Group: [grp] to lower exponent-3 central class 5 has order 3^11
#I  Group: [grp] to lower exponent-3 central class 6 has order 3^15
#I  Group: [grp] to lower exponent-3 central class 7 has order 3^23
#I  Group: [grp] to lower exponent-3 central class 8 has order 3^29
#I  Group: [grp] to lower exponent-3 central class 9 has order 3^35
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "5gp-PG-i" . . . based on: examples/pga_5gp
gap> ##Descendants of C5 x C5
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 5);
2
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^5
gap> PqPGSupplyAutomorphisms(procId, [ [[2,0],
>                                    [0,1]],
> 
>                                   [[4,1],
>                                    [4,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 2,
>                                   StepSize := 2,
>                                   RankInitialSegmentSubgroups := 4);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 5^2
#I  Nuclear rank: 3
#I  5-multiplicator rank: 3
#I  # of immediate descendants of order 5^4 is 3
#I  # of capable immediate descendants is 3
#I  **************************************************
3
gap> ##Example: "5gp-Rel-i" . . . based on: examples/5gp
gap> F := FreeGroup("a", "b", "c");
<free group on the generators [ a, b, c ]>
gap> rels := ["a^25 * c", "[a,  b] * c^-4", "[a, c]^25"];
[ "a^25 * c", "[a,  b] * c^-4", "[a, c]^25" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
3
gap> PqPcPresentation(procId : ClassBound := 27, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^4
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^6
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^12
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^16
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^20
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^22
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^24
#I  Group: [grp] to lower exponent-5 central class 13 has order 5^26
#I  Group: [grp] to lower exponent-5 central class 14 has order 5^28
#I  Group: [grp] to lower exponent-5 central class 15 has order 5^30
#I  Group: [grp] to lower exponent-5 central class 16 has order 5^32
#I  Group: [grp] to lower exponent-5 central class 17 has order 5^34
#I  Group: [grp] to lower exponent-5 central class 18 has order 5^36
#I  Group: [grp] to lower exponent-5 central class 19 has order 5^38
#I  Group: [grp] to lower exponent-5 central class 20 has order 5^40
#I  Group: [grp] to lower exponent-5 central class 21 has order 5^42
#I  Group: [grp] to lower exponent-5 central class 22 has order 5^44
#I  Group: [grp] to lower exponent-5 central class 23 has order 5^46
#I  Group: [grp] to lower exponent-5 central class 24 has order 5^48
#I  Group: [grp] to lower exponent-5 central class 25 has order 5^50
#I  Group: [grp] to lower exponent-5 central class 26 has order 5^52
#I  Group: [grp] to lower exponent-5 central class 27 has order 5^54
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "5gp-SP-Rel-i" . . . based on: isom/5gp
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^25", "[b, a, a, a]", "b^5*[b, a, a]^-1"];
[ "a^25", "[b, a, a, a]", "b^5*[b, a, a]^-1" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,3,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,3,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "5gp-SP-a-Rel-i" . . . based on: isom/5gp_a
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^25", "[b, a, a]", "[b, a, b, b, b, b]", "b^5"];
[ "a^25", "[b, a, a]", "[b, a, b, b, b, b]", "b^5" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,3,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,3,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 27,
>                            PcgsAutomorphisms);
gap> ##Example: "5gp-SP-b-Rel-i" . . . based on: isom/5gp_b
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^25", "[b, a, a]", "[b, a, a, a, a, a, b]",
>          "[b, a, b, b, b, b]", "b^5 * [b, a, a]^-1"];
[ "a^25", "[b, a, a]", "[b, a, a, a, a, a, b]", "[b, a, b, b, b, b]", 
  "b^5 * [b, a, a]^-1" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,3,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,3,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 27,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "5gp-SP-big-Rel-i" . . . based on: isom/large_5gp
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> rels := ["a * b^d", "c * d^a", "[c, a, a]", "[c, b, b, a]"];
[ "a * b^d", "c * d^a", "[c, a, a]", "[c, b, b, a]" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[4,1],
>                                     [4,0]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 5);
gap> ##Example: "5gp-SP-d-Rel-i" . . . based on: isom/5gp_d
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^25", "[b, a, a, a, b]", "[b, a, b, b, b]",
>          "[b, a, b, a]", "b^5 * [a, b, a]^-1"];
[ "a^25", "[b, a, a, a, b]", "[b, a, b, b, b]", "[b, a, b, a]", 
  "b^5 * [a, b, a]^-1" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,3,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,3,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 20,
>                            PcgsAutomorphisms);
gap> ##Example: "5gp-a-Rel-i" . . . based on: examples/5gpA
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^5", "b^5", "(a * b)^5", "(a * b^2)^5", "(a * b^-2)^5",
>          "(a * b^-1)^5", "(a^2 * b)^5", "(a^2 * b^2)^5", "(a^2 * b^-2)^5",
>          "(a^2 * b^-1)^5", "(a * b * a^-1 * b^2)^5", 
>          "(a * b * a^2 * b^-1)^5", "(a * b * a^2 * b^2)^5",
>          "(a * b * a * b^-1)^5", "(a * b * a^-1 * b)^5",
>          "(a * b * a^-1 * b^-1)^5"];
[ "a^5", "b^5", "(a * b)^5", "(a * b^2)^5", "(a * b^-2)^5", "(a * b^-1)^5", 
  "(a^2 * b)^5", "(a^2 * b^2)^5", "(a^2 * b^-2)^5", "(a^2 * b^-1)^5", 
  "(a * b * a^-1 * b^2)^5", "(a * b * a^2 * b^-1)^5", "(a * b * a^2 * b^2)^5",
  "(a * b * a * b^-1)^5", "(a * b * a^-1 * b)^5", "(a * b * a^-1 * b^-1)^5" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
3
gap> PqPcPresentation(procId : ClassBound := 5, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
gap> PqNextClass(procId); #class 6
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^14
gap> PqNextClass(procId); #class 7
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^18
gap> PqNextClass(procId); #class 8
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^22
gap> PqNextClass(procId); #class 9
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^28
gap> PqNextClass(procId); #class 10
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^39
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "5gp-b-Rel-i" . . . based on: examples/5gpB
gap> F := FreeGroup("x", "y", "z", "w");
<free group on the generators [ x, y, z, w ]>
gap> rels := ["x^25 * (x^z)^-1", "[x, y] * z^-1", "[x, z]"];
[ "x^25 * (x^z)^-1", "[x, y] * z^-1", "[x, z]" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 5, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^32
gap> ##Example: "5gp-c-Rel-i" . . . based on: examples/keyword_5gp
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> rels := ["[a, b, b, c]", "(a * b * c^d * a)^25", 
>          "(a * b)^25 * [a, c, c, d]^-2"];
[ "[a, b, b, c]", "(a * b * c^d * a)^25", "(a * b)^25 * [a, c, c, d]^-2" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
2
gap> PqPcPresentation(procId : ClassBound := 2, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^4
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^14
gap> ##Example: "5gp-maxoccur-Rel-i" . . . based on: GrpFP_2_pQuotient6
gap> ##Demonstrates usage of `PqSetMaximalOccurrences'.
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^5", "b^5"];
[ "a^5", "b^5" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
3
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
gap> PqSetMaximalOccurrences(procId, [3, 2]);
gap> for class in [2 .. 6] do
>   PqNextClass(procId);
> od;
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^7
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^9
#I  Group completed. Lower exponent-5 central class = 5, Order = 5^9
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^9
gap> PqDisplayPcPresentation(procId);
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^9
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "5gp-metabelian-Rel" . . . based on: examples/metabelian
gap> ##Construction of a metabelian 5-quotient
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "a^625", "b^625", "[b, a, b]", 
>           "[b, a, a, a, a] * [b, a]^-5" ];
[ "a^625", "b^625", "[b, a, b]", "[b, a, a, a, a] * [b, a]^-5" ]
gap> Pq( F : Prime := 5, ClassBound := 20, Metabelian, Relators := rels, 
>         OutputLevel := 1 );
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^11
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^12
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^13
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^15
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^16
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^17
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^19
#I  Group: [grp] to lower exponent-5 central class 13 has order 5^20
#I  Group completed. Lower exponent-5 central class = 13, Order = 5^20
<pc group of size 95367431640625 with 20 generators>
gap> # Executing interactive variant of example: "5gp-metabelian-Rel"
gap> ##Example: "5gp-metabelian-Rel" . . . based on: examples/metabelian
gap> ##Construction of a metabelian 5-quotient
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "a^625", "b^625", "[b, a, b]", 
>           "[b, a, a, a, a] * [b, a]^-5" ];
[ "a^625", "b^625", "[b, a, b]", "[b, a, a, a, a] * [b, a]^-5" ]
gap> procId := PqStart( F );
1
gap> Pq( procId : Prime := 5, ClassBound := 20, Metabelian, Relators := rels, 
>         OutputLevel := 1 );
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^11
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^12
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^13
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^15
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^16
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^17
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^19
#I  Group: [grp] to lower exponent-5 central class 13 has order 5^20
#I  Group completed. Lower exponent-5 central class = 13, Order = 5^20
<pc group of size 95367431640625 with 20 generators>
gap> ##Example: "5gp-metabelian-Rel-i" . . . based on: examples/metabelian
gap> ##Demonstrates usage of `PqSetMetabelian'.
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^625", "b^625", "[b, a, b]", "[b, a, a, a, a] * [b, a]^-5"];
[ "a^625", "b^625", "[b, a, b]", "[b, a, a, a, a] * [b, a]^-5" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
2
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
gap> PqSetMetabelian(procId);
gap> for class in [2 .. 14] do
>   PqNextClass(procId);
> od;
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^11
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^12
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^13
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^15
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^16
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^17
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^19
#I  Group: [grp] to lower exponent-5 central class 13 has order 5^20
#I  Group completed. Lower exponent-5 central class = 13, Order = 5^20
#I  Group: [grp] to lower exponent-5 central class 13 has order 5^20
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "7gp-PG-i" . . . based on: examples/pga_7gp
gap> ##Descendants of C7 x C7
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 7);
3
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-7 central series for [grp]
#I  Group: [grp] to lower exponent-7 central class 1 has order 7^2
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-7 central class 2 has order 7^5
gap> PqPGSupplyAutomorphisms(procId, [ [[3,0],
>                                    [0,1]],
> 
>                                   [[6,1],
>                                    [6,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 2,
>                                   CapableDescendants,
>                                   StepSize := 1);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 7^2
#I  Nuclear rank: 3
#I  7-multiplicator rank: 3
#I  # of immediate descendants of order 7^3 is 3
#I  # of capable immediate descendants is 2
#I  **************************************************
2
gap> PqQuitAll();
gap> ##Example: "7gp-Rel-i" . . . based on: examples/7gp
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["[b, a, a, b, b]^49", "(a * b * b * a * b * a)^49", "b^7"];
[ "[b, a, a, b, b]^49", "(a * b * b * a * b * a)^49", "b^7" ]
gap> procId := PqStart(F : Prime := 7, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 10, 
>                           OutputLevel := 1);
#I  Lower exponent-7 central series for [grp]
#I  Group: [grp] to lower exponent-7 central class 1 has order 7^2
#I  Group: [grp] to lower exponent-7 central class 2 has order 7^4
#I  Group: [grp] to lower exponent-7 central class 3 has order 7^6
#I  Group: [grp] to lower exponent-7 central class 4 has order 7^9
#I  Group: [grp] to lower exponent-7 central class 5 has order 7^15
#I  Group: [grp] to lower exponent-7 central class 6 has order 7^24
#I  Group: [grp] to lower exponent-7 central class 7 has order 7^42
#I  Group: [grp] to lower exponent-7 central class 8 has order 7^72
#I  Group: [grp] to lower exponent-7 central class 9 has order 7^128
#I  Group: [grp] to lower exponent-7 central class 10 has order 7^226
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "7gp-SP-Rel-i" . . . based on: isom/7gp.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^7", "b^7 * [b, a, a]^-1", "[b, a, b, b, b, b, b]"];
[ "a^7", "b^7 * [b, a, a]^-1", "[b, a, b, b, b, b, b]" ]
gap> procId := PqStart(F : Prime := 7, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0,1],
>                                     [0,1,0,0,0]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,1]],
> 
>                                    [[1,0,0,0,0],
>                                     [0,1,0,0,1]],
> 
>                                    [[1,0,0,0,0],
>                                     [6,1,0,0,0]],
> 
>                                    [[6,0,0,0,0],
>                                     [2,1,0,0,0]],
> 
>                                    [[6,0,0,0,0],
>                                     [0,6,0,0,0]],
> 
>                                    [[6,0,0,0,0],
>                                     [4,5,0,0,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "7gp-SP-a-Rel-i" . . . based on: isom/7gp_a
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["b^7", "a^7 * [b, a, b]^-1", "(a * b * [b, a, b] )^7",
>          "[b, a, a, a, a, b, a]"];
[ "b^7", "a^7 * [b, a, b]^-1", "(a * b * [b, a, b] )^7", 
  "[b, a, a, a, a, b, a]" ]
gap> procId := PqStart(F : Prime := 7, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[6,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,4,0,0],
>                                     [0,4,0,0]] ]
> 
>                          : # options
>                            ClassBound := 20,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "7gp-SP-b-Rel-i" . . . based on: isom/7gp_b
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["b^7 * [b, a, a]^-1", "[b, a, b]"];
[ "b^7 * [b, a, a]^-1", "[b, a, b]" ]
gap> procId := PqStart(F : Prime := 7, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[6,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,6,0,0]],
> 
>                                    [[1,4,0,0],
>                                     [0,4,0,0]] ]
> 
>                          : # options
>                            ClassBound := 40,
>                            PcgsAutomorphisms);
gap> ##Example: "B2-4" . . . the ``right'' way to generate B(2, 4)
gap> ##Generates B(2, 4) by using the `Exponent' option
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> Pq( F : Prime := 2, Exponent := 4 );
<pc group of size 4096 with 12 generators>
gap> # Executing interactive variant of example: "B2-4"
gap> ##Example: "B2-4" . . . the ``right'' way to generate B(2, 4)
gap> ##Generates B(2, 4) by using the `Exponent' option
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart( F );
2
gap> Pq( procId : Prime := 2, Exponent := 4 );
<pc group of size 4096 with 12 generators>
gap> ##Example: "B2-4-Id" . . . alternative way to generate B(2, 4)
gap> ##Generates B(2, 4) by using the `Identities' option
gap> ##... this is not as efficient as using `Exponent' but
gap> ##demonstrates the usage of the `Identities' option.
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> # All words w in the pc generators of B(2, 4) satisfy f(w) = 1 
gap> f := w -> w^4;
function( w ) ... end
gap> Pq( F : Prime := 2, Identities := [ f ] );
#I  Class 1 with 2 generators.
#I  Class 2 with 5 generators.
#I  Class 3 with 7 generators.
#I  Class 4 with 10 generators.
#I  Class 5 with 12 generators.
#I  Class 5 with 12 generators.
<pc group of size 4096 with 12 generators>
gap> # Executing interactive variant of example: "B2-4-Id"
gap> ##Example: "B2-4-Id" . . . alternative way to generate B(2, 4)
gap> ##Generates B(2, 4) by using the `Identities' option
gap> ##... this is not as efficient as using `Exponent' but
gap> ##demonstrates the usage of the `Identities' option.
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> # All words w in the pc generators of B(2, 4) satisfy f(w) = 1 
gap> f := w -> w^4;
function( w ) ... end
gap> procId := PqStart( F );
3
gap> Pq( procId : Prime := 2, Identities := [ f ] );
#I  Class 1 with 2 generators.
#I  Class 2 with 5 generators.
#I  Class 3 with 7 generators.
#I  Class 4 with 10 generators.
#I  Class 5 with 12 generators.
#I  Class 5 with 12 generators.
<pc group of size 4096 with 12 generators>
gap> PqQuitAll();
gap> ##Example: "B2-4-SP-i" . . . based on: isom/B2-4.com
gap> ##Computes standard presentation for B(2,4)
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> procId := PqStart(F : Prime := 2, Exponent := 4);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[0,1],
>                                     [1,1]],
> 
>                                    [[0,1],
>                                     [1,0]] ]
> 
>                          : # options
>                            ClassBound := 6,
>                            PcgsAutomorphisms);
gap> ##Example: "B2-5" . . . based on: examples/B2-5
gap> ##Construction of B(2,5)
gap> ##i.e. largest finite 2-generator group of exponent 5
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> Pq( F : Prime := 5, ClassBound := 14, Exponent := 5,
>         OutputLevel := 1 );
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^22
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^28
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^31
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^33
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^34
#I  Group completed. Lower exponent-5 central class = 12, Order = 5^34
<pc group of size 582076609134674072265625 with 34 generators>
gap> # Executing interactive variant of example: "B2-5"
gap> ##Example: "B2-5" . . . based on: examples/B2-5
gap> ##Construction of B(2,5)
gap> ##i.e. largest finite 2-generator group of exponent 5
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart( F );
2
gap> Pq( procId : Prime := 5, ClassBound := 14, Exponent := 5,
>         OutputLevel := 1 );
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^22
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^28
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^31
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^33
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^34
#I  Group completed. Lower exponent-5 central class = 12, Order = 5^34
<pc group of size 582076609134674072265625 with 34 generators>
gap> ##Example: "B2-5-i" . . . based on: examples/B2-5
gap> ##Construction of B(2,5)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 5, Exponent := 5);
3
gap> PqPcPresentation(procId : ClassBound := 15, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^22
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^28
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^31
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^33
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^34
#I  Group completed. Lower exponent-5 central class = 12, Order = 5^34
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "B2-8-i" . . . based on: examples/B2-8
gap> ##Construction of B(2,8)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 2, Exponent := 8);
1
gap> PqPcPresentation(procId : ClassBound := 10, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^2
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^5
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^10
#I  Group: [grp] to lower exponent-2 central class 4 has order 2^15
#I  Group: [grp] to lower exponent-2 central class 5 has order 2^24
#I  Group: [grp] to lower exponent-2 central class 6 has order 2^35
#I  Group: [grp] to lower exponent-2 central class 7 has order 2^53
#I  Group: [grp] to lower exponent-2 central class 8 has order 2^83
#I  Group: [grp] to lower exponent-2 central class 9 has order 2^139
#I  Group: [grp] to lower exponent-2 central class 10 has order 2^238
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "B4-4-a-i" . . . based on: examples/B4-4A
gap> ##Construction of B(4,4)
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> procId := PqStart(F : Prime := 2, Exponent := 4);
2
gap> PqPcPresentation(procId : ClassBound := 6, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^4
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^14
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^34
#I  Group: [grp] to lower exponent-2 central class 4 has order 2^89
#I  Group: [grp] to lower exponent-2 central class 5 has order 2^188
#I  Group: [grp] to lower exponent-2 central class 6 has order 2^272
gap> PqSupplyAutomorphisms(procId, [ [[0,0,0,1],
>                                  [1,0,0,0],
>                                  [0,1,0,0],
>                                  [0,0,1,0]],
> 
>                                 [[1,1,0,0],
>                                  [0,1,0,0],
>                                  [0,0,1,0],
>                                  [0,0,0,1]] ]);
gap> for class in [7 .. 11] do
>   PqNextClass(procId : QueueFactor := 20);
> od;
#I  Group: [grp] to lower exponent-2 central class 7 has order 2^352
#I  Group: [grp] to lower exponent-2 central class 8 has order 2^392
#I  Group: [grp] to lower exponent-2 central class 9 has order 2^412
#I  Group: [grp] to lower exponent-2 central class 10 has order 2^422
#I  Group completed. Lower exponent-2 central class = 10, Order = 2^422
#I  Group: [grp] to lower exponent-2 central class 10 has order 2^422
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "B4-4-i" . . . based on: examples/B4-4
gap> ##Construction of B(4,4)
gap> F := FreeGroup("a", "b", "c", "d");
<free group on the generators [ a, b, c, d ]>
gap> procId := PqStart(F : Prime := 2, Exponent := 4);
3
gap> PqPcPresentation(procId : ClassBound := 6, 
>                           OutputLevel := 1);
#I  Lower exponent-2 central series for [grp]
#I  Group: [grp] to lower exponent-2 central class 1 has order 2^4
#I  Group: [grp] to lower exponent-2 central class 2 has order 2^14
#I  Group: [grp] to lower exponent-2 central class 3 has order 2^34
#I  Group: [grp] to lower exponent-2 central class 4 has order 2^89
#I  Group: [grp] to lower exponent-2 central class 5 has order 2^188
#I  Group: [grp] to lower exponent-2 central class 6 has order 2^272
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqQuitAll();
gap> ##Example: "B5-4.g" . . . by Werner Nickel
gap> ##. . . . . . . . . . . . and based on a pq input file by M.F.Newman
gap> ##(constructs the Burnside group B(5,4), which is the largest group of 
gap> ## exponent 4 generated by 5 elements; it has order 2^2728 and p-central
gap> ## class 13)
gap> ##Note: It is a construction only and makes use of specialised knowledge
gap> ##gained by Newman & O'Brien in their investigations of B(5,4).
gap> LoadPackage( "anupq" );
true
gap> #You might like to try setting: `SetInfoLevel( InfoANUPQ, 3 );'
gap> 
gap> procId := PqStart( FreeGroup(5) : Exponent := 4, Prime := 2 );
1
gap> Pq( procId : ClassBound := 2 );
<pc group of size 1048576 with 20 generators>
gap> PqSupplyAutomorphisms( procId,
>       [
>         [ [ 1, 1, 0, 0, 0],      #1st automorphism
>           [ 0, 1, 0, 0, 0],
>           [ 0, 0, 1, 0, 0],
>           [ 0, 0, 0, 1, 0],
>           [ 0, 0, 0, 0, 1] ],
> 
>         [ [ 0, 0, 0, 0, 1],      #2nd automorphism
>           [ 1, 0, 0, 0, 0],
>           [ 0, 1, 0, 0, 0],
>           [ 0, 0, 1, 0, 0],
>           [ 0, 0, 0, 1, 0] ]
>                              ] );
gap> 
gap> Relations :=
>   [ [],          ## class 1
>     [],          ## class 2
>     [],          ## class 3
>     [],          ## class 4
>     [],          ## class 5
>     [],          ## class 6
>     ## class 7     
>     [ [ "x2","x1","x1","x3","x4","x4","x4" ] ],
>     ## class 8
>     [ [ "x2","x1","x1","x3","x4","x5","x5","x5" ] ],
>     ## class 9
>     [ [ "x2","x1","x1","x3","x4","x4","x5","x5","x5" ],
>       [ "x2","x1","x1","x2","x3","x4","x5","x5","x5" ],
>       [ "x2","x1","x1","x3","x3","x4","x5","x5","x5" ] ],
>     ## class 10
>     [ [ "x2","x1","x1","x2","x3","x3","x4","x5","x5","x5" ],
>       [ "x2","x1","x1","x3","x3","x4","x4","x5","x5","x5" ] ],
>     ## class 11
>     [ [ "x2","x1","x1","x2","x3","x3","x4","x4","x5","x5","x5" ],
>       [ "x2","x1","x1","x2","x3","x1","x3","x4","x2","x4","x3" ] ],
>     ## class 12
>     [ [ "x2","x1","x1","x2","x3","x1","x3","x4","x2","x5","x5","x5" ],
>       [ "x2","x1","x1","x3","x2","x4","x3","x5","x4","x5","x5","x5" ] ],
>     ## class 13
>     [ [ "x2","x1","x1","x2","x3","x1","x3","x4","x2","x4","x5","x5","x5" 
>         ] ]
> ];
[ [  ], [  ], [  ], [  ], [  ], [  ], 
  [ [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ] ], 
  [ [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ] ], 
  [ [ "x2", "x1", "x1", "x3", "x4", "x4", "x5", "x5", "x5" ], 
      [ "x2", "x1", "x1", "x2", "x3", "x4", "x5", "x5", "x5" ], 
      [ "x2", "x1", "x1", "x3", "x3", "x4", "x5", "x5", "x5" ] ], 
  [ [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x5", "x5", "x5" ], 
      [ "x2", "x1", "x1", "x3", "x3", "x4", "x4", "x5", "x5", "x5" ] ], 
  [ [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x4", "x5", "x5", "x5" ], 
      [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x4", "x3" ] ], 
  [ [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x5", "x5", "x5" ]
        , [ "x2", "x1", "x1", "x3", "x2", "x4", "x3", "x5", "x4", "x5", "x5", 
          "x5" ] ], 
  [ [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x4", "x5", "x5", 
          "x5" ] ] ]
gap> 
gap> for class in [ 3 .. 13 ] do
>     Print( "Computing class ", class, "\n" );
>     PqSetupTablesForNextClass( procId );
> 
>     for w in [ class, class-1 .. 7 ] do
> 
>         PqAddTails( procId, w );   
>         PqDisplayPcPresentation( procId );
> 
>         if Relations[ w ] <> [] then
>             # recalculate automorphisms
>             PqExtendAutomorphisms( procId );
> 
>             for r in Relations[ w ] do
>                 Print( "Collecting ", r, "\n" );
>                 PqCommutator( procId, r, 1 );
>                 PqEchelonise( procId );
>                 PqApplyAutomorphisms( procId, 15 ); #queue factor = 15
>             od;
> 
>             PqEliminateRedundantGenerators( procId );
>         fi;   
>         PqComputeTails( procId, w );
>     od;
>     PqDisplayPcPresentation( procId );
> 
>     smallclass := Minimum( class, 6 );
>     for w in [ smallclass, smallclass-1 .. 2 ] do
>         PqTails( procId, w );
>     od;
>     # recalculate automorphisms
>     PqExtendAutomorphisms( procId );
>     PqCollect( procId, "x5^4" );
>     PqEchelonise( procId );
>     PqApplyAutomorphisms( procId, 15 ); #queue factor = 15
>     PqEliminateRedundantGenerators( procId );
>     PqDisplayPcPresentation( procId );
> od;
Computing class 3
Computing class 4
Computing class 5
Computing class 6
Computing class 7
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
Computing class 8
Collecting [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
Computing class 9
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
Computing class 10
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
Computing class 11
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x4", "x5", "x5", "x5" 
 ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x4", "x3" 
 ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
Computing class 12
Collecting [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x5", 
  "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x2", "x4", "x3", "x5", "x4", "x5", 
  "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x4", "x5", "x5", "x5" 
 ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x4", "x3" 
 ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
Computing class 13
Collecting [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x4", 
  "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x5", 
  "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x2", "x4", "x3", "x5", "x4", "x5", 
  "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x4", "x5", "x5", "x5" 
 ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x1", "x3", "x4", "x2", "x4", "x3" 
 ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x2", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x5", "x5", "x5" ]
Collecting [ "x2", "x1", "x1", "x3", "x4", "x4", "x4" ]
gap> 
gap> PqWritePcPresentation( procId, "/tmp/B54" );
gap> PqQuit( procId );
gap> ##Example: "B5-5-Engel3-Id" . . . Burnside group satisfying 2 identities
gap> ##Generates largest Burnside group of exponent 5 that also satisfies
gap> ##a 3-Engel identity. Demonstrates the usage of the `Identities' option.
gap> F := FreeGroup(2);
<free group on the generators [ f1, f2 ]>
gap> Burnside5 := x->x^5;
function( x ) ... end
gap> Engel3 := function( x,y ) return PqLeftNormComm( [x,y,y,y] ); end;
function( x, y ) ... end
gap> Pq( F : Prime := 5, Identities := [ Burnside5, Engel3 ] );
#I  Class 1 with 2 generators.
#I  Class 2 with 3 generators.
#I  Class 3 with 5 generators.
#I  Class 3 with 5 generators.
<pc group of size 3125 with 5 generators>
gap> # Executing interactive variant of example: "B5-5-Engel3-Id"
gap> ##Example: "B5-5-Engel3-Id" . . . Burnside group satisfying 2 identities
gap> ##Generates largest Burnside group of exponent 5 that also satisfies
gap> ##a 3-Engel identity. Demonstrates the usage of the `Identities' option.
gap> F := FreeGroup(2);
<free group on the generators [ f1, f2 ]>
gap> Burnside5 := x->x^5;
function( x ) ... end
gap> Engel3 := function( x,y ) return PqLeftNormComm( [x,y,y,y] ); end;
function( x, y ) ... end
gap> procId := PqStart( F );
1
gap> Pq( procId : Prime := 5, Identities := [ Burnside5, Engel3 ] );
#I  Class 1 with 2 generators.
#I  Class 2 with 3 generators.
#I  Class 3 with 5 generators.
#I  Class 3 with 5 generators.
<pc group of size 3125 with 5 generators>
gap> ##Example: "EpimorphismStandardPresentation" . . . based on manual example
gap> ##(demonstrates `EpimorphismStandardPresentation' usage)
gap> F := FreeGroup(6, "F");
<free group on the generators [ F1, F2, F3, F4, F5, F6 ]>
gap> # For printing GAP uses the symbols F1, ... for the generators of F
gap> x := F.1; y := F.2; z := F.3; w := F.4; a := F.5; b := F.6;
F1
F2
F3
F4
F5
F6
gap> R := [x^3 / w, y^3 / w * a^2 * b^2, w^3 / b,
>       Comm (y, x) / z, Comm (z, x), Comm (z, y) / a, z^3 ];
[ F1^3*F4^-1, F2^3*F4^-1*F5^2*F6^2, F4^3*F6^-1, F2^-1*F1^-1*F2*F1*F3^-1, 
  F3^-1*F1^-1*F3*F1, F3^-1*F2^-1*F3*F2*F5^-1, F3^3 ]
gap> Q := F / R;
<fp group on the generators [ F1, F2, F3, F4, F5, F6 ]>
gap> # For printing GAP also uses the symbols F1, ... for the generators of Q
gap> # (the same as used for F) ... but the gen'rs of Q and F are different:
gap> GeneratorsOfGroup(F) = GeneratorsOfGroup(Q);
false
gap> G := Pq( Q : Prime := 3, ClassBound := 3 );
<pc group of size 729 with 6 generators>
gap> phi := EpimorphismStandardPresentation( Q : Prime := 3,
>                                             ClassBound := 3 );
[ F1, F2, F3, F4, F5, F6 ] -> [ f1*f2^2*f3*f4^2*f5^2, f1*f2*f3*f5, f3^2, 
  f4*f6^2, f5, f6 ]
gap> Source(phi); # This is the group Q (GAP uses F1, ... for gen'r symbols)
<fp group of size infinity on the generators [ F1, F2, F3, F4, F5, F6 ]>
gap> Range(phi);  # This is the group G (GAP uses f1, ... for gen'r symbols)
<fp group on the generators [ f1, f2, f3, f4, f5, f6 ]>
gap> AssignGeneratorVariables(G); # so f1, ... are now variables
#I  Assigned the global variables [ f1, f2, f3, f4, f5, f6 ]
gap> # Just to see that the images of [F1, ..., F6] do generate G
gap> Group([ f1*f2^2*f3, f1*f2*f3*f4*f5^2*f6^2, f3^2, f4, f5, f6 ]) = G;
true
gap> Size( Image(phi) );
729
gap> # Executing interactive variant of example: "EpimorphismStandardPresentation"
gap> ##Example: "EpimorphismStandardPresentation" . . . based on manual example
gap> ##(demonstrates `EpimorphismStandardPresentation' usage)
gap> F := FreeGroup(6, "F");
<free group on the generators [ F1, F2, F3, F4, F5, F6 ]>
gap> # For printing GAP uses the symbols F1, ... for the generators of F
gap> x := F.1; y := F.2; z := F.3; w := F.4; a := F.5; b := F.6;
F1
F2
F3
F4
F5
F6
gap> R := [x^3 / w, y^3 / w * a^2 * b^2, w^3 / b,
>       Comm (y, x) / z, Comm (z, x), Comm (z, y) / a, z^3 ];
[ F1^3*F4^-1, F2^3*F4^-1*F5^2*F6^2, F4^3*F6^-1, F2^-1*F1^-1*F2*F1*F3^-1, 
  F3^-1*F1^-1*F3*F1, F3^-1*F2^-1*F3*F2*F5^-1, F3^3 ]
gap> Q := F / R;
<fp group on the generators [ F1, F2, F3, F4, F5, F6 ]>
gap> # For printing GAP also uses the symbols F1, ... for the generators of Q
gap> # (the same as used for F) ... but the gen'rs of Q and F are different:
gap> GeneratorsOfGroup(F) = GeneratorsOfGroup(Q);
false
gap> procId := PqStart( Q );
2
gap> G := Pq( procId : Prime := 3, ClassBound := 3 );
<pc group of size 729 with 6 generators>
gap> phi := EpimorphismStandardPresentation( procId : Prime := 3,
>                                             ClassBound := 3 );
[ F1, F2, F3, F4, F5, F6 ] -> [ f1*f2^2*f3*f4^2*f5^2, f1*f2*f3*f5, f3^2, 
  f4*f6^2, f5, f6 ]
gap> Source(phi); # This is the group Q (GAP uses F1, ... for gen'r symbols)
<fp group of size infinity on the generators [ F1, F2, F3, F4, F5, F6 ]>
gap> Range(phi);  # This is the group G (GAP uses f1, ... for gen'r symbols)
<fp group on the generators [ f1, f2, f3, f4, f5, f6 ]>
gap> AssignGeneratorVariables(G); # so f1, ... are now variables
#I  Global variable `f1' is already defined and will be overwritten
#I  Global variable `f2' is already defined and will be overwritten
#I  Global variable `f3' is already defined and will be overwritten
#I  Global variable `f4' is already defined and will be overwritten
#I  Global variable `f5' is already defined and will be overwritten
#I  Global variable `f6' is already defined and will be overwritten
#I  Assigned the global variables [ f1, f2, f3, f4, f5, f6 ]
gap> # Just to see that the images of [F1, ..., F6] do generate G
gap> Group([ f1*f2^2*f3, f1*f2*f3*f4*f5^2*f6^2, f3^2, f4, f5, f6 ]) = G;
true
gap> Size( Image(phi) );
729
gap> PqQuitAll();
gap> ##Example: "F2-5-i" . . . based on: examples/F2-5
gap> ##Construction of 5-quotient of a 2-generator free group
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 5);
1
gap> PqPcPresentation(procId : ClassBound := 6, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^32
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^55
gap> PqNextClass(procId); #class 7
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^96
gap> PqNextClass(procId); #class 8
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^167
gap> PqNextClass(procId); #class 9
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^294
gap> ##Example: "G2-SP-Rel-i" . . . based on: isom/G_2.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["b^4", "b^2 * [b, a, a]^-1", "b * (a^2 * b^-1 * a^2)^-1" ];
[ "b^4", "b^2 * [b, a, a]^-1", "b * (a^2 * b^-1 * a^2)^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[0,1],
>                                     [1,1]],
> 
>                                    [[0,1],
>                                     [1,0]] ]
> 
>                          : # options
>                            ClassBound := 12,
>                            PcgsAutomorphisms);
gap> ##Example: "G3-SP-Rel-i" . . . based on: isom/G_3.com
gap> F := FreeGroup("x", "y");
<free group on the generators [ x, y ]>
gap> rels := ["(x * y * x)^3"];
[ "(x * y * x)^3" ]
gap> procId := PqStart(F : Prime := 3, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[2,0],
>                                     [0,2]],
> 
>                                    [[0,2],
>                                     [1,0]],
> 
>                                    [[1,2],
>                                     [2,2]],
> 
>                                    [[1,0],
>                                     [2,1]],
> 
>                                    [[2,0],
>                                     [0,1]] ]
> 
>                          : # options
>                            ClassBound := 5,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "G5-SP-Rel-i" . . . based on: isom/G_5.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["[a, b, b, b]", "[b, a, b, b, b]", "[a, b, a]", "b^5"];
[ "[a, b, b, b]", "[b, a, b, b, b]", "[a, b, a]", "b^5" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
1
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,3,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,3,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "G5-SP-a-Rel-i" . . . based on: isom/G_5A.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["[a, b, b, a]", "[a, b, a]", "b^5"];
[ "[a, b, b, a]", "[a, b, a]", "b^5" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
2
gap> PqSPComputePcpAndPCover(procId : ClassBound := 2);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,1],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,3,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,3,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,2,0,0],
>                                     [0,3,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[3,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> ##Example: "IsIsomorphicPGroup-ni" . . . based on manual example
gap> ##(demonstrates `IsIsomorphicPGroup' usage)
gap> G := Group( (1,2,3,4), (1,3) );
Group([ (1,2,3,4), (1,3) ])
gap> P1 := Image( IsomorphismPcGroup( G ) );
Group([ f1, f2, f3 ])
gap> P2 := SmallGroup( 8, 5 );
<pc group of size 8 with 3 generators>
gap> IsIsomorphicPGroup( P1, P2 );
false
gap> P3 := SmallGroup( 8, 4 );
<pc group of size 8 with 3 generators>
gap> IsIsomorphicPGroup( P1, P3 );
false
gap> P4 := SmallGroup( 8, 3 );
<pc group of size 8 with 3 generators>
gap> IsIsomorphicPGroup( P1, P4 );
true
gap> PqQuitAll();
gap> ##Example: "Nott-APG-Rel-i" . . . based on: examples/pga_interactive
gap> ##Start point for Nottingham group
gap> ##Interactive construction of 40 5-groups of order 5^7
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^5", "b^5", "[b, a, b]"];
[ "a^5", "b^5", "[b, a, b]" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
1
gap> PqPcPresentation(procId : ClassBound := 3, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^4
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqPGSupplyAutomorphisms(procId, [ [[1,0,0,0],
>                                    [0,1,0,1]],
> 
>                                   [[1,1,0,0],
>                                    [0,1,0,1]],
> 
>                                   [[1,0,0,0],
>                                    [0,4,0,0]],
> 
>                                   [[1,0,0,0],
>                                    [0,2,0,0]],
> 
>                                   [[4,0,0,0],
>                                    [0,1,0,0]],
> 
>                                   [[2,0,0,0],
>                                    [0,1,0,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 4,
>                                   CapableDescendants,
>                                   StepSize := 1,
>                                   PcgsAutomorphisms,
>                                   RankInitialSegmentSubgroups := 4);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 5^4
#I  Nuclear rank: 1
#I  5-multiplicator rank: 4
#I  # of immediate descendants of order 5^5 is 9
#I  # of capable immediate descendants is 2
#I  **************************************************
2
gap> PqPGSetDescendantToPcp(procId, 4, 1);
gap> PqAPGDegree(procId, 2, 3);
#I  Degree of permutation group is 25
25
gap> PqAPGPermutations(procId);
gap> PqAPGOrbits(procId : CustomiseOutput := rec(orbit := [1]));
#I    Orbit          Length      Representative
#I        1               5               1
#I        2              20               2
#I  Number of orbits is 2
2
gap> PqAPGOrbitRepresentatives(procId);
gap> PqPGSetDescendantToPcp(procId);
gap> PqDisplayPcPresentation(procId);
#I  Group: [grp] #1;1 to lower exponent-5 central class 5 has order 5^9
#I  Class 1
#I  1 is defined on image of defining generator 1
#I  2 is defined on image of defining generator 2
#I  Class 2
#I  3 is defined on [2, 1] = 2 1 
#I  Class 3
#I  4 is defined on [3, 1] = 2 1 1 
#I  Class 4
#I  5 is defined on [4, 1] = 2 1 1 1 
#I  Class 5
#I  6 is defined on [5, 1] = 2 1 1 1 1 
#I  7 is defined on [5, 2] = 2 1 1 1 2 
#I  8 is defined on 1^5 = 1 1 
#I  9 is defined on 2^5 = 2 2 
#I  Non-trivial powers:
#I   .1^5 = .8
#I   .2^5 = .9
#I  Non-trivial commutators:
#I  [ .2, .1 ] = .3
#I  [ .3, .1 ] = .4
#I  [ .4, .1 ] = .5
#I  [ .4, .2 ] = .7
#I  [ .4, .3 ] = .7^4
#I  [ .5, .1 ] = .6
#I  [ .5, .2 ] = .7
gap> PqAPGSingleStage(procId : StepSize:=2, BasicAlgorithm, 
>                           CustomiseOutput := rec());
#I  **************************************************
#I  Starting group: [grp] #1;1
#I  Order: 5^5
#I  Nuclear rank: 2
#I  5-multiplicator rank: 4
#I  # of immediate descendants of order 5^7 is 40
#I  # of capable immediate descendants is 5
gap> ##Example: "Nott-PG-Rel-i" . . . based on: examples/pga_nott
gap> ##Start point for Nottingham group
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^5", "b^5", "[b, a, b]"];
[ "a^5", "b^5", "[b, a, b]" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
2
gap> PqPcPresentation(procId : ClassBound := 3, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^4
gap> PqComputePCover(procId);
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> PqPGSupplyAutomorphisms(procId, [ [[1,0,0,0],
>                                    [0,1,0,1]],
> 
>                                   [[1,1,0,0],
>                                    [0,1,0,1]],
> 
>                                   [[1,0,0,0],
>                                    [0,4,0,0]],
> 
>                                   [[1,0,0,0],
>                                    [0,2,0,0]],
> 
>                                   [[4,0,0,0],
>                                    [0,1,0,0]],
> 
>                                   [[2,0,0,0],
>                                    [0,1,0,0]] ]);
gap> PqPGConstructDescendants(procId : ClassBound := 4,
>                                   CapableDescendants,
>                                   StepSize := 1,
>                                   PcgsAutomorphisms,
>                                   RankInitialSegmentSubgroups := 4);
#I  **************************************************
#I  Starting group: [grp]
#I  Order: 5^4
#I  Nuclear rank: 1
#I  5-multiplicator rank: 4
#I  # of immediate descendants of order 5^5 is 9
#I  # of capable immediate descendants is 2
#I  **************************************************
2
gap> ##Example: "Nott-SP-Rel-i" . . . based on: isom/nott.com
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^5", "b^5", "[b, a, b]"];
[ "a^5", "b^5", "[b, a, b]" ]
gap> procId := PqStart(F : Prime := 5, Relators := rels);
3
gap> PqSPComputePcpAndPCover(procId : ClassBound := 3);
gap> PqSPStandardPresentation(procId, [ [[1,0,0,0],
>                                     [0,1,0,1]],
> 
>                                    [[1,1,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,4,0,0]],
> 
>                                    [[1,0,0,0],
>                                     [0,2,0,0]],
> 
>                                    [[4,0,0,0],
>                                     [0,1,0,0]],
> 
>                                    [[2,0,0,0],
>                                     [0,1,0,0]] ]
> 
>                          : # options
>                            ClassBound := 10,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> ##Example: "Pq" . . . based on manual example illustrating `Pq' usage
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> Pq( F : Prime := 2, ClassBound := 3 );
<pc group of size 1024 with 10 generators>
gap> 
gap> # Now let us get a p-quotient of an fp group
gap> G := F / [a^4, b^4];
<fp group on the generators [ a, b ]>
gap> Pq( G : Prime := 2, ClassBound := 3 );
<pc group of size 256 with 8 generators>
gap> 
gap> # Now let's get a different p-quotient of the same group
gap> Pq( G : Prime := 2, ClassBound := 3,
>               Exponent := 4 );
<pc group of size 128 with 7 generators>
gap> 
gap> # Now we'll get a p-quotient of another fp group
gap> # which we will redo using the `Relators' option
gap> R := [ a^25, Comm(Comm(b, a), a), b^5 ];
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> Pq( H : Prime := 5, ClassBound := 5, Metabelian );
<pc group of size 78125 with 7 generators>
gap> 
gap> # Now we redo the previous example using the `Relators' option
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> # `F' was defined for `Relators'. We use the same strings that GAP uses
gap> # for printing the free group generators. It is *not* necessary to
gap> # predefine: a := F.1; etc. (as it was above).
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> R := PqGAPRelators(F, rels);
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> Pq( H : Prime := 5, ClassBound := 5, Metabelian, 
>               Relators := rels );
<pc group of size 78125 with 7 generators>
gap> 
gap> # Above we could have just passed `F' (rather than `H'):
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> Pq( F : Prime := 5, ClassBound := 5, Metabelian, 
>         Relators := rels );
<pc group of size 78125 with 7 generators>
gap> # Executing interactive variant of example: "Pq"
gap> ##Example: "Pq" . . . based on manual example illustrating `Pq' usage
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> procId1 := PqStart( F );
1
gap> Pq( procId1 : Prime := 2, ClassBound := 3 );
<pc group of size 1024 with 10 generators>
gap> 
gap> # Now let us get a p-quotient of an fp group
gap> G := F / [a^4, b^4];
<fp group on the generators [ a, b ]>
gap> procId2 := PqStart( G );
2
gap> Pq( procId2 : Prime := 2, ClassBound := 3 );
<pc group of size 256 with 8 generators>
gap> 
gap> # Now let's get a different p-quotient of the same group
gap> Pq( procId2 : Prime := 2, ClassBound := 3,
>               RedoPcp,
>               Exponent := 4 );
<pc group of size 128 with 7 generators>
gap> 
gap> # Now we'll get a p-quotient of another fp group
gap> # which we will redo using the `Relators' option
gap> R := [ a^25, Comm(Comm(b, a), a), b^5 ];
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> procId3 := PqStart( H );
3
gap> Pq( procId3 : Prime := 5, ClassBound := 5, Metabelian );
<pc group of size 78125 with 7 generators>
gap> 
gap> # Now we redo the previous example using the `Relators' option
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> # `F' was defined for `Relators'. We use the same strings that GAP uses
gap> # for printing the free group generators. It is *not* necessary to
gap> # predefine: a := F.1; etc. (as it was above).
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> R := PqGAPRelators(F, rels);
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> Pq( procId3 : Prime := 5, ClassBound := 5, Metabelian, 
>               RedoPcp,
>               Relators := rels );
<pc group of size 78125 with 7 generators>
gap> 
gap> # Above we could have just passed `F' (rather than `H'):
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> procId4 := PqStart( F );
4
gap> Pq( procId4 : Prime := 5, ClassBound := 5, Metabelian, 
>         Relators := rels );
<pc group of size 78125 with 7 generators>
gap> ##Example: "Pq-ni" . . . based on manual example illustrating `Pq' usage
gap> ##(demonstrates the alternative ways of passing options non-interactively)
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> R := PqGAPRelators(F, rels);
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> # This demonstrates how the options may be passed as a record argument
gap> Pq( H, rec( Prime := 5, ClassBound := 5, Metabelian := true ) );
<pc group of size 78125 with 7 generators>
gap> # This demonstrates how the options may be passed as additional arguments
gap> Pq( H, "Prime", 5, "ClassBound", 5, "Metabelian" );
<pc group of size 78125 with 7 generators>
gap> ##Example: "PqDescendants-1" . . . based on manual example
gap> ##(demonstrates `PqDescendants' usage)
gap> F := FreeGroup( "a", "b" ); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := PcGroupFpGroup( F / [ a^2, b^2, Comm(b, a) ] );
<pc group of size 4 with 2 generators>
gap> des := PqDescendants( G : OrderBound := 6, ClassBound := 5 );
[ <pc group of size 8 with 3 generators>, 
  <pc group of size 8 with 3 generators>, 
  <pc group of size 8 with 3 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators> ]
gap> Length(des);
83
gap> List(des, Size);
[ 8, 8, 8, 16, 16, 16, 32, 16, 16, 16, 16, 16, 32, 32, 64, 64, 32, 32, 32, 
  32, 32, 32, 32, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 32, 32, 32, 32, 
  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 32, 32, 32, 32, 32, 64, 64, 64, 
  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 
  64, 64, 64, 64, 64, 64, 64 ]
gap> List(des, d -> Length( PCentralSeries( d, 2 ) ) - 1 );
[ 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
  4, 4, 4, 5, 5, 5, 5, 5 ]
gap> # Executing interactive variant of example: "PqDescendants-1"
gap> ##Example: "PqDescendants-1" . . . based on manual example
gap> ##(demonstrates `PqDescendants' usage)
gap> F := FreeGroup( "a", "b" ); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := PcGroupFpGroup( F / [ a^2, b^2, Comm(b, a) ] );
<pc group of size 4 with 2 generators>
gap> procId := PqStart( G );
5
gap> des := PqDescendants( procId : OrderBound := 6, ClassBound := 5 );
[ <pc group of size 8 with 3 generators>, 
  <pc group of size 8 with 3 generators>, 
  <pc group of size 8 with 3 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators> ]
gap> Length(des);
83
gap> List(des, Size);
[ 8, 8, 8, 16, 16, 16, 32, 16, 16, 16, 16, 16, 32, 32, 64, 64, 32, 32, 32, 
  32, 32, 32, 32, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 32, 32, 32, 32, 
  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 32, 32, 32, 32, 32, 64, 64, 64, 
  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 
  64, 64, 64, 64, 64, 64, 64 ]
gap> List(des, d -> Length( PCentralSeries( d, 2 ) ) - 1 );
[ 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
  4, 4, 4, 5, 5, 5, 5, 5 ]
gap> PqQuitAll();
gap> ##Example: "PqDescendants-1-i" . . . based on manual example
gap> ##(demonstrates `PqSetPQuotientToGroup' usage)
gap> F := FreeGroup( "a", "b" );
<free group on the generators [ a, b ]>
gap> procId := PqStart( F : Prime := 2 );
1
gap> Pq( procId : ClassBound := 1 );
<pc group of size 4 with 2 generators>
gap> PqSetPQuotientToGroup( procId );
gap> des := PqDescendants( procId : OrderBound := 6, ClassBound := 5 );
[ <pc group of size 8 with 3 generators>, 
  <pc group of size 8 with 3 generators>, 
  <pc group of size 8 with 3 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 16 with 4 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 32 with 5 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators>, 
  <pc group of size 64 with 6 generators> ]
gap> Length(des);
83
gap> List(des, Size);
[ 8, 8, 8, 16, 16, 16, 32, 16, 16, 16, 16, 16, 32, 32, 64, 64, 32, 32, 32, 
  32, 32, 32, 32, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 32, 32, 32, 32, 
  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 32, 32, 32, 32, 32, 64, 64, 64, 
  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 
  64, 64, 64, 64, 64, 64, 64 ]
gap> List(des, d -> Length( PCentralSeries( d, 2 ) ) - 1 );
[ 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 
  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 
  4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 
  4, 4, 4, 5, 5, 5, 5, 5 ]
gap> ##Example: "PqDescendants-2" . . . based on manual example
gap> ##(demonstrates `PqDescendants' usage)
gap> ##We compute all capable descendants of order 27
gap> ##of an elementary abelian group of order 9.
gap> F := FreeGroup( 2, "g" );
<free group on the generators [ g1, g2 ]>
gap> G := PcGroupFpGroup( F / [ F.1^3, F.2^3, Comm(F.1, F.2) ] );
<pc group of size 9 with 2 generators>
gap> des := PqDescendants( G : OrderBound := 3, ClassBound := 2,
>                           CapableDescendants );
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> List(des, d -> Length( PCentralSeries( d, 3 ) ) - 1 );
[ 2, 2 ]
gap> # For comparison let us now compute all descendants
gap> PqDescendants( G : OrderBound := 3, ClassBound := 2);
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> # Executing interactive variant of example: "PqDescendants-2"
gap> ##Example: "PqDescendants-2" . . . based on manual example
gap> ##(demonstrates `PqDescendants' usage)
gap> ##We compute all capable descendants of order 27
gap> ##of an elementary abelian group of order 9.
gap> F := FreeGroup( 2, "g" );
<free group on the generators [ g1, g2 ]>
gap> G := PcGroupFpGroup( F / [ F.1^3, F.2^3, Comm(F.1, F.2) ] );
<pc group of size 9 with 2 generators>
gap> procId := PqStart( G );
2
gap> des := PqDescendants( procId : OrderBound := 3, ClassBound := 2,
>                           CapableDescendants );
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> List(des, d -> Length( PCentralSeries( d, 3 ) ) - 1 );
[ 2, 2 ]
gap> # For comparison let us now compute all descendants
gap> PqDescendants( G : OrderBound := 3, ClassBound := 2);
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> ##Example: "PqDescendants-3" . . . based on manual example
gap> ##(demonstrates `PqDescendants' usage)
gap> ##We compute all capable descendants of order 25 which have
gap> ##exponent-5 class at most 3, exponent 5 and are metabelian.
gap> F := FreeGroup( 2, "g" );
<free group on the generators [ g1, g2 ]>
gap> G := PcGroupFpGroup( F / [ F.1^5, F.2^5, Comm(F.2, F.1) ] );
<pc group of size 25 with 2 generators>
gap> des := PqDescendants( G : Metabelian, ClassBound := 3,
>                           Exponent := 5, CapableDescendants );
[ <pc group of size 125 with 3 generators>, 
  <pc group of size 625 with 4 generators>, 
  <pc group of size 3125 with 5 generators> ]
gap> List(des, d -> Length( PCentralSeries( d, 5 ) ) - 1 );
[ 2, 3, 3 ]
gap> List(des, d -> Length( DerivedSeries( d ) ) );
[ 3, 3, 3 ]
gap> List(des, d -> Maximum( List( d, Order ) ) );
[ 5, 5, 5 ]
gap> # Executing interactive variant of example: "PqDescendants-3"
gap> ##Example: "PqDescendants-3" . . . based on manual example
gap> ##(demonstrates `PqDescendants' usage)
gap> ##We compute all capable descendants of order 25 which have
gap> ##exponent-5 class at most 3, exponent 5 and are metabelian.
gap> F := FreeGroup( 2, "g" );
<free group on the generators [ g1, g2 ]>
gap> G := PcGroupFpGroup( F / [ F.1^5, F.2^5, Comm(F.2, F.1) ] );
<pc group of size 25 with 2 generators>
gap> procId := PqStart( G );
3
gap> des := PqDescendants( procId : Metabelian, ClassBound := 3,
>                           Exponent := 5, CapableDescendants );
[ <pc group of size 125 with 3 generators>, 
  <pc group of size 625 with 4 generators>, 
  <pc group of size 3125 with 5 generators> ]
gap> List(des, d -> Length( PCentralSeries( d, 5 ) ) - 1 );
[ 2, 3, 3 ]
gap> List(des, d -> Length( DerivedSeries( d ) ) );
[ 3, 3, 3 ]
gap> List(des, d -> Maximum( List( d, Order ) ) );
[ 5, 5, 5 ]
gap> PqQuitAll();
gap> ##Example: "PqDescendants-treetraverse-i" . . . based on manual example
gap> ##(demonstrates how one may use the p-group generation machinery to
gap> ## traverse a descendants tree, by exploring the 3-groups of rank 2
gap> ## and 3-coclass 1 up to 3-class 5.)
gap> G := ElementaryAbelianGroup( 9 );
<pc group of size 9 with 2 generators>
gap> procId := PqStart( G );
1
gap> #
gap> #  Below, we use the option StepSize in order to construct descendants
gap> #  of coclass 1. This is equivalent to setting the StepSize to 1 in
gap> #  each descendant calculation.
gap> #
gap> #  The elementary abelian group of order 9 has 3 descendants of
gap> #  3-class 2 and 3-coclass 1, as the result of the next command
gap> #  shows. 
gap> #
gap> PqDescendants( procId : StepSize := 1 );
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> #
gap> #  Now we will compute the descendants of coclass 1 for each of the
gap> #  groups above. Then we will compute the descendants  of coclass 1
gap> #  of each descendant and so  on.  Note  that the  pq program keeps
gap> #  one file for each class at a time.  For example, the descendants
gap> #  calculation for  the  second  group  of class  2  overwrites the
gap> #  descendant file  obtained  from  the  first  group  of  class 2.
gap> #  Hence,  we have to traverse the descendants tree  in depth first
gap> #  order.
gap> #
gap> PqPGSetDescendantToPcp( procId, 2, 1 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
2
gap> PqPGSetDescendantToPcp( procId, 3, 1 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
2
gap> PqPGSetDescendantToPcp( procId, 4, 1 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
2
gap> #
gap> #  At this point we stop traversing the ``left most'' branch of the
gap> #  descendants tree and move upwards.
gap> #
gap> PqPGSetDescendantToPcp( procId, 4, 2 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> PqPGSetDescendantToPcp( procId, 3, 2 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> #  
gap> #  The computations above indicate that the descendants subtree under
gap> #  the first descendant of the elementary abelian group of order 9
gap> #  will have only one path of infinite length.
gap> #
gap> PqPGSetDescendantToPcp( procId, 2, 2 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
4
gap> #
gap> #  We get four descendants here, three of which will turn out to be
gap> #  incapable, i.e., they have no descendants and are terminal nodes
gap> #  in the descendants tree.
gap> #
gap> PqPGSetDescendantToPcp( procId, 2, 3 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> #
gap> #  The third descendant of class three is incapable.  Let us return
gap> #  to the second descendant of class 2.
gap> #
gap> PqPGSetDescendantToPcp( procId, 2, 2 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
4
gap> PqPGSetDescendantToPcp( procId, 3, 1 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> PqPGSetDescendantToPcp( procId, 3, 2 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> #
gap> #  We skip the third descendant for the moment ... 
gap> #
gap> PqPGSetDescendantToPcp( procId, 3, 4 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> #
gap> #  ... and look at it now.
gap> #
gap> PqPGSetDescendantToPcp( procId, 3, 3 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
6
gap> #
gap> #  In this branch of the descendant tree we get 6 descendants of class
gap> #  three.  Of those 5 will turn out to be incapable and one will have
gap> #  7 descendants.
gap> #
gap> PqPGSetDescendantToPcp( procId, 4, 1 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> PqPGSetDescendantToPcp( procId, 4, 2 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
7
gap> PqPGSetDescendantToPcp( procId, 4, 3 );
gap> PqPGExtendAutomorphisms( procId );
gap> PqPGConstructDescendants( procId : StepSize := 1 );
#I  group restored from file is incapable
0
gap> ##Example: "PqDescendantsTreeCoclassOne-16-i" . based on manual example
gap> ##(demonstrates usage of `PqDescendantsTreeCoclassOne', by generating
gap> ## all descendants of the elementary abelian group of order 16 of
gap> ## 2-coclass 1 up to 2-class 15. Most effective in XGAP.)
gap> G := ElementaryAbelianGroup( 16 );
<pc group of size 16 with 4 generators>
gap> procId := PqStart( G );
2
gap> PqDescendantsTreeCoclassOne( procId : TreeDepth := 15, 
>                                       CapableDescendants );
#I  Number of descendants of group #1;1 to class 2: 1
#I  Number of descendants of group #1;1 #1;1 to class 3: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 to class 4: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 to class 5: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 to class 6: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 7: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 8: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 9: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 10: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 11: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 12: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 13: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 14: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 15: 1
#I  Number of descendants of group #2;1 to class 2: 1
#I  Number of descendants of group #1;1 #1;1 to class 3: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 to class 4: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 to class 5: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 to class 6: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 7: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 8: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 9: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 10: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 11: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 12: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 13: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 14: 1
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 15: 1
gap> ##Example: "PqDescendantsTreeCoclassOne-25-i" . based on manual example
gap> ##(demonstrates usage of `PqDescendantsTreeCoclassOne', by generating
gap> ## all descendants of the elementary abelian group of order 25 of
gap> ## 5-coclass 1 up to 5-class 15. Most effective in XGAP.)
gap> G := ElementaryAbelianGroup( 25 );
<pc group of size 25 with 2 generators>
gap> procId := PqStart( G );
3
gap> PqDescendantsTreeCoclassOne( procId : TreeDepth := 15 );
#I  Number of descendants of group #1;1 to class 2: 2
#I  Number of descendants of group #1;1 #1;1 to class 3: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 to class 4: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 to class 5: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 to class 6: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 7: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 8: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 9: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 10: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 11: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 12: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 13: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 14: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 to class 15: 2
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #1;1 #2;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 #2;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #2;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #2;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #2;1 to class 3: 0
#I  Number of descendants of group #2;1 to class 2: 4
#I  Number of descendants of group #2;1 #1;1 to class 3: 9
#I  Number of descendants of group #2;1 #1;1 #1;1 to class 4: 21
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #1;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #2;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #3;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #4;1 to class 5: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 to class 5: 12
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #1;1 to class 6: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 to class 6: 14
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #1;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #2;1 to class 7: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 to class 7: 12
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #1;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 to class 8: 18
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #1;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 to class 9: 12
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #1;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 to class 10: 14
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #2;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 to class 11: 12
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #1;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 to class 12: 18
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 to class 13: 12
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #1;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 to class 14: 14
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #2;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #3;1 to class 15: 12
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #6;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #7;1 to class 15: 17
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #2;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #4;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 to class 14: 18
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #4;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #5;1 to class 15: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #11;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #12;1 to class 15: 26
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #5;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #1;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #6;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 to class 13: 17
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #1;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 to class 14: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #30;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #31;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #32;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #33;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #34;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #35;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #36;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #37;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #38;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #39;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #40;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #41;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #42;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #43;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #44;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #45;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #46;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #47;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #48;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #2;1 #49;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #10;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 to class 14: 37
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #27;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #28;1 to class 15: 8
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #30;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #31;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #32;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #33;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #34;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #35;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #36;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #11;1 #37;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #7;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #2;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #4;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 to class 12: 20
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 to class 13: 26
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #19;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #20;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 to class 14: 9
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #7;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #8;1 to class 15: 16
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #21;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #1;1 #26;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #6;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 to class 13: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #19;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #20;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #26;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #27;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #30;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #31;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #32;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #33;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #34;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #35;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #36;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #7;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #8;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #37;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #38;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #39;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #40;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #41;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #42;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #43;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #44;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #45;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #46;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #47;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #48;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #7;1 #49;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #5;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #3;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #6;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 to class 11: 17
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #4;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 to class 12: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #26;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #29;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #30;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #31;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #32;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #33;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #34;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #35;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 to class 13: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #19;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #17;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #18;1 to class 15: 145
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #20;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #26;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #27;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #36;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #37;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #38;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #39;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #40;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #41;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #42;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #43;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #44;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #45;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #46;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #47;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #48;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #5;1 #49;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #10;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 to class 12: 39
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #25;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 to class 13: 8
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #6;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 to class 14: 15
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #11;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #12;1 to class 15: 37
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #7;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #26;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #29;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #30;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #31;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #32;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #33;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #34;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #35;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #36;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #37;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #38;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #11;1 #39;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #7;1 #17;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #2;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #4;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 to class 10: 18
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #2;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 to class 11: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #17;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #18;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #19;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #20;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #21;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #22;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #23;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #24;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #25;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #26;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #27;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #28;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #29;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #30;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #31;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #32;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #33;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #34;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #35;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #36;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #37;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #38;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #39;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 to class 12: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #18;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 to class 13: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #19;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 to class 14: 145
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #30;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #31;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #32;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #33;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #34;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #35;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #36;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #37;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #38;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #39;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #40;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #41;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #42;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #43;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #44;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #45;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #46;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #47;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #48;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #49;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #50;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #51;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #52;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #53;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #54;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #55;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #56;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #57;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #58;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #59;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #60;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #61;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #62;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #63;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #64;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #65;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #66;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #67;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #68;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #69;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #70;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #71;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #72;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #73;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #74;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #75;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #76;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #77;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #78;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #79;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #80;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #81;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #82;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #83;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #84;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #85;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #86;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #87;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #88;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #89;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #90;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #91;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #92;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #93;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #94;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #95;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #96;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #97;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #98;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #99;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #100;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #101;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #102;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #103;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #104;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #105;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #106;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #107;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #108;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #109;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #110;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #111;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #112;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #113;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #114;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #115;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #116;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #117;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #118;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #119;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #120;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #121;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #122;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #123;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #124;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #125;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #126;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #127;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #128;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #129;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #130;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #131;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #132;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #133;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #134;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #135;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #136;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #137;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #138;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #139;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #140;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #141;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #142;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #143;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #144;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #20;1 #145;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #26;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #27;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #19;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #26;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #40;1 #29;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #41;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #42;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #43;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #44;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #45;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #46;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #47;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #48;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #3;1 #49;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #11;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 to class 11: 26
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #17;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #18;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #19;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #20;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 to class 12: 11
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #8;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 to class 13: 16
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #6;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 to class 14: 38
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #1;1 to class 15: 16
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #4;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #5;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #30;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #31;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #32;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #33;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #34;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #35;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #36;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #37;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #7;1 #38;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #9;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #21;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #22;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #23;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #24;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #25;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #12;1 #26;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #16;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #5;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #6;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #7;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #10;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #11;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #2;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #7;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 to class 9: 17
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #3;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 to class 10: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #16;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #19;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #20;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #21;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #22;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #23;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #24;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #25;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #26;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #27;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #28;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #29;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #30;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #31;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #32;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #33;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #34;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #35;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #36;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #37;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 to class 11: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #17;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 to class 12: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #18;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 to class 13: 145
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #19;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #20;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #26;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #27;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #30;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #31;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #32;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #33;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #34;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #35;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #36;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #37;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #38;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #39;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #40;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #41;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #42;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #43;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #44;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #45;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #46;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #47;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #48;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #49;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #50;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #51;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #52;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #53;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #54;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #20;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #21;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #55;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #56;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #57;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #58;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #59;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #60;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #61;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #62;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #63;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #20;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #21;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #64;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #65;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #66;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #67;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #68;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #69;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #70;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #71;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #72;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #20;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #21;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #73;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #74;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #75;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #76;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #77;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #78;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #79;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #80;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #81;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #20;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #21;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #82;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #83;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #84;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #85;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #86;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #87;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #88;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #89;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #90;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #20;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #21;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #91;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #92;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #93;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #94;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #95;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #96;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #97;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #98;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #99;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #100;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #101;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #102;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #103;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #104;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #105;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #106;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #107;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #108;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #109;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #110;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #111;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #112;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #113;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #114;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #115;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #116;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #117;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #118;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #119;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #120;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #121;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #122;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #123;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #124;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #125;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #126;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #127;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #128;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #129;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #130;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #131;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #132;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #133;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #134;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #135;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #136;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #137;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #138;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #139;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #140;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #141;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #142;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #143;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #144;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #19;1 #145;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #26;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #18;1 #29;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #19;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #20;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #21;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #22;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #23;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #24;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #25;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #26;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #27;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #28;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #38;1 #29;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #39;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #40;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #41;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #42;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #43;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #44;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #45;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #46;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #47;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #48;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #4;1 #49;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #6;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #7;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #10;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 to class 10: 37
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #16;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #19;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #20;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #21;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #22;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #23;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #24;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #25;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #26;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #27;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 to class 11: 8
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #1;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 to class 12: 17
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 to class 13: 37
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #19;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #20;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #26;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 to class 14: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #12;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #13;1 to class 15: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #27;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #30;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #31;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #32;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #33;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #34;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #35;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 to class 14: 27
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #12;1 to class 15: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #13;1 to class 15: 8
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #36;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #1;1 #37;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #2;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #28;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #29;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #30;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #31;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #32;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #33;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #34;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #35;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #36;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #11;1 #37;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #13;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #14;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #15;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #16;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #8;1 #17;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #10;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #16;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #17;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #2;1 #18;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #3;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #4;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 to class 8: 20
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #1;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 to class 9: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #4;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #6;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #7;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #10;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #11;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #13;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #14;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #15;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #16;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #17;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #18;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #19;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #20;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #21;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #22;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #23;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #24;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #25;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #26;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #27;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #28;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #29;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #30;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #31;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #32;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #33;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #34;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #35;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #36;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #37;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #38;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 to class 10: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #15;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 to class 11: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #17;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 to class 12: 125
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #26;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #29;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #30;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #31;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #32;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #33;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #34;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #35;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #36;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #37;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #38;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #39;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #40;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #41;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #42;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #43;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #44;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #45;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #46;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #47;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #48;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #49;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #50;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #51;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #52;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #53;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 to class 13: 125
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #19;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #20;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #25;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #26;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #27;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #30;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #31;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #32;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #33;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #34;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #35;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #36;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #37;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #38;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #39;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #40;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #41;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #42;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #43;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #44;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #45;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #46;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #47;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #48;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #49;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #50;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #51;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #52;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #53;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #54;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #55;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #56;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #57;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #58;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #59;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #60;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #61;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #62;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #63;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #64;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #65;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #66;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #67;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #68;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #69;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #70;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #71;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #72;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #73;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #74;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #75;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #76;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #77;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #78;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #79;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #80;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #81;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #82;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #83;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #84;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #85;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #86;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #87;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #88;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #89;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #90;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #91;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #92;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #93;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #94;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #95;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #96;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #97;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #98;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #99;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #100;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #101;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #102;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 to class 14: 125
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #30;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #31;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #32;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #33;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #34;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #35;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #36;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #37;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #38;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #39;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #40;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #41;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #42;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #43;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #44;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #45;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #46;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #47;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #48;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #49;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #50;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #51;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #52;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #53;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #54;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #55;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #56;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #57;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #58;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #59;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #60;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #61;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #62;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #63;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #64;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #65;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #66;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #67;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #68;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #69;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #70;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #71;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #72;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #73;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #74;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #75;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #76;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #77;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #78;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #79;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #80;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #81;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #82;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #83;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #84;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #85;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #86;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #87;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #88;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #89;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #90;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #91;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #92;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #93;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #94;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #95;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #96;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #97;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #98;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #99;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #100;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #101;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #102;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #103;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #104;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #105;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #106;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #107;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #108;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #109;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #110;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #111;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #112;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #113;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #114;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #115;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #116;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #117;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #118;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #119;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #120;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #121;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #122;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #123;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #124;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #103;1 #125;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #104;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #105;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #106;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #107;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #108;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #109;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #110;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #111;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #112;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #113;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #114;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #115;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #116;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #117;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #118;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #119;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #120;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #121;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #122;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #123;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #124;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #54;1 #125;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #55;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #56;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #57;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #58;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #59;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #60;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #61;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #62;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #63;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #64;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #65;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #66;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #67;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #68;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #69;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #70;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #71;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #72;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #73;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #74;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #75;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #76;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #77;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #78;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #79;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #80;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #81;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #82;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #83;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #84;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #85;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #86;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #87;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #88;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #89;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #90;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #91;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #92;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #93;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #94;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #95;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #96;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #97;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #98;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #99;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #100;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #101;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #102;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #103;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #104;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #105;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #106;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #107;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #108;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #109;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #110;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #111;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #112;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #113;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #114;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #115;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #116;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #117;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #118;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #119;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #120;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #121;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #122;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #123;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #124;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #18;1 #125;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #19;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #20;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #21;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #22;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #23;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #24;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #25;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #26;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #27;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #28;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #16;1 #29;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #19;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #20;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #21;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #22;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #23;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #24;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #25;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #26;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #27;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #28;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #39;1 #29;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #40;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #41;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #42;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #43;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #44;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #45;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #46;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #47;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #48;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #2;1 #49;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #10;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 to class 9: 26
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #4;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #6;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #7;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #10;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #11;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #13;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #14;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #15;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #16;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #17;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #18;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #19;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #20;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 to class 10: 9
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #5;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 to class 11: 16
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #11;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 to class 12: 35
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #26;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #29;1 to class 13: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 to class 13: 40
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #1;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #2;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #3;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #4;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #5;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #6;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #7;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #8;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #9;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #10;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #11;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #12;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #13;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #14;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #15;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #16;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #17;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #18;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #19;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #20;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #21;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #22;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #23;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #24;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #25;1 to class 14: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 to class 14: 33
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #1;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #2;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #3;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #4;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #5;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #6;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #7;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #8;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #9;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #10;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #11;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #12;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #13;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #14;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #15;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #16;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #17;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #18;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #19;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #20;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #21;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #22;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #23;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #24;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #25;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #26;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #27;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #28;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #29;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #30;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #31;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #32;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #26;1 #33;1 to class 15: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #27;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #28;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #29;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #30;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #31;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #32;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #33;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #34;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #35;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #36;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #37;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #38;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #39;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #30;1 #40;1 to class 14: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #31;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #32;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #33;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #34;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #12;1 #35;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #6;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #21;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #22;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #23;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #24;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #25;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #11;1 #26;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #16;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #17;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #18;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #19;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #5;1 #20;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #6;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #7;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #8;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #9;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #10;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #11;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #3;1 #12;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #4;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #5;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #6;1 to class 7: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 to class 7: 17
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #1;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 to class 8: 39
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #1;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #10;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #16;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #17;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #18;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #19;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #20;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #21;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #22;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #23;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #24;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #25;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #26;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 to class 9: 8
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #4;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #6;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 to class 10: 15
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #10;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 to class 11: 37
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #1;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #17;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #18;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #19;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #20;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #21;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #22;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #23;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #24;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #25;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #26;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #27;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 to class 12: 25
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #28;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #29;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #30;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #31;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #32;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #33;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #34;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #35;1 to class 12: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 to class 12: 25
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #36;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #11;1 #37;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #7;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #27;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #28;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #29;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #30;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #31;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #32;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #33;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #34;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #35;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #36;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #37;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #38;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #2;1 #39;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #3;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #4;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 to class 8: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #1;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #10;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #16;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #17;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #18;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #19;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #20;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #21;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #22;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #23;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #24;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #25;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #26;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #27;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #28;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #29;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #30;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #31;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #32;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #33;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #34;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #35;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #36;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 to class 9: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #4;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #6;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 to class 10: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #8;1 to class 11: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 to class 11: 125
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 to class 12: 125
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #1;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #2;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #3;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #4;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #5;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #6;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #7;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #8;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #9;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #10;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #11;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #12;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #13;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #14;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #15;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #16;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #17;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #18;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #19;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #20;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #21;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #22;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #23;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #24;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #25;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #26;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #27;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #28;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #29;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #30;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #31;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #32;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #33;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #34;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #35;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #36;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #37;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #38;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #39;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #40;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #41;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #42;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #43;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #44;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #45;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #46;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #47;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #48;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #49;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #50;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #51;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #52;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #53;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #54;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #55;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #56;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #57;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #58;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #59;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #60;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #61;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #62;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #63;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #64;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #65;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #66;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #67;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #68;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #69;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #70;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #71;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #72;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #73;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #74;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #75;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #76;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #77;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #78;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #79;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #80;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #81;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #82;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #83;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #84;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #85;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #86;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #87;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #88;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #89;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #90;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #91;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #92;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #93;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #94;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #95;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #96;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #97;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #98;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #99;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #100;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #101;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #102;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #103;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #104;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #105;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #106;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #107;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #108;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #109;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #110;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #111;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #112;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #113;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #114;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #115;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #116;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #117;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #118;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #119;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #120;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #121;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #122;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #123;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #124;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #1;1 #125;1 to class 13: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #2;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #3;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #4;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #5;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #6;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #7;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #8;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #9;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #10;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #11;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #12;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #13;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #14;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #15;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #16;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #17;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #18;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #19;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #20;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #21;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #22;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #23;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #24;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #25;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #26;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #27;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #28;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #29;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #30;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #31;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #32;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #33;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #34;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #35;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #36;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #37;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #38;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #39;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #40;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #41;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #42;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #43;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #44;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #45;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #46;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #47;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #48;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #49;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #50;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #51;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #52;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #53;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #54;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #55;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #56;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #57;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #58;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #59;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #60;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #61;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #62;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #63;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #64;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #65;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #66;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #67;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #68;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #69;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #70;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #71;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #72;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #73;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #74;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #75;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #76;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #77;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #78;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #79;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #80;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #81;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #82;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #83;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #84;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #85;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #86;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #87;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #88;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #89;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #90;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #91;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #92;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #93;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #94;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #95;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #96;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #97;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #98;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #99;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #100;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #101;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #102;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #103;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #104;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #105;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #106;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #107;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #108;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #109;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #110;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #111;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #112;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #113;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #114;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #115;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #116;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #117;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #118;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #119;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #120;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #121;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #122;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #123;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #124;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #9;1 #125;1 to class 12: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #16;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #19;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #20;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #21;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #22;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #23;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #24;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #25;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #26;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #27;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #28;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #7;1 #29;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #10;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #11;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #13;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #14;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #15;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #16;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #17;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #18;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #19;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #20;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #21;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #22;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #23;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #24;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #25;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #26;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #27;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #28;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #37;1 #29;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #38;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #39;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #40;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #41;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #42;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #43;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #44;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #45;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #46;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #47;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #48;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #5;1 #49;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #6;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #7;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #8;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #9;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #10;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #11;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #12;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #13;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #14;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #15;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #16;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #7;1 #17;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #8;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #9;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #10;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #11;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #12;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #13;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #2;1 #14;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #3;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #4;1 to class 6: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 to class 6: 18
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #1;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #2;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #3;1 to class 7: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 to class 7: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #1;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #2;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #3;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #4;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #5;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #6;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #7;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #8;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #9;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #10;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #11;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #12;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #13;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #14;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #15;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #16;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #17;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #18;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #19;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #20;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #21;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #22;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #23;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #24;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #25;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #26;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #27;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #28;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #29;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #30;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #31;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #32;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #33;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #34;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #35;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #36;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 to class 8: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #1;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #10;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #16;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #17;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #18;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #19;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 to class 9: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #4;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #6;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #7;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #10;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #11;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #13;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #14;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #15;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #16;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 to class 10: 125
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #16;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #19;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #20;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #21;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #22;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #23;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #24;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #25;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #26;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #27;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #28;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #29;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #30;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #31;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #32;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #33;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #34;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #35;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #36;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #37;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #38;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #39;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #40;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #41;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #42;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #43;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #44;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #45;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #46;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #47;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #48;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #49;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #50;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #51;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #52;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #53;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #54;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #55;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #56;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #57;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #58;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #59;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #60;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #61;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #62;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #63;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #64;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #65;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #66;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #67;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #68;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #69;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #70;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #71;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #72;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #73;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #74;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #75;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #76;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #77;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #78;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #79;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #80;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #81;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #82;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #83;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #84;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #85;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #86;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #87;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #88;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #89;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #90;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #91;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #92;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #93;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #94;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #95;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #96;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #97;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #98;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #99;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #100;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #101;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #102;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #103;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #104;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #105;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #106;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #107;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #108;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #109;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #110;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #111;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #112;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #113;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #114;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #115;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #116;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #117;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #118;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #119;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #120;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #121;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #122;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #123;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #124;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #17;1 #125;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #18;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #19;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #20;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #21;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #22;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #23;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #24;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #25;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #26;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #27;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #28;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #20;1 #29;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #21;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #22;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #23;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #24;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #25;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #26;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #27;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #28;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #37;1 #29;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #38;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #39;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #40;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #41;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #42;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #43;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #44;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #45;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #46;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #47;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #48;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #4;1 #49;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #5;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #6;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #7;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #8;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #9;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #10;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #11;1 to class 7: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 to class 7: 26
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #1;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #2;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #3;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #4;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #5;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #6;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #7;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #8;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #9;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #10;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #11;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #12;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #13;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #14;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #15;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #16;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #17;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #18;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #19;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #20;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 to class 8: 11
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #1;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #9;1 to class 9: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 to class 9: 16
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #1;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #2;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #3;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #4;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #5;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #6;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #7;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #8;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #9;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #10;1 to class 10: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 to class 10: 33
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #1;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #2;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #3;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #4;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #5;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #6;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #7;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #8;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #9;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #10;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #11;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #12;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #13;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #14;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #15;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #16;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #17;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #18;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #19;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #20;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #21;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #22;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #23;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #24;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #25;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #26;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #27;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #28;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #29;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #30;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #31;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #32;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #11;1 #33;1 to class 11: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #12;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #13;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #14;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #15;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #10;1 #16;1 to class 10: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #21;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #22;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #23;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #24;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #25;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #12;1 #26;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #13;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #14;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #15;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #16;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #17;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #5;1 #18;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #6;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #7;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #8;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #9;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #10;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #11;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #5;1 #12;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #6;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #7;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #8;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #9;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #10;1 to class 5: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 to class 5: 17
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #1;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #2;1 to class 6: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 to class 6: 49
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #1;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #2;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #3;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #4;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #5;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #6;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #7;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #8;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #9;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #10;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #11;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #12;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #13;1 to class 7: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 to class 7: 29
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #1;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #2;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #3;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #4;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #5;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #6;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #7;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #8;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #9;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #10;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #11;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #12;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #13;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #14;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #15;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #16;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #17;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #18;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #19;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 to class 8: 25
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #1;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #10;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #16;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #17;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #18;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #19;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #20;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #21;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #22;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #23;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #24;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #20;1 #25;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #21;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #22;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #23;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #24;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #25;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #26;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #27;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #28;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #14;1 #29;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #15;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #16;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #17;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #18;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #19;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #20;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #21;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #22;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #23;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #24;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #25;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #26;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #27;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #28;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #29;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #30;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #31;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #32;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #33;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #34;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #35;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #36;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #37;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #38;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #39;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #40;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #41;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #42;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #43;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #44;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #45;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #46;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #47;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #48;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #3;1 #49;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #4;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #5;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #6;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #7;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #8;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #9;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #10;1 to class 6: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 to class 6: 37
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #1;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #2;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #3;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #4;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #5;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #6;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #7;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #8;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #9;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #10;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #11;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #12;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #13;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #14;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #15;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #16;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #17;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #18;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #19;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #20;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #21;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #22;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #23;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #24;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #25;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #26;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #27;1 to class 7: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 to class 7: 8
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #1;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #2;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #3;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #4;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #5;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #6;1 to class 8: 0
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 to class 8: 15
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #1;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #2;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #3;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #4;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #5;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #6;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #7;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #8;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #9;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #10;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #11;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #12;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #13;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #14;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #7;1 #15;1 to class 9: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #28;1 #8;1 to class 8: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #29;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #30;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #31;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #32;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #33;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #34;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #35;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #36;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #11;1 #37;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #12;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #13;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #14;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #15;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #16;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #11;1 #17;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #12;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #13;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #14;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #15;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #16;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #17;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #18;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #19;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #20;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #1;1 #21;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #2;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #3;1 to class 4: 0
#I  Number of descendants of group #2;1 #1;1 #4;1 to class 4: 18
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #1;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #2;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #3;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #4;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #5;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #6;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #7;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #8;1 to class 5: 0
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 to class 5: 70
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #1;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #2;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #3;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #4;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #5;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #6;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #7;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #8;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #9;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #10;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #11;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #12;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #13;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #14;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #15;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #16;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #17;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #18;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #19;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #20;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #21;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #22;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #23;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #24;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #25;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #26;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #27;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #28;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #29;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #30;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #31;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #32;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #33;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #34;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #35;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #36;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #37;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #38;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #39;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #40;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #41;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #42;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #43;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #44;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #45;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #46;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #47;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #48;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #49;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #50;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #51;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #52;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #53;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #54;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #55;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #56;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #57;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #58;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #59;1 to class 6: 0
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 to class 6: 33
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #1;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #2;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #3;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #4;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #5;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #6;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #7;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #8;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #9;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #10;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #11;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #12;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #13;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #14;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #15;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #16;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #17;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #18;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #19;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #20;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #21;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #22;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #23;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #24;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #25;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #26;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #27;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #28;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #29;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #30;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #31;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #32;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #60;1 #33;1 to class 7: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #61;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #62;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #63;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #64;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #65;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #66;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #67;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #68;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #69;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #9;1 #70;1 to class 6: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #10;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #11;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #12;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #13;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #14;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #15;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #16;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #17;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #4;1 #18;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #5;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #6;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #7;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #8;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 #9;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #2;1 to class 3: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 to class 3: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #4;1 to class 3: 0
#I  group restored from file is incapable
#I  Number of descendants of group #3;1 to class 2: 0
gap> PqQuitAll();
gap> ##Example: "PqDescendantsTreeCoclassOne-9-i" . based on manual example
gap> ##(demonstrates how one may use `PqDescendantsTreeCoclassOne' to
gap> ## develop a descendants tree, by generating the 3-groups of rank 2
gap> ## and 3-coclass 1 up to 3-class 5. Most effective in XGAP.)
gap> G := ElementaryAbelianGroup( 9 );
<pc group of size 9 with 2 generators>
gap> procId := PqStart( G );
1
gap> PqDescendantsTreeCoclassOne( procId : TreeDepth := 5 );
#I  Number of descendants of group #1;1 to class 2: 2
#I  Number of descendants of group #1;1 #1;1 to class 3: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 to class 4: 2
#I  Number of descendants of group #1;1 #1;1 #1;1 #1;1 to class 5: 2
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #1;1 #2;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #1;1 #2;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #1;1 #2;1 to class 3: 0
#I  Number of descendants of group #2;1 to class 2: 4
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #1;1 to class 3: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #2;1 to class 3: 0
#I  Number of descendants of group #2;1 #3;1 to class 3: 6
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #1;1 to class 4: 0
#I  Number of descendants of group #2;1 #3;1 #2;1 to class 4: 7
#I  Number of descendants of group #2;1 #3;1 #2;1 #1;1 to class 5: 6
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #2;1 #2;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #2;1 #3;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #2;1 #4;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #2;1 #5;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #2;1 #6;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #2;1 #7;1 to class 5: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #3;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #4;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #5;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #3;1 #6;1 to class 4: 0
#I  group restored from file is incapable
#I  Number of descendants of group #2;1 #4;1 to class 3: 0
#I  group restored from file is incapable
#I  Number of descendants of group #3;1 to class 2: 0
gap> ##Example: "PqEpimorphism" . . . based on `PqEpimorphism' manual example
gap> F := FreeGroup (2, "F");
<free group on the generators [ F1, F2 ]>
gap> phi := PqEpimorphism( F : Prime := 5, ClassBound := 2 );
[ F1, F2 ] -> [ f1, f2 ]
gap> Image( phi );
<pc group of size 3125 with 5 generators>
gap> # Executing interactive variant of example: "PqEpimorphism"
gap> ##Example: "PqEpimorphism" . . . based on `PqEpimorphism' manual example
gap> F := FreeGroup (2, "F");
<free group on the generators [ F1, F2 ]>
gap> procId := PqStart( F );
2
gap> phi := PqEpimorphism( procId : Prime := 5, ClassBound := 2 );
[ F1, F2 ] -> [ f1, f2 ]
gap> Image( phi );
<pc group of size 3125 with 5 generators>
gap> ##Example: "PqPCover" . . . based on `PqPCover' manual example
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> PqPCover( F : Prime := 2, ClassBound := 3 );
<pc group of size 262144 with 18 generators>
gap> 
gap> # Now let's get a p-cover of a p-quotient of an fp group
gap> G := F / [a^4, b^4];
<fp group on the generators [ a, b ]>
gap> PqPCover( G : Prime := 2, ClassBound := 3 );
<pc group of size 16384 with 14 generators>
gap> 
gap> # Now let's get a p-cover of a different p-quotient of the same group
gap> PqPCover( G : Prime := 2, ClassBound := 3, 
>               Exponent := 4 );
<pc group of size 8192 with 13 generators>
gap> 
gap> # Now we'll get a p-cover of a p-quotient of another fp group
gap> # which we will redo using the `Relators' option
gap> R := [ a^25, Comm(Comm(b, a), a), b^5 ];
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> PqPCover( H : Prime := 5, ClassBound := 5, Metabelian );
<pc group of size 48828125 with 11 generators>
gap> 
gap> # Now we redo the previous example using the `Relators' option
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> PqPCover( F : Prime := 5, ClassBound := 5, Metabelian, 
>               Relators := rels );
<pc group of size 48828125 with 11 generators>
gap> # Executing interactive variant of example: "PqPCover"
gap> ##Example: "PqPCover" . . . based on `PqPCover' manual example
gap> F := FreeGroup("a", "b"); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> procId1 := PqStart( F );
3
gap> PqPCover( procId1 : Prime := 2, ClassBound := 3 );
<pc group of size 262144 with 18 generators>
gap> 
gap> # Now let's get a p-cover of a p-quotient of an fp group
gap> G := F / [a^4, b^4];
<fp group on the generators [ a, b ]>
gap> procId2 := PqStart( G );
4
gap> PqPCover( procId2 : Prime := 2, ClassBound := 3 );
<pc group of size 16384 with 14 generators>
gap> 
gap> # Now let's get a p-cover of a different p-quotient of the same group
gap> PqPCover( procId2 : Prime := 2, ClassBound := 3, 
>               RedoPcp,
>               Exponent := 4 );
<pc group of size 8192 with 13 generators>
gap> 
gap> # Now we'll get a p-cover of a p-quotient of another fp group
gap> # which we will redo using the `Relators' option
gap> R := [ a^25, Comm(Comm(b, a), a), b^5 ];
[ a^25, a^-1*b^-1*a*b*a^-1*b^-1*a^-1*b*a^2, b^5 ]
gap> H := F / R;
<fp group on the generators [ a, b ]>
gap> procId3 := PqStart( H );
5
gap> PqPCover( procId3 : Prime := 5, ClassBound := 5, Metabelian );
<pc group of size 48828125 with 11 generators>
gap> 
gap> # Now we redo the previous example using the `Relators' option
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := [ "a^25", "[b, a, a]", "b^5" ];
[ "a^25", "[b, a, a]", "b^5" ]
gap> procId4 := PqStart( F );
6
gap> PqPCover( procId4 : Prime := 5, ClassBound := 5, Metabelian, 
>               Relators := rels );
<pc group of size 48828125 with 11 generators>
gap> PqQuitAll();
gap> ##Example: "PqSupplementInnerAutomorphisms" . . . based on manual example
gap> ##Illustrates `PqSupplementInnerAutomorphisms' usage
gap> Q := Pq( FreeGroup(2) : Prime := 3, ClassBound := 1 );
<pc group of size 9 with 2 generators>
gap> des := PqDescendants( Q : StepSize := 1 );
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> S := PqSupplementInnerAutomorphisms( des[3] );
rec( agAutos := [  ], agOrder := [ 3, 2, 2, 2 ], 
  glAutos := [ Pcgs([ f1, f2, f3 ]) -> [ f1*f2^2, f2, f3 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1^2, f2, f3^2 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1^2, f2, f3^2 ] ] )
gap> A := AutomorphismGroupPGroup( des[3] );
rec( 
  agAutos := [ Pcgs([ f1, f2, f3 ]) -> [ f1^2, f2, f3^2 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1*f2^2, f2, f3 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1*f3, f2, f3 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1, f2*f3, f3 ] ], agOrder := [ 2, 3, 3, 3 ], 
  glAutos := [  ], glOper := [  ], glOrder := 1, 
  group := <pc group of size 27 with 3 generators>, 
  one := IdentityMapping( <pc group of size 27 with 3 generators> ), 
  size := 54 )
gap> # Executing interactive variant of example: "PqSupplementInnerAutomorphisms"
gap> ##Example: "PqSupplementInnerAutomorphisms" . . . based on manual example
gap> ##Illustrates `PqSupplementInnerAutomorphisms' usage
gap> procId := PqStart( FreeGroup(2) );
1
gap> Q := Pq( procId : Prime := 3, ClassBound := 1 );
<pc group of size 9 with 2 generators>
gap> PqSetPQuotientToGroup( procId );
gap> des := PqDescendants( procId : StepSize := 1 );
[ <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators>, 
  <pc group of size 27 with 3 generators> ]
gap> S := PqSupplementInnerAutomorphisms( des[3] );
rec( agAutos := [  ], agOrder := [ 3, 2, 2, 2 ], 
  glAutos := [ Pcgs([ f1, f2, f3 ]) -> [ f1*f2^2, f2, f3 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1^2, f2, f3^2 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1^2, f2, f3^2 ] ] )
gap> A := AutomorphismGroupPGroup( des[3] );
rec( 
  agAutos := [ Pcgs([ f1, f2, f3 ]) -> [ f1^2, f2, f3^2 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1*f2^2, f2, f3 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1*f3, f2, f3 ], 
      Pcgs([ f1, f2, f3 ]) -> [ f1, f2*f3, f3 ] ], agOrder := [ 2, 3, 3, 3 ], 
  glAutos := [  ], glOper := [  ], glOrder := 1, 
  group := <pc group of size 27 with 3 generators>, 
  one := IdentityMapping( <pc group of size 27 with 3 generators> ), 
  size := 54 )
gap> ##Example: "R2-5-i" . . . based on: examples/R2-5
gap> ##Construction of R(2,5)
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 5, Exponent := 5);
2
gap> PqPcPresentation(procId : ClassBound := 1, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
gap> PqSupplyAutomorphisms(procId, [ [[2,0],
>                                  [0,1]],
> 
>                                 [[4,1],
>                                  [4,0]] ]);
gap> for class in [2 .. 13] do
>   PqNextClass(procId : QueueFactor := 20);
> od;
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^14
#I  Group: [grp] to lower exponent-5 central class 7 has order 5^18
#I  Group: [grp] to lower exponent-5 central class 8 has order 5^22
#I  Group: [grp] to lower exponent-5 central class 9 has order 5^28
#I  Group: [grp] to lower exponent-5 central class 10 has order 5^31
#I  Group: [grp] to lower exponent-5 central class 11 has order 5^33
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^34
#I  Group completed. Lower exponent-5 central class = 12, Order = 5^34
#I  Group: [grp] to lower exponent-5 central class 12 has order 5^34
gap> PqSavePcPresentation(procId, ANUPQData.outfile);
gap> ##Example: "R2-5-x-i" . . . based on: GrpFP_2_pQuotient7
gap> ##Construction of R(2,5) extended to see how many
gap> ##words starting with the first pc generator squared
gap> ##need to be considered when doing the exponent law checks
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> procId := PqStart(F : Prime := 5, Exponent := 5);
3
gap> PqPcPresentation(procId : ClassBound := 6, 
>                           OutputLevel := 1);
#I  Lower exponent-5 central series for [grp]
#I  Group: [grp] to lower exponent-5 central class 1 has order 5^2
#I  Group: [grp] to lower exponent-5 central class 2 has order 5^3
#I  Group: [grp] to lower exponent-5 central class 3 has order 5^5
#I  Group: [grp] to lower exponent-5 central class 4 has order 5^8
#I  Group: [grp] to lower exponent-5 central class 5 has order 5^10
#I  Group: [grp] to lower exponent-5 central class 6 has order 5^14
gap> PqSetupTablesForNextClass(procId);
#I  Setup performed for class 7
gap> PqTails(procId, 0);
gap> PqDoConsistencyChecks(procId, 0, 0);
gap> PqSetOutputLevel(procId, 3);
gap> PqDoExponentChecks(procId);
#I  The quotient relation is  .28
#I  Generator 28 is trivial
#I  Collected power 5 of the following word: 1^1 
#I  The quotient relation is  .29
#I  Generator 29 is trivial
#I  Collected power 5 of the following word: 2^1 
#I  2 relations of class 1 collected
#I  The quotient relation is  .15 .16 .17^2 .18^3 .19^4 .20^3 .21^3 .22 .23 .24^4 .26^4 .27^2 .28 .29
#I  Generator 26 is redundant
#I  Collected power 5 of the following word: 1^1 2^1 
#I  1 relation of class 2 collected
#I  The quotient relation is  .15^3 .16^2 .17 .18^3 .22^4 .23^2 .24 .25^3 .26^4 .27^3 .28 .29^2
#I  Generator 25 is redundant
#I  Collected power 5 of the following word: 1^1 2^2 
#I  1 relation of class 3 collected
#I  The quotient relation is  .15 .17 .18^4 .22^4 .23^3 .24 .25 .26^4 .27^3 .28 .29^3
#I  Generator 24 is redundant
#I  Collected power 5 of the following word: 1^1 2^3 
#I  The quotient relation is  .15^2 .28
#I  Generator 15 is trivial
#I  Collected power 5 of the following word: 1^1 5^1 
#I  The quotient relation is  .22 .29
#I  Generator 22 is trivial
#I  Collected power 5 of the following word: 2^1 4^1 
#I  3 relations of class 4 collected
#I  The quotient relation is  .16^2 .17 .19 .20^3 .21^3 .23^4 .24^4 .25 .26^4 .27^2 .28 .29^4
#I  Generator 23 is redundant
#I  Collected power 5 of the following word: 1^1 2^4 
#I  The quotient relation is  .15^4 .16^4 .18 .20^4 .21^4 .22^2 .23 .24^4 .26^4 .27^2 .28 .29
#I  Generator 21 is redundant
#I  Collected power 5 of the following word: 1^1 2^1 4^1 
#I  2 relations of class 5 collected
#I  The quotient relation is  .15^4 .16^4 .17^3 .18^2 .19^4 .20^3 .21^3 .23^2 .24 .25^3 .26^4 .27^3 .28 .29^2
#I  Generator 19 is redundant
#I  Collected power 5 of the following word: 1^1 2^2 4^1 
#I  1 relation of class 6 collected
#I  The quotient relation is  .16^2 .17^3 .19^4 .20^2 .21^2 .23^3 .24 .25 .26^4 .27^3 .28 .29^3
#I  Collected power 5 of the following word: 1^1 2^3 4^1 
#I  1 relation of class 7 collected
gap> PqQuitAll();
gap> ##Example: "StandardPresentation" . . . based on manual example
gap> ##(demonstrates `StandardPresentation' usage)
gap> F := FreeGroup( "a", "b" ); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := F / [a^25, Comm(Comm(b, a), a), b^5];
<fp group on the generators [ a, b ]>
gap> S := StandardPresentation( G : Prime := 5, ClassBound := 10 );
<fp group on the generators [ f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, 
  f12, f13, f14, f15, f16, f17, f18, f19, f20, f21, f22, f23, f24, f25, f26 ]>
gap> IsPcGroup( S );
false
gap> # if we need to compute with S we should convert it to a pc group
gap> Spc := PcGroupFpGroup( S );
<pc group of size 1490116119384765625 with 26 generators>
gap> 
gap> H := F / [ a^625, Comm(Comm(Comm(Comm(b, a), a), a), a)/Comm(b, a)^5,
>            Comm(Comm(b, a), b), b^625 ];
<fp group on the generators [ a, b ]>
gap> StandardPresentation( H : Prime := 5, ClassBound := 15, 
>                           Metabelian );
<fp group on the generators [ f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, 
  f12, f13, f14, f15, f16, f17, f18, f19, f20 ]>
gap> 
gap> F4 := FreeGroup( "a", "b", "c", "d" );
<free group on the generators [ a, b, c, d ]>
gap> a := F4.1; b := F4.2; c := F4.3; d := F4.4;
a
b
c
d
gap> G4 := F4 / [ b^4, b^2 / Comm(Comm (b, a), a), d^16,
>              a^16 / (c * d), b^8 / (d * c^4) ];
<fp group on the generators [ a, b, c, d ]>
gap> K := Pq( G4 : Prime := 2, ClassBound := 1 );
<pc group of size 4 with 2 generators>
gap> StandardPresentation( G4 : pQuotient := K, ClassBound := 14 );
<fp group with 53 generators>
gap> # Executing interactive variant of example: "StandardPresentation"
gap> ##Example: "StandardPresentation" . . . based on manual example
gap> ##(demonstrates `StandardPresentation' usage)
gap> F := FreeGroup( "a", "b" ); a := F.1; b := F.2;
<free group on the generators [ a, b ]>
a
b
gap> G := F / [a^25, Comm(Comm(b, a), a), b^5];
<fp group on the generators [ a, b ]>
gap> procId1 := PqStart( G );
1
gap> S := StandardPresentation( procId1 : Prime := 5, ClassBound := 10 );
<fp group on the generators [ f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, 
  f12, f13, f14, f15, f16, f17, f18, f19, f20, f21, f22, f23, f24, f25, f26 ]>
gap> IsPcGroup( S );
false
gap> # if we need to compute with S we should convert it to a pc group
gap> Spc := PcGroupFpGroup( S );
<pc group of size 1490116119384765625 with 26 generators>
gap> 
gap> H := F / [ a^625, Comm(Comm(Comm(Comm(b, a), a), a), a)/Comm(b, a)^5,
>            Comm(Comm(b, a), b), b^625 ];
<fp group on the generators [ a, b ]>
gap> procId2 := PqStart( H );
2
gap> StandardPresentation( procId2 : Prime := 5, ClassBound := 15, 
>                           Metabelian );
<fp group on the generators [ f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, 
  f12, f13, f14, f15, f16, f17, f18, f19, f20 ]>
gap> 
gap> F4 := FreeGroup( "a", "b", "c", "d" );
<free group on the generators [ a, b, c, d ]>
gap> a := F4.1; b := F4.2; c := F4.3; d := F4.4;
a
b
c
d
gap> G4 := F4 / [ b^4, b^2 / Comm(Comm (b, a), a), d^16,
>              a^16 / (c * d), b^8 / (d * c^4) ];
<fp group on the generators [ a, b, c, d ]>
gap> procId3 := PqStart( G4 );
3
gap> K := Pq( procId3 : Prime := 2, ClassBound := 1 );
<pc group of size 4 with 2 generators>
gap> StandardPresentation( procId3 : pQuotient := K, ClassBound := 14 );
<fp group with 53 generators>
gap> ##Example: "StandardPresentation-i" . . . based on manual example
gap> ##(demonstrates `StandardPresentation' interactive usage)
gap> F4 := FreeGroup( "a", "b", "c", "d" );
<free group on the generators [ a, b, c, d ]>
gap> a := F4.1; b := F4.2; c := F4.3; d := F4.4;
a
b
c
d
gap> G4 := F4 / [ b^4, b^2 / Comm(Comm (b, a), a), d^16,
>              a^16 / (c * d), b^8 / (d * c^4) ];
<fp group on the generators [ a, b, c, d ]>
gap> procId := PqStart( G4 );
4
gap> Pq( procId : Prime := 2, ClassBound := 1 );
<pc group of size 4 with 2 generators>
gap> # the computed p-quotient is ``remembered'' and supplies the
gap> # `pQuotient' option to `StandardPresentation' ...
gap> StandardPresentation( procId : ClassBound := 14 );
<fp group with 53 generators>
gap> # `StandardPresentation' and `EpimorphismStandardPresentation'
gap> # behave like attributes, so no computation is done when
gap> # either is called again for the same process ...
gap> lev := InfoLevel(InfoANUPQ);
1
gap> SetInfoLevel(InfoANUPQ, 2); # To see computation time
gap> EpimorphismStandardPresentation( procId : ClassBound := 14 );
[ a, b, c, d ] -> [ f1, f2, f13*f20*f28, f20*f24*f33 ]
gap> # No timing data was Info-ed since no computation was done
gap> SetInfoLevel(InfoANUPQ, lev); # Restore previous InfoANUPQ level
gap> 
gap> ##Example: "gp-256-SP-Rel-i" . . . based on: isom/gp_256
gap> F := FreeGroup("a", "b");
<free group on the generators [ a, b ]>
gap> rels := ["a^4 * [b, a, a, a, a, a]^-1", 
>          "b * a^2 * b^-1 * a^-2 * [b, a, a, a, a, a]^-1", 
>          "b^2 * [b, a, a, a, a, a]^-1" ];
[ "a^4 * [b, a, a, a, a, a]^-1", 
  "b * a^2 * b^-1 * a^-2 * [b, a, a, a, a, a]^-1", 
  "b^2 * [b, a, a, a, a, a]^-1" ]
gap> procId := PqStart(F : Prime := 2, Relators := rels);
5
gap> PqSPComputePcpAndPCover(procId : ClassBound := 1);
gap> PqSPStandardPresentation(procId, [ [[0,1],
>                                     [1,1]],
> 
>                                    [[0,1],
>                                     [1,0]] ]
> 
>                          : # options
>                            ClassBound := 8,
>                            PcgsAutomorphisms);
gap> PqQuitAll();
gap> STOP_TEST( "anupqeg.tst", 1000000 );
